@@ -41,6 +41,13 @@
 
 #include "ikea.h"
 
+#define AUTH_HYBRID_RSA_XAUTH	0
+#define AUTH_MUTUAL_RSA_XAUTH	1
+#define AUTH_MUTUAL_PSK_XAUTH	2
+#define AUTH_MUTUAL_RSA		3
+#define AUTH_MUTUAL_PSK		4
+
+#define IDTXT_NONE	"No Identity"
 #define IDTXT_ASN1	"ASN.1 Distinguished Name"
 #define IDTXT_FQDN	"Fully Qualified Domain Name"
 #define IDTXT_UFQDN	"User Fully Qualified Domain Name"
@@ -64,6 +71,71 @@ bool combobox_setbytext( const char * text, QComboBox * cbox )
 	return false;
 }
 
+bool dhgrp_to_string( long & dhgrp, QString & string )
+{
+	switch( dhgrp )
+	{
+		case -1:
+			string = "disabled";
+			break;
+
+		case 0:
+			string = "auto";
+			break;
+
+		case 1:
+			string = "group 1";
+			break;
+
+		case 2:
+			string = "group 2";
+			break;
+
+		case 5:
+			string = "group 5";
+			break;
+
+		case 14:
+			string = "group 14";
+			break;
+
+		case 15:
+			string = "group 15";
+			break;
+
+		default:
+			return false;
+	}
+
+	return true;
+}
+
+bool string_to_dhgrp( QString & string, long & dhgrp )
+{
+	if( !strcmp( string.ascii(), "disabled" ) )
+		dhgrp = -1;
+
+	if( !strcmp( string.ascii(), "auto" ) )
+		dhgrp = 0;
+
+	if( !strcmp( string.ascii(), "group 1" ) )
+		dhgrp = 1;
+
+	if( !strcmp( string.ascii(), "group 2" ) )
+		dhgrp = 2;
+
+	if( !strcmp( string.ascii(), "group 5" ) )
+		dhgrp = 5;
+
+	if( !strcmp( string.ascii(), "group 14" ) )
+		dhgrp = 14;
+
+	if( !strcmp( string.ascii(), "group 15" ) )
+		dhgrp = 15;
+
+	return true;
+}
+
 void site::AddPolicy()
 {
 	topology t( this );
@@ -102,6 +174,8 @@ void site::AddPolicy()
 
 void site::init()
 {
+	comboBoxConfigMethod->setCurrentItem( 1 );
+
 	lineEditAddress->setInputMask( "00D . 00D . 00D . 00D" );
 	lineEditAddress->setText( "0.0.0.0" );
 
@@ -196,6 +270,7 @@ void site::DelPolicy()
 
 bool site::Load( CONFIG & config )
 {
+	QString string;
 	char text[ MAX_CONFSTRING ];
 	long numb;
 
@@ -214,9 +289,20 @@ bool site::Load( CONFIG & config )
 
 	if( config.get_string( "client-auto-mode",
 		text, MAX_CONFSTRING, 0 ) )
-		if( !strcmp( text, "push" ) )
+	{
+		if( !strcmp( text, "disabled" ) )
+			comboBoxConfigMethod->setCurrentItem( 0 );
+
+		if( !strcmp( text, "pull" ) )
 			comboBoxConfigMethod->setCurrentItem( 1 );
 
+		if( !strcmp( text, "push" ) )
+			comboBoxConfigMethod->setCurrentItem( 2 );
+
+		if( !strcmp( text, "dhcp" ) )
+			comboBoxConfigMethod->setCurrentItem( 3 );
+	}
+
 	// local adapter mode ( default virtual )
 
 	if( config.get_string( "client-iface",
@@ -226,6 +312,12 @@ bool site::Load( CONFIG & config )
 
 	if( !comboBoxAddressMethod->currentItem() )
 	{
+		// virtual adapter mtu
+
+		numb = 0;
+		if( config.get_number( "network-mtu-size", &numb ) )
+			lineEditMTU->setText( QString::number( numb, 10 ) );
+
 		// virtual adapter address
 
 		numb = 0;
@@ -325,10 +417,10 @@ bool site::Load( CONFIG & config )
 	else
 		checkBoxBanner->setChecked( false );
 
-	// dns enabled ( default enabled )
+	// dns used ( default used )
 
 	numb = 1;
-	config.get_number( "client-dns-enable", &numb );
+	config.get_number( "client-dns-used", &numb );
 
 	if( !numb )
 	{
@@ -415,6 +507,9 @@ bool site::Load( CONFIG & config )
 	if( config.get_string( "ident-client-type",
 		text, MAX_CONFSTRING, 0 ) )
 	{
+		if( !strcmp( "none", text ) )
+			combobox_setbytext( IDTXT_NONE, comboBoxLocalIDType );
+
 		if( !strcmp( "asn1dn", text ) )
 			combobox_setbytext( IDTXT_ASN1, comboBoxLocalIDType );
 
@@ -484,36 +579,20 @@ bool site::Load( CONFIG & config )
 		text, MAX_CONFSTRING, 0 ) )
 		lineEditPKeyFile->setText( text );
 
-	if( config.get_string( "auth-mutual-psk",
-		text, MAX_CONFSTRING, 0 ) )
-		lineEditPSK->setText( text );
+	BDATA psk;
+	if( config.get_binary( "auth-mutual-psk", psk ) )
+	{
+		psk.add( "", 1 );
+		lineEditPSK->setText( psk.text() );
+	}
 
 	// phase1 dh group ( default auto )
 
 	numb = 0;
 	config.get_number( "phase1-dhgroup", &numb );
-	switch( numb )
-	{
-		case 1: // group 1
-			comboBoxP1DHGroup->setCurrentItem( 1 );
-			break;
 
-		case 2: // group 2
-			comboBoxP1DHGroup->setCurrentItem( 2 );
-			break;
-
-		case 5: // group 5
-			comboBoxP1DHGroup->setCurrentItem( 3 );
-			break;
-
-		case 14: // group 14
-			comboBoxP1DHGroup->setCurrentItem( 4 );
-			break;
-
-		case 15: // group 15
-			comboBoxP1DHGroup->setCurrentItem( 5 );
-			break;
-	}
+	if( dhgrp_to_string( numb, string ) )
+		combobox_setbytext( string.ascii(), comboBoxP1DHGroup );
 
 	// phase1 cipher algorithm ( default auto )
 
@@ -570,36 +649,8 @@ bool site::Load( CONFIG & config )
 	numb = -1;
 	config.get_number( "phase2-pfsgroup", &numb );
 
-	switch( numb )
-	{
-		case -1: // disabled
-			comboBoxP2PFSGroup->setCurrentItem( 0 );
-			break;
-
-		case 0: // automatic
-			comboBoxP2PFSGroup->setCurrentItem( 1 );
-			break;
-
-		case 1:	// group 1
-			comboBoxP2PFSGroup->setCurrentItem( 2 );
-			break;
-
-		case 2:	// group 2
-			comboBoxP2PFSGroup->setCurrentItem( 3 );
-			break;
-
-		case 5:	// group 5
-			comboBoxP2PFSGroup->setCurrentItem( 4 );
-			break;
-
-		case 14: // group 14
-			comboBoxP2PFSGroup->setCurrentItem( 5 );
-			break;
-
-		case 15: // group 15
-			comboBoxP2PFSGroup->setCurrentItem( 6 );
-			break;
-	}
+	if( dhgrp_to_string( numb, string ) )
+		combobox_setbytext( string.ascii(), comboBoxP2PFSGroup );
 
 	// ipcomp transform algorithm
 
@@ -617,6 +668,16 @@ bool site::Load( CONFIG & config )
 	if( config.get_number( "phase2-life-kbytes", &numb ) )
 		lineEditP2LifeData->setText( QString::number( numb, 10 ) );
 
+	// policy nailed sa option ( defailt off )
+
+	numb = 0;
+	config.get_number( "policy-nailed", &numb );
+	if( numb )
+		checkBoxPolicyNailed->setChecked( true );
+	else
+		checkBoxPolicyNailed->setChecked( false );
+		
+
 	// policy configuration ( default auto )
 
 	numb = 1;
@@ -676,9 +737,28 @@ bool site::Save( CONFIG & config )
 
 	// remote config method
 
-	config.set_string( "client-auto-mode",
-		( char * ) comboBoxConfigMethod->currentText().ascii(),
-		comboBoxConfigMethod->currentText().length() );
+	switch( comboBoxConfigMethod->currentItem() )
+	{
+		case 0:	// disabled
+			config.set_string( "client-auto-mode",
+				"disabled", strlen( "disabled" ) );
+			break;
+
+		case 1: // ike config pull
+			config.set_string( "client-auto-mode",
+				"pull", strlen( "pull" ) );
+			break;
+
+		case 2:	// ike config push
+			config.set_string( "client-auto-mode",
+				"push", strlen( "push" ) );
+			break;
+
+		case 3:	// dhcp over ipsec
+			config.set_string( "client-auto-mode",
+				"dhcp", strlen( "dhcp" ) );
+			break;
+	}
 
 	// local adapter mode
 
@@ -698,6 +778,11 @@ bool site::Save( CONFIG & config )
 			"virtual",
 			strlen( "virtual" ) );
 
+		// adapter mtu
+
+		config.set_number( "network-mtu-size",
+			lineEditMTU->text().toLong() );
+
 		// adapter address
 
 		if( checkBoxAddressAuto->isChecked() )
@@ -800,13 +885,13 @@ bool site::Save( CONFIG & config )
 	{
 		// disabled
 
-		config.set_number( "client-dns-enable", 0 );
+		config.set_number( "client-dns-used", 0 );
 	}
 	else
 	{
 		// enabled
 
-		config.set_number( "client-dns-enable", 1 );
+		config.set_number( "client-dns-used", 1 );
 
 		// dns settings
 
@@ -843,27 +928,27 @@ bool site::Save( CONFIG & config )
 
 	switch( comboBoxAuthMethod->currentItem() )
 	{
-		case 0:	// hybrid rsa xauth
+		case AUTH_HYBRID_RSA_XAUTH:
 			config.set_string( "auth-method",
 				"hybrid-rsa-xauth", strlen( "hybrid-rsa-xauth" ) );
 			break;
 
-		case 1:	// mutual rsa xauth
+		case AUTH_MUTUAL_RSA_XAUTH:
 			config.set_string( "auth-method",
 				"mutual-rsa-xauth", strlen( "mutual-rsa-xauth" ) );
 			break;
 
-		case 2:	// mutual psk xauth
+		case AUTH_MUTUAL_PSK_XAUTH:
 			config.set_string( "auth-method",
 				"mutual-psk-xauth", strlen( "mutual-psk-xauth" ) );
 			break;
 
-		case 3:	// mutual rsa
+		case AUTH_MUTUAL_RSA:
 			config.set_string( "auth-method",
 				"mutual-rsa", strlen( "mutual-rsa" ) );
 			break;
 
-		case 4:	// mutual psk
+		case AUTH_MUTUAL_PSK:
 			config.set_string( "auth-method",
 				"mutual-psk", strlen( "mutual-psk" ) );
 			break;
@@ -873,6 +958,10 @@ bool site::Save( CONFIG & config )
 
 	QString locid = comboBoxLocalIDType->currentText();
 
+	if( !locid.compare( IDTXT_NONE ) )
+		config.set_string( "ident-client-type",
+			"none", strlen( "none" ) );
+
 	if( !locid.compare( IDTXT_ASN1 ) )
 		config.set_string( "ident-client-type",
 			"asn1dn", strlen( "asn1dn" ) );
@@ -959,9 +1048,13 @@ bool site::Save( CONFIG & config )
 		config.del( "auth-client-key" );
 
 	if( lineEditPSK->isEnabled() )
-		config.set_string( "auth-mutual-psk",
-			( char * ) lineEditPSK->text().ascii(),
+	{
+		BDATA psk;
+		psk.set(
+			lineEditPSK->text().ascii(),
 			lineEditPSK->text().length() );
+		config.set_binary( "auth-mutual-psk", psk );
+	}
 	else
 		config.del( "auth-mutual-psk" );
 
@@ -986,39 +1079,11 @@ bool site::Save( CONFIG & config )
 
 	// phase1 dh group
 
-	long dhindex = comboBoxP1DHGroup->currentItem();
-	if( comboBoxP1Exchange->currentItem() )
-	{
-		// aggressive mode ( no auto )
-		dhindex++;
-	}
-
-	switch( dhindex )
-	{
-		case 0: // auto
-			config.set_number( "phase1-dhgroup", 0 );
-			break;
-
-		case 1: // group 1
-			config.set_number( "phase1-dhgroup", 1 );
-			break;
-
-		case 2: // group 2
-			config.set_number( "phase1-dhgroup", 2 );
-			break;
-
-		case 3: // group 5
-			config.set_number( "phase1-dhgroup", 5 );
-			break;
-
-		case 4: // group 14
-			config.set_number( "phase1-dhgroup", 14 );
-			break;
-
-		case 5: // group 15
-			config.set_number( "phase1-dhgroup", 15 );
-			break;
-	}
+	QString string;
+	long dhgrp;
+	string = comboBoxP1DHGroup->currentText();
+	string_to_dhgrp( string, dhgrp );
+	config.set_number( "phase1-dhgroup", dhgrp );
 
 	// phase1 cipher algorithm
 
@@ -1066,36 +1131,9 @@ bool site::Save( CONFIG & config )
 
 	// phase2 pfs group
 
-	switch( comboBoxP2PFSGroup->currentItem() )
-	{
-		case 0:	// disabled
-			config.set_number( "phase2-pfsgroup", -1 );
-			break;
-
-		case 1: // automatic
-			config.set_number( "phase2-pfsgroup", 0 );
-			break;
-
-		case 2:	// group 1
-			config.set_number( "phase2-pfsgroup", 1 );
-			break;
-
-		case 3:	// group 2
-			config.set_number( "phase2-pfsgroup", 2 );
-			break;
-
-		case 4:	// group 5
-			config.set_number( "phase2-pfsgroup", 5 );
-			break;
-
-		case 5:	// group 14
-			config.set_number( "phase2-pfsgroup", 14 );
-			break;
-
-		case 6:	// group 15
-			config.set_number( "phase2-pfsgroup", 15 );
-			break;
-	}
+	string = comboBoxP2PFSGroup->currentText();
+	string_to_dhgrp( string, dhgrp );
+	config.set_number( "phase2-pfsgroup", dhgrp );
 
 	// phase2 key life time
 
@@ -1113,7 +1151,12 @@ bool site::Save( CONFIG & config )
 		( char * ) comboBoxP2Compress->currentText().ascii(),
 		comboBoxP2Compress->currentText().length() );
 
-	// policy configuration
+	// policy nailed sa option
+
+	if( !checkBoxPolicyNailed->isChecked() )
+		config.set_number( "policy-nailed", 0 );
+	else
+		config.set_number( "policy-nailed", 1 );
 
 	if( checkBoxPolicyAuto->isChecked() )
 	{
@@ -1174,15 +1217,17 @@ bool site::Verify()
 
 	// check local id data
 
-	if( lineEditLocalIDData->isEnabled() )
-		if( lineEditLocalIDData->text().length() < 1 )
-			errmsg = "Please enter valid local ID data.";
+	if( !comboBoxLocalIDType->currentText().compare( IDTXT_ADDR ) )
+		if( lineEditLocalIDData->isEnabled() )
+			if( lineEditLocalIDData->text().length() < 1 )
+				errmsg = "Please enter valid local ID address data.";
 
 	// check remote id data
 
-	if( lineEditRemoteIDData->isEnabled() )
-		if( lineEditRemoteIDData->text().length() < 1 )
-			errmsg = "Please enter valid remote ID data.";
+	if( !comboBoxRemoteIDType->currentText().compare( IDTXT_ADDR ) )
+		if( lineEditRemoteIDData->isEnabled() )
+			if( lineEditRemoteIDData->text().length() < 1 )
+				errmsg = "Please enter valid remote ID address data.";
 
 	// check cert authority file
 
@@ -1234,13 +1279,53 @@ bool site::Verify()
 
 void site::Update()
 {
+	// auto configuration
+
+	long aconf = comboBoxConfigMethod->currentItem();
+
+	if( aconf == 3 )
+	{
+		// dhcp over ipsec
+
+		comboBoxAddressMethod->setEnabled( true );
+		comboBoxAddressMethod->clear();
+		comboBoxAddressMethod->insertItem( "Use virtual adapter and assigned address" );
+	}
+	else
+	{
+		// other modes
+
+		comboBoxAddressMethod->setEnabled( true );
+		comboBoxAddressMethod->clear();
+		comboBoxAddressMethod->insertItem( "Use virtual adapter and assigned address" );
+		comboBoxAddressMethod->insertItem( "Use existing adapter and current address" );
+	}
+
 	// local adapter mode
 
 	if( !comboBoxAddressMethod->currentItem() )
 	{
 		// virtual
 
-		checkBoxAddressAuto->setEnabled( true );
+		switch( aconf )
+		{
+			case 0:	// autoconf disabled
+				checkBoxAddressAuto->setEnabled( false );
+				checkBoxAddressAuto->setChecked( false );
+				break;
+
+			case 1: // ike config push
+			case 2: // ike config pull
+				checkBoxAddressAuto->setEnabled( true );
+				checkBoxAddressAuto->setChecked( true );
+				break;
+
+			case 3:	// dhcp over ipsec
+				checkBoxAddressAuto->setEnabled( false );
+				checkBoxAddressAuto->setChecked( true );
+				break;
+		}
+
 		textLabelAddress->setEnabled( true );
 		textLabelNetmask->setEnabled( true );
 
@@ -1309,13 +1394,29 @@ void site::Update()
 		lineEditFragSize->setEnabled( true );
 	}
 
+	// client long banner
+
+	if( aconf == 0 )
+	{
+		checkBoxBanner->setEnabled( false );
+		checkBoxBanner->setChecked( false );
+	}
+	else
+		checkBoxBanner->setEnabled( true );
+
 	// dns enabled
 
 	if( checkBoxDNSEnable->isChecked() )
 	{
 		// enabled
 
-		checkBoxDNSAuto->setEnabled( true );
+		if( aconf == 0 )
+		{
+			checkBoxDNSAuto->setEnabled( false );
+			checkBoxDNSAuto->setChecked( false );
+		}
+		else
+			checkBoxDNSAuto->setEnabled( true );
 
 		textLabelDNSServer->setEnabled( true );
 		textLabelDNSSuffix->setEnabled( true );
@@ -1512,29 +1613,48 @@ void site::UpdateTransform()
 
 void site::UpdateAuth()
 {
+	// authentication method
+
+	long auth = comboBoxAuthMethod->currentItem();
+
 	// local identity
 
 	QString locid = comboBoxLocalIDType->currentText();
 
-	switch( comboBoxAuthMethod->currentItem() )
+	switch( auth )
 	{
-		case 0: // hybrid rsa xauth
-		case 2: // mutual psk xauth
-		case 4: // mutual psk
+		case AUTH_HYBRID_RSA_XAUTH:
 		{
+			comboBoxLocalIDType->clear();
+
+			// main or aggressive mode
+
+//			comboBoxLocalIDType->insertItem( IDTXT_NONE );
+			comboBoxLocalIDType->insertItem( IDTXT_FQDN );
+			comboBoxLocalIDType->insertItem( IDTXT_UFQDN );
+			comboBoxLocalIDType->insertItem( IDTXT_ADDR );
+			comboBoxLocalIDType->insertItem( IDTXT_KEYID );
+
+			break;
+		}
+
+		case AUTH_MUTUAL_RSA_XAUTH:
+		case AUTH_MUTUAL_RSA:
+		{
+			comboBoxLocalIDType->clear();
 
 			if( !comboBoxP1Exchange->currentItem() )
 			{
 				// main mode
 
-				comboBoxLocalIDType->clear();
+				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
 				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
 			}
 			else
 			{
 				// aggressive mode
 
-				comboBoxLocalIDType->clear();
+				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
 				comboBoxLocalIDType->insertItem( IDTXT_FQDN );
 				comboBoxLocalIDType->insertItem( IDTXT_UFQDN );
 				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
@@ -1544,23 +1664,21 @@ void site::UpdateAuth()
 			break;
 		}
 
-		case 1: // mutual rsa xauth
-		case 3: // mutual rsa
+		case AUTH_MUTUAL_PSK_XAUTH:
+		case AUTH_MUTUAL_PSK:
 		{
+			comboBoxLocalIDType->clear();
+
 			if( !comboBoxP1Exchange->currentItem() )
 			{
 				// main mode
 
-				comboBoxLocalIDType->clear();
-				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
 				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
 			}
 			else
 			{
 				// aggressive mode
 
-				comboBoxLocalIDType->clear();
-				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
 				comboBoxLocalIDType->insertItem( IDTXT_FQDN );
 				comboBoxLocalIDType->insertItem( IDTXT_UFQDN );
 				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
@@ -1583,17 +1701,18 @@ void site::UpdateAuth()
 
 	QString rmtid = comboBoxRemoteIDType->currentText();
 
-	switch( comboBoxAuthMethod->currentItem() )
+	switch( auth )
 	{
-		case 0: // hybrid rsa xauth
-		case 1: // mutual rsa xauth
-		case 3: // mutual rsa
+		case AUTH_HYBRID_RSA_XAUTH:
+		case AUTH_MUTUAL_RSA_XAUTH:
+		case AUTH_MUTUAL_RSA:
 		{
+			comboBoxRemoteIDType->clear();
+
 			if( !comboBoxP1Exchange->currentItem() )
 			{
 				// main mode
 
-				comboBoxRemoteIDType->clear();
 				comboBoxRemoteIDType->insertItem( IDTXT_ASN1 );
 				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
 			}
@@ -1601,7 +1720,6 @@ void site::UpdateAuth()
 			{
 				// aggressive mode
 
-				comboBoxRemoteIDType->clear();
 				comboBoxRemoteIDType->insertItem( IDTXT_ASN1 );
 				comboBoxRemoteIDType->insertItem( IDTXT_FQDN );
 				comboBoxRemoteIDType->insertItem( IDTXT_UFQDN );
@@ -1612,21 +1730,21 @@ void site::UpdateAuth()
 			break;
 		}
 
-		case 2: // mutual psk xauth
-		case 4: // mutual psk
+		case AUTH_MUTUAL_PSK_XAUTH:
+		case AUTH_MUTUAL_PSK:
 		{
+			comboBoxRemoteIDType->clear();
+
 			if( !comboBoxP1Exchange->currentItem() )
 			{
 				// main mode
 
-				comboBoxRemoteIDType->clear();
 				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
 			}
 			else
 			{
 				// aggressive mode
 
-				comboBoxRemoteIDType->clear();
 				comboBoxRemoteIDType->insertItem( IDTXT_FQDN );
 				comboBoxRemoteIDType->insertItem( IDTXT_UFQDN );
 				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
@@ -1649,7 +1767,7 @@ void site::UpdateAuth()
 
 	switch( comboBoxAuthMethod->currentItem() )
 	{
-		case 0: // hybrid rsa xauth
+		case AUTH_HYBRID_RSA_XAUTH:
 		{
 			lineEditCAFile->setEnabled( true );
 			toolButtonCAFile->setEnabled( true );
@@ -1665,8 +1783,8 @@ void site::UpdateAuth()
 			break;
 		}
 
-		case 1: // mutual rsa xauth
-		case 3: // mutual rsa
+		case AUTH_MUTUAL_RSA_XAUTH:
+		case AUTH_MUTUAL_RSA:
 		{
 			lineEditCAFile->setEnabled( true );
 			toolButtonCAFile->setEnabled( true );
@@ -1682,8 +1800,8 @@ void site::UpdateAuth()
 			break;
 		}
 
-		case 2: // mutual psk xauth
-		case 4: // mutual psk
+		case AUTH_MUTUAL_PSK_XAUTH:
+		case AUTH_MUTUAL_PSK:
 		{
 			lineEditCAFile->setEnabled( false );
 			toolButtonCAFile->setEnabled( false );
@@ -1708,6 +1826,14 @@ void site::UpdateLocalID()
 {
 	QString type = comboBoxLocalIDType->currentText();
 
+	if( !type.compare( IDTXT_NONE ) )
+	{
+		textLabelLocalIDData->setText( "" );
+		lineEditLocalIDData->setEnabled( false );
+		checkBoxLocalIDOption->setHidden( true );
+		return;
+	}
+
 	if( !type.compare( IDTXT_ASN1 ) )
 	{
 		textLabelLocalIDData->setText( "ASN.1 DN String" );
