@@ -44,7 +44,6 @@
 _IKEC::_IKEC()
 {
 	active = false;
-	cancel = false;
 }
 
 _IKEC::~_IKEC()
@@ -115,8 +114,9 @@ void _IKEC::run()
 	// load the config into ipsecc
 	//
 
-	char text[ MAX_CONFSTRING ];
-	long numb;
+	long	numb;
+	char	text[ MAX_CONFSTRING ];
+	BDATA	btext;
 
 	//
 	// ---------- PEER CONFIG ----------
@@ -171,12 +171,18 @@ void _IKEC::run()
 
 	// client auto config mode
 
-	peer.xconf_mode = CONFIG_MODE_PULL;
+	peer.xconf_mode = CONFIG_MODE_NONE;
 
 	if( config.get_string( "client-auto-mode", text, MAX_CONFSTRING, 0 ) )
 	{
 		if( !strcmp( "push", text ) )
 			peer.xconf_mode = CONFIG_MODE_PUSH;
+
+		if( !strcmp( "pull", text ) )
+			peer.xconf_mode = CONFIG_MODE_PULL;
+
+		if( !strcmp( "dhcp", text ) )
+			peer.xconf_mode = CONFIG_MODE_DHCP;
 	}
 
 	// nat-t enable
@@ -193,7 +199,13 @@ void _IKEC::run()
 			peer.natt_mode = IPSEC_NATT_ENABLE;
 
 		if( !strcmp( "force", text ) )
-			peer.natt_mode = IPSEC_NATT_FORCE;
+			peer.natt_mode = IPSEC_NATT_FORCE_RFC;
+
+		if( !strcmp( "force-draft", text ) )
+			peer.natt_mode = IPSEC_NATT_FORCE_DRAFT;
+
+		if( !strcmp( "force-rfc", text ) )
+			peer.natt_mode = IPSEC_NATT_FORCE_RFC;
 
 		// nat-t udp port
 
@@ -231,7 +243,8 @@ void _IKEC::run()
 	// dpd enable
 
 	peer.dpd_mode = IPSEC_DPD_DISABLE;
-	peer.dpd_rate = 10;
+	peer.dpd_delay = 15;
+	peer.dpd_retry = 5;
 
 	numb = 0;
 	config.get_number( "network-dpd-enable", &numb );
@@ -258,12 +271,17 @@ void _IKEC::run()
 
 	// client identification type
 
+	peer.idtype_l = 255;
+
 	if( !config.get_string( "ident-client-type", text, MAX_CONFSTRING, 0 ) )
 	{
 		log( STATUS_FAIL, "config error : auth-client-type undefined\n" );
 		return;
 	}
 
+	if( !strcmp( "none", text ) )
+		peer.idtype_l = ISAKMP_ID_NONE;
+
 	if( !strcmp( "asn1dn", text ) )
 		peer.idtype_l = ISAKMP_ID_ASN1_DN;
 
@@ -279,7 +297,7 @@ void _IKEC::run()
 	if( !strcmp( "address", text ) )
 		peer.idtype_l = ISAKMP_ID_IPV4_ADDR;
 
-	if( !peer.idtype_l )
+	if( peer.idtype_l == 255 )
 	{
 		log( STATUS_FAIL, "config error : ident-client-type is invalid\n" );
 		return;
@@ -287,6 +305,8 @@ void _IKEC::run()
 
 	// server identification type
 
+	peer.idtype_r = 255;
+
 	if( !config.get_string( "ident-server-type", text, MAX_CONFSTRING, 0 ) )
 	{
 		log( STATUS_FAIL, "config error : auth-server-idtype undefined\n" );
@@ -308,7 +328,7 @@ void _IKEC::run()
 	if( !strcmp( "address", text ) )
 		peer.idtype_r = ISAKMP_ID_IPV4_ADDR;
 
-	if( !peer.idtype_r )
+	if( peer.idtype_r == 255 )
 	{
 		log( STATUS_FAIL, "config error : ident-server-type is invalid\n" );
 		return;
@@ -575,6 +595,15 @@ void _IKEC::run()
 	// ---------- CLIENT CONFIG ----------
 	//
 
+	// unity save password option
+
+	if( ( proposal_isakmp.auth_id == XAUTH_AUTH_INIT_PSK ) ||
+	    ( proposal_isakmp.auth_id == XAUTH_AUTH_INIT_RSA ) ||
+	    ( proposal_isakmp.auth_id == HYBRID_AUTH_INIT_RSA ) )
+		if( ( peer.xconf_mode == CONFIG_MODE_PULL ) ||
+		    ( peer.xconf_mode == CONFIG_MODE_PUSH ) )
+			xconf.rqst |= IPSEC_OPTS_SAVEPW;
+
 	// network interface type
 
 	if( !config.get_string( "client-iface", text, MAX_CONFSTRING, 0 ) )
@@ -616,39 +645,56 @@ void _IKEC::run()
 			}
 		}
 
-		// enable wins options
+		// adapter mtu
 
-		numb = 0;
-		if( config.get_number( "client-wins-used", &numb ) )
+		xconf.vmtu = 1500;
+		if( config.get_number( "network-mtu-size", &numb ) )
+			xconf.vmtu = numb;
+	}
+
+	// enable wins options
+
+	if( config.get_number( "client-wins-enable", &numb ) )
+	{
+		if( numb )
 		{
-			if( numb )
+			xconf.opts |= IPSEC_OPTS_NBNS;
+			xconf.rqst |= IPSEC_OPTS_NBNS;
+
+			// netbios name server address
+
+			numb = 0;
+			config.get_number( "client-wins-auto", &numb );
+
+			if( !numb )
 			{
-				xconf.opts |= IPSEC_OPTS_NBNS;
-				xconf.rqst |= IPSEC_OPTS_NBNS;
+				// wins server addresses
 
-				// netbios name server address
+//				xconf.nscfg.nbns_count = 0;
 
-				numb = 0;
-				config.get_number( "client-wins-auto", &numb );
-
-				if( !numb )
+				for( long index = 0; index < IPSEC_NBNS_MAX; index++ )
 				{
-					if( !config.get_string( "client-wins-addr", text, MAX_CONFSTRING, 0 ) )
-					{
-						log( STATUS_FAIL, "config error : client-wins-addr undefined\n" );
-						return;
-					}
+					if( !config.get_string( "client-wins-addr", text, MAX_CONFSTRING, index ) )
+						break;
 
-					xconf.nbns.s_addr = inet_addr( text );
-					xconf.rqst &= ~IPSEC_OPTS_NBNS;
+					xconf.nscfg.nbns_list[ index ].s_addr = inet_addr( text );
+					xconf.nscfg.nbns_count++;
 				}
+
+				if( !xconf.nscfg.nbns_count )
+				{
+					log( STATUS_FAIL, "config error : client-wins-addr undefined\n" );
+					return;
+				}
+
+				xconf.rqst &= ~IPSEC_OPTS_NBNS;
 			}
 		}
 	}
 
 	// enable dns options
 
-	if( config.get_number( "client-dns-enable", &numb ) )
+	if( config.get_number( "client-dns-used", &numb ) )
 	{
 		if( numb )
 		{
@@ -660,15 +706,25 @@ void _IKEC::run()
 
 			if( !numb )
 			{
-				// dns server address
+				// dns server addresses
 
-				if( !config.get_string( "client-dns-addr", text, MAX_CONFSTRING, 0 ) )
+//				xconf.nscfg.dnss_count = 0;
+
+				for( long index = 0; index < IPSEC_DNSS_MAX; index++ )
+				{
+					if( !config.get_string( "client-dns-addr", text, MAX_CONFSTRING, index ) )
+						break;
+
+					xconf.nscfg.dnss_list[ index ].s_addr = inet_addr( text );
+					xconf.nscfg.dnss_count++;
+				}
+
+				if( !xconf.nscfg.dnss_count )
 				{
 					log( STATUS_FAIL, "config error : client-dns-addr undefined\n" );
 					return;
 				}
 
-				xconf.dnss.s_addr = inet_addr( text );
 				xconf.rqst &= ~IPSEC_OPTS_DNSS;
 
 				// domain name suffix
@@ -679,12 +735,18 @@ void _IKEC::run()
 					xconf.rqst &= ~IPSEC_OPTS_DOMAIN;
 				}
 				else
-					strncpy( xconf.suffix, text, CONF_STRLEN );
-
+					strncpy( xconf.nscfg.dnss_suffix, text, CONF_STRLEN );
 			}
 		}
 	}
 
+	// nailed policy enable
+
+	numb = 0;
+	config.get_number( "policy-nailed", &numb );
+	if( numb )
+		peer.nailed = true;
+
 	// auto policy enable
 
 	numb = 0;
@@ -704,13 +766,13 @@ void _IKEC::run()
 	// ---------- CONNECT TO IKED ----------
 	//
 
-	IKEI	ikei;
 	long	result;
 	long	msgres;
 
-	result = ikei.attach( 10000 );
+	IKEI	ikei;
+	pikei = &ikei;
 
-	if( result != IKEI_OK )
+	if( ikei.attach( 3000 ) != IPCERR_OK )
 	{
 		log( STATUS_FAIL, "failed to attach to key daemon ...\n" );
 		return;
@@ -723,7 +785,6 @@ void _IKEC::run()
 	//
 
 	active = true;
-	cancel = false;
 
 	QApplication::postEvent( r, new RunningEvent( true, host ) );
 
@@ -731,9 +792,12 @@ void _IKEC::run()
 	// send the peer configuration message
 	//
 
-	result = ikei.send_msg_peer( &peer, &msgres );
+	IKEI_MSG msg;
 
-	if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+	msg.set_peer( &peer );
+	result = ikei.send_message( msg, &msgres );
+
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
 	{
 		log( STATUS_FAIL, "peer config failed\n" );
 		goto config_failed;
@@ -745,9 +809,10 @@ void _IKEC::run()
 	// send proposal config messages
 	//
 
-	result = ikei.send_msg_proposal( &proposal_isakmp, &msgres );
+	msg.set_proposal( &proposal_isakmp );
+	result = ikei.send_message( msg, &msgres );
 
-	if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
 	{
 		log( STATUS_FAIL, "isakmp proposal config failed\n" );
 		goto config_failed;
@@ -755,9 +820,10 @@ void _IKEC::run()
 
 	log( STATUS_INFO, "iskamp proposal configured\n" );
 
-	result = ikei.send_msg_proposal( &proposal_esp, &msgres );
+	msg.set_proposal( &proposal_esp );
+	result = ikei.send_message( msg, &msgres );
 
-	if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
 	{
 		log( STATUS_FAIL, "esp proposal config failed\n" );
 		goto config_failed;
@@ -767,9 +833,10 @@ void _IKEC::run()
 
 	if( proposal_ipcomp.xform )
 	{
-		result = ikei.send_msg_proposal( &proposal_ipcomp, &msgres );
+		msg.set_proposal( &proposal_ipcomp );
+		result = ikei.send_message( msg, &msgres );
 
-		if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+		if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
 		{
 			log( STATUS_FAIL, "ipcomp proposal config failed\n" );
 			goto config_failed;
@@ -782,9 +849,10 @@ void _IKEC::run()
 	// send the client configuration message
 	//
 
-	result = ikei.send_msg_client( &xconf, &msgres );
+	msg.set_client( &xconf );
+	result = ikei.send_message( msg, &msgres );
 
-	if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
 	{
 		log( STATUS_FAIL, "client config failed\n" );
 		goto config_failed;
@@ -800,13 +868,30 @@ void _IKEC::run()
 		( proposal_isakmp.auth_id == XAUTH_AUTH_INIT_RSA ) ||
 		( proposal_isakmp.auth_id == HYBRID_AUTH_INIT_RSA ) )
 	{
-		ikei.send_msg_cfgstr( CFGSTR_CRED_XAUTH_USER,
-			( char * ) ikec.username.ascii(),
-			ikec.username.length() );
+		BDATA user;
+		user.set( ikec.username.ascii(), ikec.username.length() );
+
+		msg.set_cfgstr( CFGSTR_CRED_XAUTH_USER, &user );
+		result = ikei.send_message( msg, &msgres );
+
+		if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+		{
+			log( STATUS_FAIL, "xauth username config failed\n" );
+			goto config_failed;
+		}
+
+		BDATA pass;
+		pass.set( ikec.password.ascii(), ikec.password.length() );
+
+		msg.set_cfgstr( CFGSTR_CRED_XAUTH_PASS, &pass );
+		result = ikei.send_message( msg, &msgres );
+
+		if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+		{
+			log( STATUS_FAIL, "xauth password config failed\n" );
+			goto config_failed;
+		}
 
-		ikei.send_msg_cfgstr( CFGSTR_CRED_XAUTH_PASS,
-			( char * ) ikec.password.ascii(),
-			ikec.password.length() );
 	}
 
 	//
@@ -815,55 +900,32 @@ void _IKEC::run()
 
 	// client id data
 
-	if( !config.get_string( "ident-client-data", text, MAX_CONFSTRING, 0 ) )
-	{
-		
-		if( ( peer.idtype_l == ISAKMP_ID_KEY_ID ) ||
-			( peer.idtype_l == ISAKMP_ID_FQDN ) ||
-			( peer.idtype_l == ISAKMP_ID_USER_FQDN ) )
-		{
-			log( STATUS_FAIL, "config error : ident-client-data undefined\n" );
-			goto config_failed;
-		}
-	}
-	else
-	{
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_LID, text, strlen( text ), &msgres );
+	config.get_string( "ident-client-data", btext, 0 );
+	msg.set_cfgstr( CFGSTR_CRED_LID, &btext );
+	result = ikei.send_message( msg, &msgres );
 
-		if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
-		{
-			log( STATUS_FAIL, "local id config failed\n" );
-			goto config_failed;
-		}
-
-		log( STATUS_INFO, "local id configured\n" );
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+	{
+		log( STATUS_FAIL, "local id config failed\n" );
+		goto config_failed;
 	}
 
+	log( STATUS_INFO, "local id configured\n" );
+
 	// server id data
 
-	if( !config.get_string( "ident-server-data", text, MAX_CONFSTRING, 0 ) )
-	{
-		if( ( peer.idtype_r == ISAKMP_ID_KEY_ID ) ||
-			( peer.idtype_r == ISAKMP_ID_FQDN ) ||
-			( peer.idtype_r == ISAKMP_ID_USER_FQDN ) )
-		{
-			log( STATUS_FAIL, "config error : ident-server-data undefined\n" );
-			goto config_failed;
-		}
-	}
-	else
-	{
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_RID, text, strlen( text ), &msgres );
+	config.get_string( "ident-server-data", btext, 0 );
+	msg.set_cfgstr( CFGSTR_CRED_RID, &btext );
+	result = ikei.send_message( msg, &msgres );
 
-		if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
-		{
-			log( STATUS_FAIL, "remote id config failed\n" );
-			goto config_failed;
-		}
-
-		log( STATUS_INFO, "remote id configured\n" );
+	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+	{
+		log( STATUS_FAIL, "remote id config failed\n" );
+		goto config_failed;
 	}
 
+	log( STATUS_INFO, "remote id configured\n" );
+
 	//
 	// verify and send our peer authentication info
 	//
@@ -874,7 +936,7 @@ void _IKEC::run()
 	{
 		// server certificate
 
-		if( !config.get_string( "auth-server-cert", text, MAX_CONFSTRING, 0 ) )
+		if( !config.get_string( "auth-server-cert", btext, 0 ) )
 		{
 			log( STATUS_FAIL, "config error : auth-server-cert undefined\n" );
 			goto config_failed;
@@ -882,15 +944,16 @@ void _IKEC::run()
 
 		server_cert_rety:
 
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_RSA_RCRT, text, strlen( text ), &msgres );
+		msg.set_cfgstr( CFGSTR_CRED_RSA_RCRT, &btext );
+		result = ikei.send_message( msg, &msgres );
 
-		if( ( result == IKEI_FAILED ) || ( msgres == IKEI_FAILED ) )
+		if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
 		{
 			log( STATUS_FAIL, "server cert config failed\n" );
 			goto config_failed;
 		}
 
-		if( msgres == IKEI_PASSWD )
+		if( msgres == IKEI_RESULT_PASSWD )
 		{
 			FilePassData PassData;
 			PassData.filepath = text;
@@ -904,9 +967,11 @@ void _IKEC::run()
 				goto config_failed;
 			}
 
-			ikei.send_msg_cfgstr( CFGSTR_CRED_FILE_PASS,
-				( char * ) PassData.password.ascii(),
-				PassData.password.length() );
+			BDATA fpass;
+			fpass.set( PassData.password.ascii(), PassData.password.length() );
+
+			msg.set_cfgstr( CFGSTR_CRED_FILE_PASS, &fpass );
+			result = ikei.send_message( msg, &msgres );
 
 			goto server_cert_rety;
 		}
@@ -919,7 +984,7 @@ void _IKEC::run()
 	{
 		// client certificate
 
-		if( !config.get_string( "auth-client-cert", text, MAX_CONFSTRING, 0 ) )
+		if( !config.get_string( "auth-client-cert", btext, 0 ) )
 		{
 			log( STATUS_FAIL, "config error : auth-client-cert undefined\n" );
 			goto config_failed;
@@ -927,15 +992,16 @@ void _IKEC::run()
 
 		client_cert_rety:
 
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_RSA_LCRT, text, strlen( text ), &msgres );
+		msg.set_cfgstr( CFGSTR_CRED_RSA_LCRT, &btext );
+		result = ikei.send_message( msg, &msgres );
 
-		if( ( result == IKEI_FAILED ) || ( msgres == IKEI_FAILED ) )
+		if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
 		{
 			log( STATUS_FAIL, "client cert config failed\n" );
 			goto config_failed;
 		}
 
-		if( msgres == IKEI_PASSWD )
+		if( msgres == IKEI_RESULT_PASSWD )
 		{
 			FilePassData PassData;
 			PassData.filepath = text;
@@ -949,9 +1015,11 @@ void _IKEC::run()
 				goto config_failed;
 			}
 
-			ikei.send_msg_cfgstr( CFGSTR_CRED_FILE_PASS,
-				( char * ) PassData.password.ascii(),
-				PassData.password.length() );
+			BDATA fpass;
+			fpass.set( PassData.password.ascii(), PassData.password.length() );
+
+			msg.set_cfgstr( CFGSTR_CRED_FILE_PASS, &fpass );
+			result = ikei.send_message( msg, &msgres );
 
 			goto client_cert_rety;
 		}
@@ -960,7 +1028,7 @@ void _IKEC::run()
 
 		// client private key
 
-		if( !config.get_string( "auth-client-key", text, MAX_CONFSTRING, 0 ) )
+		if( !config.get_string( "auth-client-key", btext, 0 ) )
 		{
 			log( STATUS_FAIL, "config error : auth-client-key undefined\n" );
 			goto config_failed;
@@ -968,15 +1036,16 @@ void _IKEC::run()
 
 		client_pkey_rety:
 
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_RSA_LKEY, text, strlen( text ), &msgres );
+		msg.set_cfgstr( CFGSTR_CRED_RSA_LKEY, &btext );
+		result = ikei.send_message( msg, &msgres );
 
-		if( ( result == IKEI_FAILED ) || ( msgres == IKEI_FAILED ) )
+		if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
 		{
 			log( STATUS_FAIL, "client key config failed\n" );
 			goto config_failed;
 		}
 
-		if( msgres == IKEI_PASSWD )
+		if( msgres == IKEI_RESULT_PASSWD )
 		{
 			FilePassData PassData;
 			PassData.filepath = text;
@@ -990,9 +1059,11 @@ void _IKEC::run()
 				goto config_failed;
 			}
 
-			ikei.send_msg_cfgstr( CFGSTR_CRED_FILE_PASS,
-				( char * ) PassData.password.ascii(),
-				PassData.password.length() );
+			BDATA fpass;
+			fpass.set( PassData.password.ascii(), PassData.password.length() );
+
+			msg.set_cfgstr( CFGSTR_CRED_FILE_PASS, &fpass );
+			result = ikei.send_message( msg, &msgres );
 
 			goto client_pkey_rety;
 		}
@@ -1005,14 +1076,18 @@ void _IKEC::run()
 	{
 		// mutual preshared key
 
-		if( !config.get_string( "auth-mutual-psk", text, MAX_CONFSTRING, 0 ) )
+		BDATA psk;
+
+		if( !config.get_binary( "auth-mutual-psk", psk ) )
 		{
 			log( STATUS_FAIL, "config error : auth-mutual-psk undefined\n" );
 			goto config_failed;
 		}
 
-		result = ikei.send_msg_cfgstr( CFGSTR_CRED_PSK, text, strlen( text ), &msgres );
-		if( ( result != IKEI_OK ) || ( msgres != IKEI_OK ) )
+		msg.set_cfgstr( CFGSTR_CRED_PSK, &psk );
+		result = ikei.send_message( msg, &msgres );
+
+		if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
 		{
 			log( STATUS_FAIL, "pre-shared key config failed\n" );
 			goto config_failed;
@@ -1029,8 +1104,17 @@ void _IKEC::run()
 		!( xconf.rqst & IPSEC_OPTS_SPLITDNS ) )
 	{
 		long index = 0;
-		while( config.get_string( "client-splitdns-list", text, MAX_CONFSTRING, index++ ) )
-			ikei.send_msg_cfgstr( CFGSTR_SPLIT_DOMAIN, text, strlen( text ) );
+		while( config.get_string( "client-splitdns-list", btext, index++ ) )
+		{
+			msg.set_cfgstr( CFGSTR_SPLIT_DOMAIN, &btext );
+			result = ikei.send_message( msg, &msgres );
+
+			if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
+			{
+				log( STATUS_FAIL, "split domain name config failed\n" );
+				goto config_failed;
+			}
+		}
 	}
 
 	//
@@ -1054,7 +1138,14 @@ void _IKEC::run()
 			ph2id.addr1.s_addr = addr;
 			ph2id.addr2.s_addr = mask;
 
-			ikei.send_msg_network( &ph2id, UNITY_SPLIT_EXCLUDE );
+			msg.set_network( UNITY_SPLIT_EXCLUDE, &ph2id );
+			result = ikei.send_message( msg, &msgres );
+
+			if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
+			{
+				log( STATUS_FAIL, "policy include config failed\n" );
+				goto config_failed;
+			}
 
 			index++;
 		}
@@ -1074,7 +1165,14 @@ void _IKEC::run()
 			ph2id.addr1.s_addr = addr;
 			ph2id.addr2.s_addr = mask;
 
-			ikei.send_msg_network( &ph2id, UNITY_SPLIT_INCLUDE );
+			msg.set_network( UNITY_SPLIT_INCLUDE, &ph2id );
+			result = ikei.send_message( msg, &msgres );
+
+			if( ( result != IPCERR_OK ) || ( msgres == IKEI_RESULT_FAILED ) )
+			{
+				log( STATUS_FAIL, "policy include config failed\n" );
+				goto config_failed;
+			}
 
 			index++;
 		}
@@ -1084,8 +1182,12 @@ void _IKEC::run()
 	// ---------- ENABLE TUNNEL ----------
 	//
 
-	if( !ikei.send_msg_enable( true ) )
-		return;
+	msg.set_enable( true );
+	if( ikei.send_message( msg ) != IPCERR_OK )
+	{
+		log( STATUS_FAIL, "send enable failed\n" );
+		goto config_failed;
+	}
 
 	//
 	// ---------- FEEDBACK LOOP ----------
@@ -1095,32 +1197,37 @@ void _IKEC::run()
 
 	while( true )
 	{
-		//
-		// check for user cancelation
-		//
-
-		if( cancel == true )
-			if( !ikei.send_msg_enable( false ) )
-				break;
-
 		//
 		// get the next message
 		//
 
-		IKEI_MSG msg;
-		result = ikei.next_msg( msg );
+		result = ikei.recv_message( msg );
 
-		if( result == IKEI_NODATA )
+		if( result == IPCERR_NODATA )
 			continue;
 
-		if( result == IKEI_FAILED )
+		if( ( result == IPCERR_FAILED ) ||
+		    ( result == IPCERR_CLOSED ) )
 			break;
 
+		//
+		// check for user cancelation
+		//
+
+		if( result == IPCERR_WAKEUP )
+		{
+			msg.set_enable( false );
+			if( ikei.send_message( msg ) != IPCERR_OK )
+				break;
+
+			continue;
+		}
+
 		//
 		// evaluate the message
 		//
 
-		switch( msg.type )
+		switch( msg.header.type )
 		{
 			//
 			// enable message
@@ -1128,8 +1235,7 @@ void _IKEC::run()
 
 			case IKEI_MSGID_ENABLE:
 			
-				result = ikei.recv_msg_enable( &msgres );
-				if( result != IKEI_OK )
+				if( msg.get_enable( &msgres ) != IPCERR_OK )
 					break;
 
 				QApplication::postEvent( r, new EnableEvent( msgres ) );
@@ -1142,14 +1248,10 @@ void _IKEC::run()
 
 			case IKEI_MSGID_STATUS:
 			{
-				char	txtmsg[ IKEI_MAX_BDATA + 1 ] = { 0 };
-				size_t	txtlen = IKEI_MAX_BDATA;
-
-				result = ikei.recv_msg_status( &status, txtmsg, txtlen );
-				if( result != IKEI_OK )
+				if( msg.get_status( &status, &btext ) != IPCERR_OK )
 					break;
 
-				log( status, txtmsg );
+				log( status, btext.text(), btext.size() );
 
 				break;
 			}
@@ -1162,8 +1264,7 @@ void _IKEC::run()
 			{
 				IKEI_STATS stats;
 
-				result = ikei.recv_msg_stats( &stats );
-				if( result != IKEI_OK )
+				if( msg.get_stats( &stats ) != IPCERR_OK )
 					break;
 
 				QApplication::postEvent( r, new StatsEvent( stats ) );
@@ -1184,7 +1285,6 @@ void _IKEC::run()
 	//
 
 	active = false;
-	cancel = false;
 
 	QApplication::postEvent( r, new RunningEvent( false, "" ) );
 
