@@ -48,7 +48,7 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 	//
 
 	assert( ph1 != NULL );
-	assert( inform->nlist.count() != 0 );
+	assert( inform->notifications.count() != 0 );
 
 	//
 	// create notification packet
@@ -62,7 +62,7 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 	//
 
 	unsigned char flags = 0;
-	if( ph1->lstate & LSTATE_MATURE )
+	if( ph1->status() >= XCH_STATUS_MATURE )
 		flags |= ISAKMP_FLAG_ENCRYPT;
 
 	//
@@ -70,7 +70,7 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 	//
 
 	IKE_NOTIFY notify;
-	inform->nlist.get( notify, 0 );
+	inform->notifications.get( notify, 0 );
 
 	unsigned char next;
 	if( flags & ISAKMP_FLAG_ENCRYPT )
@@ -92,7 +92,7 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 
 	if( flags & ISAKMP_FLAG_ENCRYPT )
 	{
-		inform->hash_l.set( 0, ph1->hash_size );
+		inform->hash_l.size( ph1->hash_size );
 		payload_add_hash( packet, inform->hash_l, notify.type );
 	}
 
@@ -110,7 +110,7 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 		// get next notification
 		//
 
-		if( !inform->nlist.get( notify, nindex++ ) )
+		if( !inform->notifications.get( notify, nindex++ ) )
 			break;
 
 		//
@@ -118,10 +118,10 @@ long _IKED::process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform )
 		//
 
 		next = ISAKMP_PAYLOAD_NONE;
-		if( inform->nlist.count() > nindex )
+		if( inform->notifications.count() > nindex )
 		{
 			IKE_NOTIFY next_notify;
-			inform->nlist.get( next_notify, nindex );
+			inform->notifications.get( next_notify, nindex );
 
 			next = next_notify.type;
 		}
@@ -181,6 +181,14 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	bool	secure = false;
 	long	result = LIBIKE_OK;
 
+	//
+	// log packet type
+	//
+
+	log.txt( LLOG_INFO,
+		"ii : processing informational packet ( %i bytes )\n",
+		packet.size() );
+
 	//
 	// calculate iv for this
 	// informational exchange
@@ -194,15 +202,21 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	// decrypt packet
 	//
 
-	packet_ike_decrypt( ph1, packet, &inform.iv );
+	if( packet_ike_decrypt( ph1, packet, &inform.iv ) != LIBIKE_OK )
+	{
+		log.txt( LLOG_ERROR,
+			"!! : informational packet ignored ( packet decryption error )\n" );
+
+		return LIBIKE_OK;
+	}
 
 	//
-	// if we are dumping the ike packets,
-	// we need to rebuild a full packet
-	// to dump to pcap format 
+	// if we are dumping decrypted packets,
+	// we need to rebuild a full packet to
+	// dump to pcap format 
 	//
 
-	if( dump_ike )
+	if( dump_decrypt )
 	{
 		//
 		// strip the isakmp encryption flag
@@ -220,7 +234,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			packet_ip_dump,
 			ph1->tunnel->saddr_r,
 			ph1->tunnel->saddr_l,
-			ph1->tunnel->natt_v );
+			ph1->tunnel->natt_version );
 
 		//
 		// obtain ethernet header
@@ -233,7 +247,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		// dump the packet
 		//
 
-		pcap_ike.dump(
+		pcap_decrypt.dump(
 			ethhdr,
 			packet_ip_dump );
 	}
@@ -284,7 +298,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 				inform.hda.set( packet.buff() + beg, end - beg );
 
 				if( result == LIBIKE_OK )
-					inform.nlist.add( notify );
+					inform.notifications.add( notify );
 
 				break;
 			}
@@ -302,7 +316,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 				inform.hda.set( packet.buff() + beg, end - beg );
 
 				if( result == LIBIKE_OK )
-					inform.nlist.add( notify );
+					inform.notifications.add( notify );
 
 				break;
 			}
@@ -313,7 +327,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 			default:
 
-				log.txt( LOG_ERROR,
+				log.txt( LLOG_ERROR,
 					"!! : unhandled informational payload \'%s\' ( %i )\n",
 					find_name( NAME_PAYLOAD, payload ),
 					payload );
@@ -327,10 +341,8 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		// was the entire payload read
 		//
 
-		size_t bytes_left;
-		packet.chk_payload( bytes_left );
-		if( bytes_left )
-			log.txt( LOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
+		if( packet.get_payload_left() )
+			log.txt( LLOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
 
 		//
 		// check the result
@@ -359,12 +371,12 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	// check all notification payloads
 	//
 
-	if( inform.nlist.count() )
+	if( inform.notifications.count() )
 	{
 		IKE_NOTIFY notify;
 
 		long nindex = 0;
-		while( inform.nlist.get( notify, nindex++ ) )
+		while( inform.notifications.get( notify, nindex++ ) )
 		{
 			switch( notify.type )
 			{
@@ -386,7 +398,7 @@ long _IKED::process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 long _IKED::inform_chk_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 {
 	BDATA hash_c;
-	hash_c.set( 0, ph1->hash_size );
+	hash_c.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
 	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
@@ -396,34 +408,34 @@ long _IKED::inform_chk_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 	HMAC_cleanup( &ctx_prf );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		hash_c.buff(),
 		hash_c.size(),
 		"== : informational hash_i ( computed )" );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		inform->hash_r.buff(),
 		inform->hash_r.size(),
 		"== : informational hash_c ( received )" );
 
 	if( memcmp( inform->hash_r.buff(), hash_c.buff(), hash_c.size() ) )
 	{
-		log.txt( LOG_ERROR,	"!! : informational hash verification failed\n" );
+		log.txt( LLOG_ERROR,	"!! : informational hash verification failed\n" );
 
 		return LIBIKE_FAILED;
 	}
 
-	log.txt( LOG_DEBUG,	"ii : informational hash verified\n" );
+	log.txt( LLOG_DEBUG,	"ii : informational hash verified\n" );
 
 	return LIBIKE_OK;
 }
 
 long _IKED::inform_gen_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 {
-	inform->hash_l.set( 0, ph1->hash_size );
+	inform->hash_l.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
 	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
@@ -433,8 +445,8 @@ long _IKED::inform_gen_hash( IDB_PH1 * ph1, IDB_XCH * inform )
 	HMAC_cleanup( &ctx_prf );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		inform->hash_l.buff(),
 		inform->hash_l.size(),
 		"== : new informational hash" );
@@ -496,7 +508,7 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 	// log the notification
 	//
 
-	log.txt( LOG_INFO,
+	log.txt( LLOG_INFO,
 		"ii : received peer %s notification\n"
 		"ii : - %s -> %s\n"
 		"ii : - %s spi = %s\n"
@@ -514,8 +526,10 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 	// of a mature phase1 sa
 	//
 
-	if( secure && !( ph1->lstate & LSTATE_MATURE ) )
-		return LIBIKE_FAILED;
+	if( secure )
+		if( ( ph1->status() <= XCH_STATUS_MATURE ) ||
+			( ph1->status() >= XCH_STATUS_EXPIRING ) )
+			return LIBIKE_FAILED;
 
 	//
 	// process the notify message
@@ -533,42 +547,47 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 			{
 				case ISAKMP_N_DPD_R_U_THERE:
 				{
-					uint32_t dpdseq;
-					notify->data.get( &dpdseq, sizeof( dpdseq ) );
-					dpdseq = ntohl( dpdseq );
+					uint32_t sequence;
+					notify->data.get( &sequence, sizeof( sequence ) );
+					sequence = ntohl( sequence );
 
 					inform_new_notify( ph1, NULL, ISAKMP_N_DPD_R_U_THERE_ACK, &notify->data );
 
-					log.txt( LOG_DEBUG, "ii : DPD ARE-YOU-THERE sequence %08x returned\n", dpdseq );
+					log.txt( LLOG_DEBUG, "ii : DPD ARE-YOU-THERE sequence %08x returned\n", sequence );
 
 					break;
 				}
 
 				case ISAKMP_N_DPD_R_U_THERE_ACK:
 				{
-					if( notify->data.size() == sizeof( ph1->dpd_res ) )
+					if( notify->data.size() == sizeof( ph1->tunnel->event_dpd.sequence ) )
 					{
 						//
 						// obtain sequence number and
 						// convert to host byte order
 						//
 
-						uint32_t dpdseq;
-						notify->data.get( &dpdseq, sizeof( dpdseq ) );
-						dpdseq = ntohl( dpdseq );
+						uint32_t sequence;
+						notify->data.get( &sequence, sizeof( sequence ) );
+						sequence = ntohl( sequence );
 
 						//
-						// check dpd sequence number. if accepted,
-						// set dpd response to current sequence
+						// check dpd sequence number
 						//
 
-						if( dpdseq <= ph1->dpd_req )
+						if( sequence != ph1->tunnel->event_dpd.sequence )
 						{
-							log.txt( LOG_DEBUG, "ii : DPD ARE-YOU-THERE-ACK sequence %08x accepted\n", dpdseq );
-							ph1->dpd_res = ph1->dpd_req;
+							log.txt( LLOG_ERROR, "!! : DPD ARE-YOU-THERE-ACK sequence %08x rejected\n", sequence );
+							break;
 						}
-						else
-							log.txt( LOG_ERROR, "!! : DPD ARE-YOU-THERE-ACK sequence %08x rejected\n", dpdseq );
+
+						//
+						// setup the next dpd cycle
+						//
+
+						log.txt( LLOG_DEBUG, "ii : DPD ARE-YOU-THERE-ACK sequence %08x accepted\n", sequence );
+
+						ph1->tunnel->event_dpd.next();
 					}
 
 					break;
@@ -598,7 +617,17 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 					unsigned short	ltype;
 
 					IDB_PH2 * ph2_notify;
-					if( get_phase2( true, &ph2_notify, ph1->tunnel, 0, LSTATE_DELETE, NULL, NULL, NULL, &notify->spi ) )
+
+					if( idb_list_ph2.find(
+							true,
+							&ph2_notify,
+							ph1->tunnel,
+							XCH_STATUS_ANY,
+							XCH_STATUS_DEAD,
+							NULL,
+							NULL,
+							NULL,
+							&notify->spi ) )
 					{
 						//
 						// create a temp packet for parsing
@@ -655,7 +684,7 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 
 						if( lsecs )
 						{
-							log.txt( LOG_INFO, "ii : adjusted phase2 sa lifetime to %i seconds\n", lsecs );
+							log.txt( LLOG_INFO, "ii : adjusted phase2 sa lifetime to %i seconds\n", lsecs );
 
 							IKE_PROPOSAL * proposal_l;
 							IKE_PROPOSAL * proposal_r;
@@ -667,17 +696,17 @@ long _IKED::inform_chk_notify( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 								proposal_l->life_sec = lsecs;
 								proposal_r->life_sec = lsecs;
 
-								log.txt( LOG_DEBUG, "XX : spi_l = 0x%08x\n", ntohl( proposal_l->spi.spi ) );
-								log.txt( LOG_DEBUG, "XX : spi_r = 0x%08x\n", ntohl( proposal_r->spi.spi ) );
+								log.txt( LLOG_DEBUG, "XX : spi_l = 0x%08x\n", ntohl( proposal_l->spi.spi ) );
+								log.txt( LLOG_DEBUG, "XX : spi_r = 0x%08x\n", ntohl( proposal_r->spi.spi ) );
 								pindex++;
 							}
 						}
 
 						if( ldata )
-							log.txt( LOG_INFO, "ii : adjusted phase2 sa lifetime to %i kbytes\n", ldata );
+							log.txt( LLOG_INFO, "ii : adjusted phase2 sa lifetime to %i kbytes\n", ldata );
 
 						if( !lsecs && !ldata )
-							log.txt( LOG_INFO, "ii : invalid RESPONDER-LIFETIME attribute data\n" );
+							log.txt( LLOG_INFO, "ii : invalid RESPONDER-LIFETIME attribute data\n" );
 
 						ph2_notify->dec( true );
 					}
@@ -712,7 +741,7 @@ long _IKED::inform_chk_delete( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 	// log the delete notification
 	//
 
-	log.txt( LOG_INFO,
+	log.txt( LLOG_INFO,
 		"ii : received peer DELETE message\n"
 		"ii : - %s -> %s\n"
 		"ii : - %s spi = %s\n",
@@ -727,8 +756,10 @@ long _IKED::inform_chk_delete( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 	// of a mature phase1 sa
 	//
 
-	if( secure && !( ph1->lstate & LSTATE_MATURE ) )
-		return LIBIKE_FAILED;
+	if( secure )
+		if( ( ph1->status() <= XCH_STATUS_MATURE ) ||
+			( ph1->status() >= XCH_STATUS_EXPIRING ) )
+			return LIBIKE_FAILED;
 
 	//
 	// process the delete message
@@ -747,15 +778,19 @@ long _IKED::inform_chk_delete( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 			//
 
 			IDB_PH1 * ph1_delete;
-			if( get_phase1( true, &ph1_delete, ph1->tunnel, LSTATE_MATURE, LSTATE_DELETE, &notify->spi.cookies ) )
+			if( idb_list_ph1.find(
+					true,
+					&ph1_delete,
+					ph1->tunnel,
+					XCH_STATUS_MATURE,
+					XCH_STATUS_DEAD,
+					&notify->spi.cookies ) )
 			{
-				log.txt( LOG_INFO, "ii : cleanup, marked phase1 %s for removal\n", txtspi );
+				log.txt( LLOG_INFO,
+					"ii : cleanup, marked phase1 %s for removal\n",
+					txtspi );
 
-				if( ph1->tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
-					if( !ph1->tunnel->close )
-						ph1->tunnel->close = TERM_PEER_CLOSE;
-
-				ph1_delete->lstate |= ( LSTATE_DELETE | LSTATE_NOTIFY );
+				ph1_delete->status( XCH_STATUS_DEAD, XCH_FAILED_PEER_DELETE, 0 );
 				ph1_delete->dec( true );
 			}
 
@@ -773,11 +808,22 @@ long _IKED::inform_chk_delete( IDB_PH1 * ph1, IKE_NOTIFY * notify, bool secure )
 			//
 
 			IDB_PH2 * ph2_delete;
-			if( get_phase2( true, &ph2_delete, ph1->tunnel, LSTATE_MATURE, LSTATE_DELETE, NULL, NULL, NULL, &notify->spi ) )
+			if( idb_list_ph2.find(
+					true,
+					&ph2_delete,
+					ph1->tunnel,
+					XCH_STATUS_MATURE,
+					XCH_STATUS_DEAD,
+					NULL,
+					NULL, 
+					NULL,
+					&notify->spi ) )
 			{
-				log.txt( LOG_INFO, "DB : cleanup, marked phase2 %s for removal\n", txtspi );
+				log.txt( LLOG_INFO,
+					"DB : cleanup, marked phase2 %s for removal\n",
+					txtspi );
 
-				ph2_delete->lstate |= ( LSTATE_DELETE | LSTATE_NOTIFY );
+				ph2_delete->status( XCH_STATUS_DEAD, XCH_FAILED_PEER_DELETE, 0 );
 				ph2_delete->dec( true );
 			}
 
@@ -838,7 +884,7 @@ long _IKED::inform_new_notify( IDB_PH1 * ph1, IDB_PH2 * ph2, unsigned short code
 
 		inform_get_spi( txtspi, ph1, &notify );
 
-		log.txt( LOG_INFO,
+		log.txt( LLOG_INFO,
 			"ii : sending peer %s notification\n"
 			"ii : - %s -> %s\n"
 			"ii : - %s spi = %s\n"
@@ -854,7 +900,7 @@ long _IKED::inform_new_notify( IDB_PH1 * ph1, IDB_PH2 * ph2, unsigned short code
 		// add notification data
 		//
 
-		inform.nlist.add( notify );
+		inform.notifications.add( notify );
 	}
 	else
 	{
@@ -876,7 +922,7 @@ long _IKED::inform_new_notify( IDB_PH1 * ph1, IDB_PH2 * ph2, unsigned short code
 		// log the notification
 		//
 
-		log.txt( LOG_INFO,
+		log.txt( LLOG_INFO,
 			"ii : sending peer %s notification\n"
 			"ii : - %s -> %s\n",
 			find_name( NAME_NOTIFY, notify.code ),
@@ -887,7 +933,7 @@ long _IKED::inform_new_notify( IDB_PH1 * ph1, IDB_PH2 * ph2, unsigned short code
 		// add notification data
 		//
 
-		inform.nlist.add( notify );
+		inform.notifications.add( notify );
 	}
 
 	return process_inform_send( ph1, &inform );
@@ -939,7 +985,7 @@ long _IKED::inform_new_delete( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 		inform_get_spi( txtspi, ph1, &notify );
 
-		log.txt( LOG_INFO,
+		log.txt( LLOG_INFO,
 			"ii : sending peer DELETE message\n"
 			"ii : - %s -> %s\n"
 			"ii : - %s spi = %s\n"
@@ -954,7 +1000,7 @@ long _IKED::inform_new_delete( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 		// add notification data
 		//
 
-		inform.nlist.add( notify );
+		inform.notifications.add( notify );
 	}
 	else
 	{
@@ -970,7 +1016,7 @@ long _IKED::inform_new_delete( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 		// log the delete addresses
 		//
 
-		log.txt( LOG_INFO,
+		log.txt( LLOG_INFO,
 			"ii : sending peer DELETE message\n"
 			"ii : - %s -> %s\n",
 			txtaddr_l,
@@ -996,7 +1042,7 @@ long _IKED::inform_new_delete( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			inform_get_spi( txtspi, ph1, &notify );
 
-			log.txt( LOG_INFO,
+			log.txt( LLOG_INFO,
 				"ii : - %s spi = %s\n"
 				"ii : - data size %i\n",
 				find_name( NAME_PROTOCOL, notify.proto ),
@@ -1007,7 +1053,7 @@ long _IKED::inform_new_delete( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 			// add notification data
 			//
 
-			inform.nlist.add( notify );
+			inform.notifications.add( notify );
 		}
 	}
 
