@@ -45,17 +45,34 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 {
 	long result = LIBIKE_OK;
 
-	IDB_PH2 * ph2 = NULL;
+	//
+	// log packet type
+	//
+
+	log.txt( LLOG_INFO,
+		"ii : processing phase2 packet ( %i bytes )\n",
+		packet.size() );
 
 	//
 	// attempt to locate a known
 	// sa for this message id
 	//
 
+	IDB_PH2 * ph2 = NULL;
+
 	uint32_t msgid;
 	packet.get_msgid( msgid );
 
-	get_phase2( true, &ph2, ph1->tunnel, 0, 0, NULL, &msgid, NULL, NULL );
+	idb_list_ph2.find(
+		true,
+		&ph2,
+		ph1->tunnel,
+		XCH_STATUS_ANY,
+		XCH_STATUS_ANY,
+		NULL,
+		&msgid,
+		NULL,
+		NULL );
 
 	if( ph2 == NULL )
 	{
@@ -73,36 +90,53 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		//
 
 		phase2_gen_iv( ph1, ph2->msgid, ph2->iv );
+
+		//
+		// make sure we respond using the
+		// isakmp sa that was last seen
+		//
+
+		ph2->cookies = ph1->cookies;
 	}
 
 	//
 	// make sure we are not dealing
-	// whith a sa marked as dead
+	// whith a mature or dead sa
 	//
 
-	if( ( ph1->lstate & LSTATE_DELETE ) ||
-	    ( ph2->lstate & LSTATE_DELETE ) )
+	if( ( ph1->status() == XCH_STATUS_DEAD ) ||
+	    ( ph2->status() == XCH_STATUS_DEAD ) )
 	{
-		log.txt( LOG_ERROR, "!! : ignore phase2 packet, sa marked for death\n" );
-
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( sa marked for death )\n" );
 		ph2->dec( true );
+		return LIBIKE_OK;
+	}
 
+	if( ph2->status() >= XCH_STATUS_MATURE )
+	{
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( sa already mature )\n" );
+		ph2->dec( true );
 		return LIBIKE_OK;
 	}
 
 	//
-	// decrypt packet
+	// attempt to decrypt our packet
 	//
 
-	packet_ike_decrypt( ph1, packet, &ph2->iv );
+	if( packet_ike_decrypt( ph1, packet, &ph2->iv ) != LIBIKE_OK )
+	{
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( packet decryption error )\n" );
+		ph2->dec( true );
+		return LIBIKE_OK;
+	}
 
 	//
-	// if we are dumping the ike packets,
-	// we need to rebuild a full packet
-	// to dump to pcap format 
+	// if we are dumping decrypted packets,
+	// we need to rebuild a full packet to
+	// dump to pcap format 
 	//
 
-	if( dump_ike )
+	if( dump_decrypt )
 	{
 		//
 		// strip the isakmp encryption flag
@@ -120,7 +154,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			packet_ip_dump,
 			ph1->tunnel->saddr_r,
 			ph1->tunnel->saddr_l,
-			ph1->tunnel->natt_v );
+			ph1->tunnel->natt_version );
 
 		//
 		// obtain ethernet header
@@ -133,11 +167,88 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		// dump the packet
 		//
 
-		pcap_ike.dump(
+		pcap_decrypt.dump(
 			ethhdr,
 			packet_ip_dump );
 	}
+/*
+	//
+	// read and validate the hash
+	//
 
+	if( !packet.get_payload( false, payload ) )
+	{
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( invalid hash payload )\n" );
+		ph2->dec( true );
+		return LIBIKE_OK;
+	}
+
+	if( payload_get_hash( packet, ph2->hash_r, ph1->hash_size ) != LIBIKE_OK )
+	{
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( invalid hash value )\n" );
+		ph2->dec( true );
+		return LIBIKE_OK;
+	}
+
+	//
+	// populate hash data accumulator
+	//
+
+	ph2->hda.size( 0 );
+	ph2->hda.add(
+		packet.buff() + packet.oset(),
+		packet.size() - packet.oset() );
+
+	if( ph2->initiator )
+	{
+		//
+		// check responders hash
+		//
+
+		if( !( ph2->xstate & XSTATE_RECV_HA ) )
+		{
+			//
+			// quick mode hash
+			//
+
+			if( phase2_chk_hash_r( ph1, ph2 ) != LIBIKE_OK )
+				packet.notify = ISAKMP_N_INVALID_HASH_INFORMATION;
+
+			ph2->xstate |= XSTATE_RECV_HA;
+//			ph2->lstate |= LSTATE_CHKHASH;
+		}
+	}
+	else
+	{
+		//
+		// check initiators hash
+		//
+
+		if( !( ph2->xstate & XSTATE_RECV_HA ) )
+		{
+			//
+			// quick mode hash
+			//
+
+			if( phase2_chk_hash_i( ph1, ph2 ) != LIBIKE_OK )
+				packet.notify = ISAKMP_N_INVALID_HASH_INFORMATION;
+
+			ph2->xstate |= XSTATE_RECV_HA;
+//			ph2->lstate |= LSTATE_CHKHASH;
+		}
+		else
+		{
+			//
+			// livliness proof hash
+			//
+
+			if( phase2_chk_hash_p( ph1, ph2 ) != LIBIKE_OK )
+				packet.notify = ISAKMP_N_INVALID_HASH_INFORMATION;
+
+			ph2->xstate |= XSTATE_RECV_LP;
+		}
+	}
+*/
 	//
 	// read and process all payloads
 	//
@@ -170,7 +281,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			{
 				if( ( ph2->xstate & XSTATE_RECV_HA ) &&
 					( ph2->xstate & XSTATE_RECV_LP ) )
-					log.txt( LOG_INFO, "<< : ignoring duplicate hash payload\n" );
+					log.txt( LLOG_INFO, "<< : ignoring duplicate hash payload\n" );
 				else
 				{
 					result = payload_get_hash( packet, ph2->hash_r, ph1->hash_size );
@@ -191,7 +302,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			case ISAKMP_PAYLOAD_SA:
 			{
 				if( ph2->xstate & XSTATE_RECV_SA )
-					log.txt( LOG_INFO, "<< : ignoring duplicate security association payload\n" );
+					log.txt( LLOG_INFO, "<< : ignoring duplicate security association payload\n" );
 				else
 				{
 					size_t beg = packet.oset() - 4;
@@ -213,7 +324,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			case ISAKMP_PAYLOAD_NONCE:
 			{
 				if( ph2->xstate & XSTATE_RECV_NO )
-					log.txt( LOG_INFO, "<< : ignoring duplicate nonce payload\n" );
+					log.txt( LLOG_INFO, "<< : ignoring duplicate nonce payload\n" );
 				else
 				{
 					size_t beg = packet.oset() - 4;
@@ -293,7 +404,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 				if( ( ph2->xstate & XSTATE_RECV_IDR ) &&
 				    ( ph2->xstate & XSTATE_RECV_IDL ) )
-					log.txt( LOG_INFO, "<< : ignoring duplicate id payload\n" );
+					log.txt( LLOG_INFO, "<< : ignoring duplicate id payload\n" );
 
 				break;
 			}
@@ -305,7 +416,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			case ISAKMP_PAYLOAD_KEX:
 			{
 				if( ph2->xstate & XSTATE_RECV_KE )
-					log.txt( LOG_INFO, "<< : ignoring duplicate key excahnge payload\n" );
+					log.txt( LLOG_INFO, "<< : ignoring duplicate key excahnge payload\n" );
 				else
 				{
 					size_t beg = packet.oset() - 4;
@@ -320,6 +431,24 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 				break;
 			}
 
+			//
+			// natt original address ( ignored )
+			//
+
+			case ISAKMP_PAYLOAD_NAT_VXX_ORIG:
+			case ISAKMP_PAYLOAD_NAT_RFC_ORIG:
+			{
+				log.txt( LLOG_INFO, "<< : natt original address\n" );
+
+				size_t beg = packet.oset() - 4;
+				packet.get_null( packet.get_payload_left() );
+				size_t end = packet.oset();
+
+				ph2->hda.add( packet.buff() + beg, end - beg );
+
+				break;
+			}
+
 			//
 			// notify payload
 			//
@@ -335,7 +464,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 				ph2->hda.add( packet.buff() + beg, end - beg );
 
 				if( result == LIBIKE_OK )
-					ph2->nlist.add( notify );
+					ph2->notifications.add( notify );
 
 				break;
 			}
@@ -346,7 +475,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 			default:
 
-				log.txt( LOG_ERROR,
+				log.txt( LLOG_ERROR,
 					"!! : unhandled phase2 payload \'%s\' ( %i )\n",
 					find_name( NAME_PAYLOAD, payload ),
 					payload );
@@ -362,10 +491,8 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		// check that the entire payload was read
 		//
 
-		size_t bytes_left;
-		packet.chk_payload( bytes_left );
-		if( bytes_left )
-			log.txt( LOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
+		if( packet.get_payload_left() )
+			log.txt( LLOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
 
 		//
 		// check the final paylaod process result
@@ -373,18 +500,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 		if( result != LIBIKE_OK )
 		{
-			//
-			// potentialy notify our peer
-			//
-
-			if( packet.notify && ph2->tunnel->peer->notify )
-				inform_new_notify( ph1, NULL, packet.notify );
-
-			//
-			// flag sa for removal
-			//
-
-			ph2->lstate |= LSTATE_DELETE;
+			ph2->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_FORMAT, packet.notify );
 			ph2->dec( true );
 
 			return result;
@@ -411,10 +527,10 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			// check responders quick mode hash
 			//
 
-			if(  ( ph2->xstate & XSTATE_RECV_HA ) &&
-				 ( ph2->xstate & XSTATE_RECV_SA ) &&
-				 ( ph2->xstate & XSTATE_RECV_NO ) &&
-				!( ph2->lstate & LSTATE_MATURE ) )
+			if( ( ph2->status() < XCH_STATUS_MATURE ) &&
+				( ph2->xstate & XSTATE_RECV_HA ) &&
+				( ph2->xstate & XSTATE_RECV_SA ) &&
+				( ph2->xstate & XSTATE_RECV_NO ) )
 			{
 				result = phase2_chk_hash_r( ph1, ph2 );
 
@@ -446,7 +562,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 				text_addr( txtaddr_l, &ph1->tunnel->saddr_l, true );
 				text_addr( txtaddr_r, &ph1->tunnel->saddr_r, true );
 
-				log.txt( LOG_INFO,
+				log.txt( LLOG_INFO,
 					"ii : phase2 sa established\n"
 					"ii : %s <-> %s\n",
 					txtaddr_l,
@@ -494,8 +610,8 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			// check initiators liveliness proof hash
 			//
 
-			if(  ( ph2->xstate & XSTATE_RECV_LP ) &&
-				!( ph2->lstate & LSTATE_MATURE ) )
+			if( ( ph2->status() < XCH_STATUS_MATURE ) &&
+				( ph2->xstate & XSTATE_RECV_LP ) )
 			{
 				result = phase2_chk_hash_p( ph1, ph2 );
 
@@ -522,15 +638,15 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 					text_addr( txtaddr_l, &ph1->tunnel->saddr_l, true );
 					text_addr( txtaddr_r, &ph1->tunnel->saddr_r, true );
 
-					log.txt( LOG_INFO,
+					log.txt( LLOG_INFO,
 						"ii : phase2 sa established\n"
 						"ii : %s <-> %s\n",
 						txtaddr_l,
 						txtaddr_r );
 
+					ph2->status( XCH_STATUS_MATURE, XCH_NORMAL, 0 );
 					ph2->clean();
-
-					ph2->lstate |= LSTATE_MATURE;
+					ph2->resend_clear( true );
 				}
 			}
 		}
@@ -544,18 +660,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	if( result != LIBIKE_OK )
 	{
-		//
-		// potentialy notify our peer
-		//
-
-		if( ph2->tunnel->peer->notify )
-			inform_new_notify( ph1, ph2, packet.notify );
-
-		//
-		// flag sa for removal
-		//
-
-		ph2->lstate |= LSTATE_DELETE;
+		ph2->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_FORMAT, packet.notify );
 		ph2->dec( true );
 
 		return result;
@@ -566,9 +671,9 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	// packets that may be necessary
 	//
 
-	if(  ( ph2->lstate & LSTATE_HASSPI ) &&
-		!( ph2->lstate & LSTATE_MATURE ) &&
-		!( ph2->lstate & LSTATE_DELETE ) )
+	if( ( ph2->lstate & LSTATE_HASSPI ) &&
+		( ph1->status() != XCH_STATUS_DEAD ) &&
+		( ph2->status() < XCH_STATUS_MATURE ) )
 		process_phase2_send( ph1, ph2 );
 
 	//
@@ -601,7 +706,7 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			size_t off = packet.size();
 
-			ph2->hash_l.set( 0, ph1->hash_size );
+			ph2->hash_l.size( ph1->hash_size );
 
 			payload_add_hash( packet, ph2->hash_l, ISAKMP_PAYLOAD_SA );
 
@@ -704,14 +809,15 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			phase2_gen_keys( ph1, ph2 );
 
-			ph2->clean();
-
 			//
 			// update sa state
 			//
 
 			ph2->xstate |= XSTATE_SENT_LP;
-			ph2->lstate |= LSTATE_MATURE;
+
+			ph2->status( XCH_STATUS_MATURE, XCH_NORMAL, 0 );
+			ph2->clean();
+			ph2->resend_clear( true );
 		}
 	}
 
@@ -741,7 +847,7 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			size_t off = packet.size();
 
-			ph2->hash_l.set( 0, ph1->hash_size );
+			ph2->hash_l.size( ph1->hash_size );
 
 			payload_add_hash( packet, ph2->hash_l, ISAKMP_PAYLOAD_SA );
 
@@ -833,6 +939,7 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			size_t end = packet.size();
 
+			ph2->hda.del();
 			ph2->hda.set( packet.buff() + beg, end - beg );
 
 			//
@@ -869,13 +976,13 @@ long _IKED::phase2_gen_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	input.add( ph2->hda );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		input.buff(),
 		input.size(),
 		"== : phase2 hash_i ( input )" );
 
-	hash.set( 0, ph1->hash_size );
+	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
 	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
@@ -884,8 +991,8 @@ long _IKED::phase2_gen_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	HMAC_cleanup( &ctx_prf );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		hash.buff(),
 		hash.size(),
 		"== : phase2 hash_i ( computed )" );
@@ -906,13 +1013,13 @@ long _IKED::phase2_gen_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	input.add( ph2->hda );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		input.buff(),
 		input.size(),
 		"== : phase2 hash_r ( input )" );
 
-	hash.set( 0, ph1->hash_size );
+	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
 	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
@@ -921,8 +1028,8 @@ long _IKED::phase2_gen_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	HMAC_cleanup( &ctx_prf );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		hash.buff(),
 		hash.size(),
 		"== : phase2 hash_r ( computed )" );
@@ -948,13 +1055,13 @@ long _IKED::phase2_gen_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	}
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		input.buff(),
 		input.size(),
 		"== : phase2 hash_p ( input )" );
 
-	hash.set( 0, ph1->hash_size );
+	hash.size( ph1->hash_size );
 
 	HMAC_CTX ctx_prf;
 	HMAC_Init( &ctx_prf, ph1->skeyid_a.buff(), ( int ) ph1->skeyid_a.size(), ph1->evp_hash );
@@ -963,8 +1070,8 @@ long _IKED::phase2_gen_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2, BDATA & hash )
 	HMAC_cleanup( &ctx_prf );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		hash.buff(),
 		hash.size(),
 		"== : phase2 hash_p ( computed )" );
@@ -982,8 +1089,8 @@ long _IKED::phase2_chk_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	phase2_gen_hash_i( ph1, ph2, hash );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		ph2->hash_r.buff(),
 		ph2->hash_r.size(),
 		"== : phase2 hash_i ( received )" );
@@ -1004,7 +1111,7 @@ long _IKED::phase2_chk_hash_i( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, initiator quick mode hash invalid\n"
 			"!! : %s <-> %s\n",
 			txtaddr_l,
@@ -1026,8 +1133,8 @@ long _IKED::phase2_chk_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	phase2_gen_hash_r( ph1, ph2, hash );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		ph2->hash_r.buff(),
 		ph2->hash_r.size(),
 		"== : phase2 hash_r ( received )" );
@@ -1048,7 +1155,7 @@ long _IKED::phase2_chk_hash_r( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, responder quick mode hash invalid\n"
 			"!! : %s <-> %s\n",
 			txtaddr_l,
@@ -1070,8 +1177,8 @@ long _IKED::phase2_chk_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	phase2_gen_hash_p( ph1, ph2, hash );
 
 	log.bin(
-		LOG_DEBUG,
-		LOG_DECODE,
+		LLOG_DEBUG,
+		LLOG_DECODE,
 		ph2->hash_r.buff(),
 		ph2->hash_r.size(),
 		"== : phase2 hash_p ( received )" );
@@ -1092,7 +1199,7 @@ long _IKED::phase2_chk_hash_p( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 	if( memcmp( hash.buff(), ph2->hash_r.buff(), ph1->hash_size ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : phase2 sa rejected, initiator liveliness proof hash invalid\n"
 			"!! : %s <-> %s\n",
 			txtaddr_l,
@@ -1160,7 +1267,7 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 		if( !cmp_ph2id( ph2->ph2id_ls, ph2->ph2id_rd, true ) ||
 			!cmp_ph2id( ph2->ph2id_ld, ph2->ph2id_rs, true ) )
 		{
-			log.txt( LOG_ERROR, 
+			log.txt( LLOG_ERROR, 
 				"ii : phase2 rejected, id value mismatch\n"
 				"ii : - loc %s -> %s\n" 
 				"ii : - rmt %s -> %s\n",
@@ -1179,18 +1286,19 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 
 		IDB_POLICY * policy_out;
 
-		if( !get_policy(
+		if( !idb_list_policy.find(
 				true,
 				&policy_out,
 				IPSEC_DIR_OUTBOUND,
 				IPSEC_POLICY_IPSEC,
 				NULL,
+				NULL,
 				&ph2->tunnel->saddr_l,
 				&ph2->tunnel->saddr_r,
 				&ph2->ph2id_ls,
 				&ph2->ph2id_ld ) )
 		{
-			log.txt( LOG_ERROR, 
+			log.txt( LLOG_ERROR, 
 				"ii : phase2 rejected, no matching outbound policy found\n"
 				"ii : - loc %s -> %s\n" 
 				"ii : - rmt %s -> %s\n",
@@ -1209,12 +1317,12 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 		// notifications
 		//
 
-		if( ph2->nlist.count() )
+		if( ph2->notifications.count() )
 		{
 			IKE_NOTIFY notify;
 
 			long nindex = 0;
-			while( ph2->nlist.get( notify, nindex++ ) )
+			while( ph2->notifications.get( notify, nindex++ ) )
 				inform_chk_notify( ph1, &notify, true );
 		}
 
@@ -1245,18 +1353,19 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 		// locate inbound ipsec policy
 		//
 
-		if( !get_policy(
+		if( !idb_list_policy.find(
 				true,
 				&policy_in,
 				IPSEC_DIR_INBOUND,
 				IPSEC_POLICY_IPSEC,
 				NULL,
+				NULL,
 				&ph2->tunnel->saddr_r,
 				&ph2->tunnel->saddr_l,
 				&ph2->ph2id_rs,
 				&ph2->ph2id_rd ) )
 		{
-			log.txt( LOG_ERROR, 
+			log.txt( LLOG_ERROR, 
 				"ii : phase2 rejected, no matching inbound policy found\n"
 				"ii : - loc %s -> %s\n" 
 				"ii : - rmt %s -> %s\n",
@@ -1273,18 +1382,19 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 		// locate outbound ipsec policy
 		//
 
-		if( !get_policy(
+		if( !idb_list_policy.find(
 				true,
 				&policy_out,
 				IPSEC_DIR_OUTBOUND,
 				IPSEC_POLICY_IPSEC,
 				NULL,
+				NULL,
 				&ph2->tunnel->saddr_l,
 				&ph2->tunnel->saddr_r,
 				&ph2->ph2id_rd,
 				&ph2->ph2id_rs ) )
 		{
-			log.txt( LOG_ERROR, 
+			log.txt( LLOG_ERROR, 
 				"ii : phase2 rejected, no matching outbound policy found\n"
 				"ii : - loc %s -> %s\n" 
 				"ii : - rmt %s -> %s\n",
@@ -1305,7 +1415,7 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 
 		if( !policy_cmp_prots( policy_in, policy_out ) )
 		{
-			log.txt( LOG_ERROR, 
+			log.txt( LLOG_ERROR, 
 				"ii : phase2 rejected, inbound / outbound policy mismatch\n"
 				"ii : - loc %s -> %s\n" 
 				"ii : - rmt %s -> %s\n",
@@ -1367,7 +1477,7 @@ long _IKED::phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet
 	// ids accepted
 	//
 
-	log.txt( LOG_INFO,
+	log.txt( LLOG_INFO,
 		"ii : phase2 ids accepted\n"
 		"ii : - loc %s -> %s\n" 
 		"ii : - rmt %s -> %s\n",
@@ -1398,7 +1508,7 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 		if( ph2->xr.size() != ph2->dh_size )
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : dh group size mismatch ( %d != %d )\n",
 				ph2->xr.size(),
 				ph2->dh_size );
@@ -1409,13 +1519,13 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 		BIGNUM * gx = BN_new();
 		BN_bin2bn( ph2->xr.buff(), ph2->dh_size, gx );
 
-		shared.set( 0, ph2->dh_size );
+		shared.size( ph2->dh_size );
 		DH_compute_key( shared.buff(), gx, ph2->dh );
 		BN_free( gx );
 
 		log.bin(
-			LOG_DEBUG,
-			LOG_DECODE,
+			LLOG_DEBUG,
+			LLOG_DECODE,
 			shared.buff(),
 			shared.size(),
 			"== : pfs dh shared secret" );
@@ -1455,9 +1565,9 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 	ph2->inc( true );
 	ph2->inc( true );
 
-	ph2->event_hard.delay = lifetime;
+	ph2->event_hard.delay = lifetime + 1;
 
-	ph2->event_soft.delay = lifetime;
+	ph2->event_soft.delay = lifetime + 1;
 	ph2->event_soft.delay *= PFKEY_SOFT_LIFETIME_RATE;
 	ph2->event_soft.delay /= 100;
 	ph2->event_soft.diff = ph2->event_hard.delay - ph2->event_soft.delay;
@@ -1569,7 +1679,7 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 
 
 	//
-	// resize our key to be a multiple
+	// grow our key to be a multiple
 	// of SKEYID_d ( HMAC block size )
 
 	if( key_size % skeyid_size )
@@ -1649,8 +1759,8 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 		ekey.set( key_data, key_size_c );
 
 		log.bin(
-			LOG_DEBUG,
-			LOG_DECODE,
+			LLOG_DEBUG,
+			LLOG_DECODE,
 			ekey.buff(),
 			ekey.size(),
 			"== : spi cipher key data" );
@@ -1661,8 +1771,8 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 		akey.set( key_data + key_size_c, key_size_h );
 
 		log.bin(
-			LOG_DEBUG,
-			LOG_DECODE,
+			LLOG_DEBUG,
+			LLOG_DECODE,
 			akey.buff(),
 			akey.size(),
 			"== : spi hmac key data" );
@@ -1674,6 +1784,8 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 
 	pfkey_send_update( ph2, proposal, ekey, akey, dir );
 
+	ph2->lstate |= LSTATE_HASKEYS;
+
 	return LIBIKE_OK;
 }
 
@@ -1698,8 +1810,8 @@ long _IKED::phase2_gen_iv( IDB_PH1 * ph1, unsigned long msgid, BDATA & iv )
 		iv.set( iv_data, iv_size );
 
 		log.bin(
-			LOG_DEBUG,
-			LOG_DECODE,
+			LLOG_DEBUG,
+			LLOG_DECODE,
 			iv.buff(),
 			iv.size(),
 			"== : new phase2 iv" );
