@@ -42,7 +42,7 @@
 #ifndef _IKE_H_
 #define _IKE_H_
 
-#include "libip.h"
+#include "libidb.h"
 
 #define CONF_STRLEN					256
 
@@ -118,8 +118,8 @@
 #define ISAKMP_ENCAP_TRANSPORT				2
 #define ISAKMP_ENCAP_RFC_UDP_TUNNEL			3
 #define ISAKMP_ENCAP_RFC_UDP_TRANSPORT		4
-#define ISAKMP_ENCAP_V02_UDP_TUNNEL			61443
-#define ISAKMP_ENCAP_V02_UDP_TRANSPORT		61444
+#define ISAKMP_ENCAP_VXX_UDP_TUNNEL			61443
+#define ISAKMP_ENCAP_VXX_UDP_TRANSPORT		61444
 
 // authentication algorithm values
 
@@ -172,8 +172,8 @@
 #define ISAKMP_PAYLOAD_ATTRIB			14  // mode config attibutes
 #define ISAKMP_PAYLOAD_NAT_RFC_DISC		20  // nat discovery
 #define ISAKMP_PAYLOAD_NAT_RFC_ORIG		21  // nat original address
-#define ISAKMP_PAYLOAD_NAT_V02_DISC		130 // nat discovery
-#define ISAKMP_PAYLOAD_NAT_V02_ORIG		131  // nat original address
+#define ISAKMP_PAYLOAD_NAT_VXX_DISC		130 // nat discovery
+#define ISAKMP_PAYLOAD_NAT_VXX_ORIG		131 // nat original address
 #define ISAKMP_PAYLOAD_FRAGMENT			132	// fragmentation payload
 
 // exchange types
@@ -322,71 +322,41 @@
 // IKE protocol extensions
 //
 
-// ike fragmentation extension
+// ike fragmentation
 
 #define IKE_FRAG_FLAG_LAST			0x01
 
-// dead peer detection extension
+// dead peer detection
 
 #define ISAKMP_N_DPD_R_U_THERE		36136
 #define ISAKMP_N_DPD_R_U_THERE_ACK	36137
 
-// transactional extenstion
+// configuration exchange types
 
 #define ISAKMP_CFG_REQUEST			1
 #define ISAKMP_CFG_REPLY			2
 #define ISAKMP_CFG_SET				3
 #define ISAKMP_CFG_ACK				4
 
-// transactional attributes
+// configuration attributes
 
-#define INTERNAL_IP4_ADDRESS		1	// veriable   0 or 4 octets 
-#define INTERNAL_IP4_NETMASK		2	// veriable   0 or 4 octets
-#define INTERNAL_IP4_DNS			3	// veriable   0 or 4 octets
-#define INTERNAL_IP4_NBNS			4	// veriable   0 or 4 octets
-#define INTERNAL_ADDRESS_EXPIRY		5	// veriable   0 or 4 octets
-#define INTERNAL_IP4_DHCP			6	// veriable   0 or 4 octets
-#define APPLICATION_VERSION			7	// veriable   0 or more 
-#define INTERNAL_IP6_ADDRESS		8	// veriable   0 or 16 octets 
-#define INTERNAL_IP6_NETMASK		9	// veriable   0 or 16 octets 
-#define INTERNAL_IP6_DNS			10	// veriable   0 or 16 octets 
-#define INTERNAL_IP6_NBNS			11	// veriable   0 or 16 octets 
-#define INTERNAL_IP6_DHCP			12	// veriable   0 or 16 octets 
-#define INTERNAL_IP4_SUBNET			13	// veriable   0 or 8 octets 
-#define SUPPORTED_ATTRIBUTES		14	// veriable   0 or multiples of 2 
-#define INTERNAL_IP6_SUBNET			15	// veriable   0 or 17 octets
+#define INTERNAL_IP4_ADDRESS		1	// variable   0 or 4 octets 
+#define INTERNAL_IP4_NETMASK		2	// variable   0 or 4 octets
+#define INTERNAL_IP4_DNS			3	// variable   0 or 4 octets
+#define INTERNAL_IP4_NBNS			4	// variable   0 or 4 octets
+#define INTERNAL_ADDRESS_EXPIRY		5	// variable   0 or 4 octets
+#define INTERNAL_IP4_DHCP			6	// variable   0 or 4 octets
+#define APPLICATION_VERSION			7	// variable   0 or more 
+#define INTERNAL_IP6_ADDRESS		8	// variable   0 or 16 octets 
+#define INTERNAL_IP6_NETMASK		9	// variable   0 or 16 octets 
+#define INTERNAL_IP6_DNS			10	// variable   0 or 16 octets 
+#define INTERNAL_IP6_NBNS			11	// variable   0 or 16 octets 
+#define INTERNAL_IP6_DHCP			12	// variable   0 or 16 octets 
+#define INTERNAL_IP4_SUBNET			13	// variable   0 or 8 octets 
+#define SUPPORTED_ATTRIBUTES		14	// variable   0 or multiples of 2 
+#define INTERNAL_IP6_SUBNET			15	// variable   0 or 17 octets
 
-// unity transactional attributes
-
-#define UNITY_BANNER				28672
-#define UNITY_SAVE_PASSWD			28673
-#define UNITY_DEF_DOMAIN			28674
-#define UNITY_SPLIT_DOMAIN			28675
-#define UNITY_SPLIT_INCLUDE			28676
-#define UNITY_NATT_PORT				28677
-#define UNITY_SPLIT_EXCLUDE			28678
-#define UNITY_PFS					28679
-#define UNITY_FW_TYPE				28680
-#define UNITY_BACKUP_SERVERS		28681
-#define UNITY_DDNS_HOSTNAME			28682
-
-// xauth extension
-
-#define XAUTH_TYPE					16520	// basic
-#define XAUTH_USER_NAME				16521	// variable
-#define XAUTH_USER_PASSWORD			16522	// variable
-#define XAUTH_PASSCODE				16523	// variable
-#define XAUTH_MESSAGE				16524	// variable
-#define XAUTH_CHALLENGE				16525	// variable
-#define XAUTH_DOMAIN				16526	// variable
-#define XAUTH_STATUS				16527	// basic
-#define XAUTH_NEXT_PIN				16528	// variable
-#define XAUTH_ANSWER				16529	// variable
-
-#define XAUTH_TYPE_GENERIC			0
-#define XAUTH_TYPE_RADIUS_CHAP		1
-#define XAUTH_TYPE_OTP				2
-#define XAUTH_TYPE_SKEY				3
+// extended authentication modes
 
 #define XAUTH_AUTH_INIT_PSK			65001
 #define XAUTH_AUTH_RESP_PSK			65002
@@ -399,25 +369,75 @@
 #define XAUTH_AUTH_INIT_RSA_REV		65009
 #define XAUTH_AUTH_RESP_RSA_REV		65010
 
-// hybrid extension
+// hybrid authentication modes
 
 #define HYBRID_AUTH_INIT_RSA		64221
 #define HYBRID_AUTH_RESP_RSA		64222
 #define HYBRID_AUTH_INIT_DSS		64223
 #define HYBRID_AUTH_RESP_DSS		64224
 
+// extended authentication types
+
+#define XAUTH_TYPE_GENERIC			0
+#define XAUTH_TYPE_RADIUS_CHAP		1
+#define XAUTH_TYPE_OTP				2
+#define XAUTH_TYPE_SKEY				3
+
+// extended authentication attributes
+
+#define XAUTH_TYPE					16520	// basic
+#define XAUTH_USER_NAME				16521	// variable
+#define XAUTH_USER_PASSWORD			16522	// variable
+#define XAUTH_PASSCODE				16523	// variable
+#define XAUTH_MESSAGE				16524	// variable
+#define XAUTH_CHALLENGE				16525	// variable
+#define XAUTH_DOMAIN				16526	// variable
+#define XAUTH_STATUS				16527	// basic
+#define XAUTH_NEXT_PIN				16528	// variable
+#define XAUTH_ANSWER				16529	// variable
+
+// unity configuration attributes
+
+#define UNITY_BANNER				28672	// variable
+#define UNITY_SAVE_PASSWD			28673	// basic
+#define UNITY_DEF_DOMAIN			28674	// variable
+#define UNITY_SPLIT_DOMAIN			28675	// variable
+#define UNITY_SPLIT_INCLUDE			28676	// variable
+#define UNITY_NATT_PORT				28677	// basic
+#define UNITY_SPLIT_EXCLUDE			28678	// variable
+#define UNITY_PFS					28679	// basic
+#define UNITY_FW_TYPE				28680	// variable
+#define UNITY_BACKUP_SERVERS		28681	// variable
+#define UNITY_DDNS_HOSTNAME			28682	// variable
+
+// checkpoint extended authentication attributes
+
+#define	CHKPT_TYPE					13
+#define CHKPT_USER_NAME				14
+#define CHKPT_USER_PASSWORD			15
+#define CHKPT_MESSAGE				17
+#define CHKPT_CHALLENGE				18
+#define CHKPT_STATUS				20
+
+// checkpoint configuration attributes
+
+#define CHKPT_DEF_DOMAIN			16387
+#define CHKPT_MAC_ADDRESS			16388
+#define CHKPT_MARCIPAN_REASON_CODE	16389
+#define CHKPT_UNKNOWN1				16400
+#define CHKPT_UNKNOWN2				16401
+#define CHKPT_UNKNOWN3				16402
+
 //
 // implementation specific constants
 //
 
-#define ISAKMP_HEADER_SIZE				28
 #define ISAKMP_COOKIE_SIZE				8
 #define ISAKMP_SPI_SIZE					4
 #define ISAKMP_CPI_SIZE					2
 #define ISAKMP_NONCE_SIZE				20
 #define ISAKMP_NONCE_MIN				4
 #define ISAKMP_NONCE_MAX				252
-#define ISAKMP_VEND_MAX					16
 
 #define ISAKMP_CERT_MAX					( 1024 * 32 )
 #define ISAKMP_CREQ_MAX					( 1024 * 4 )
@@ -434,7 +454,8 @@
 
 #define IPSEC_NATT_DISABLE		0
 #define IPSEC_NATT_ENABLE		1
-#define IPSEC_NATT_FORCE		2
+#define IPSEC_NATT_FORCE_DRAFT	2
+#define IPSEC_NATT_FORCE_RFC	3
 
 #define IPSEC_FRAG_DISABLE		0
 #define IPSEC_FRAG_ENABLE		1
@@ -445,7 +466,10 @@
 #define IPSEC_DPD_FORCE			2
 
 #define IPSEC_NATT_NONE			0
-#define IPSEC_NATT_V02			2
+#define IPSEC_NATT_V00			1
+#define IPSEC_NATT_V01			2
+#define IPSEC_NATT_V02			3
+#define IPSEC_NATT_V03			4
 #define IPSEC_NATT_RFC			255
 
 #define IPSEC_OPTS_SPLITNET		0x0001
@@ -459,8 +483,13 @@
 #define IPSEC_OPTS_SPLITDNS		0x0100
 #define IPSEC_OPTS_SAVEPW		0x0200
 
+#define IPSEC_DNSS_MAX			4
+#define IPSEC_NBNS_MAX			2
+
+#define CONFIG_MODE_NONE		0
 #define CONFIG_MODE_PULL		1
 #define CONFIG_MODE_PUSH		2
+#define CONFIG_MODE_DHCP		3
 
 #define POLICY_MODE_DISABLE		0
 #define POLICY_MODE_COMPAT		1
@@ -480,7 +509,7 @@ typedef struct _IKE_SADDR
 
 }IKE_SADDR;
 
-typedef struct _IKE_ATTR
+typedef struct _IKE_ATTR : public IDB_ENTRY
 {
 	bool		basic;
 	uint16_t	atype;
@@ -557,6 +586,7 @@ typedef struct _IKE_PEER
 	unsigned char	contact;
 	unsigned char	exchange;
 	bool			notify;
+	bool			nailed;
 	long			natt_mode;
 	unsigned short	natt_port;
 	long			natt_rate;
@@ -565,7 +595,8 @@ typedef struct _IKE_PEER
 	long			frag_esp_mode;
 	size_t			frag_esp_size;
 	long			dpd_mode;
-	long			dpd_rate;
+	long			dpd_delay;
+	long			dpd_retry;
 	long			life_check;
 
 	unsigned char	idtype_l;
@@ -585,19 +616,37 @@ typedef struct _IKE_XAUTH
 
 }IKE_XAUTH;
 
+typedef struct _IKE_NSCFG
+{
+	bool		dhcp_dnss;						// dns server list from dhcp
+	in_addr		dnss_list[ IPSEC_DNSS_MAX ];	// dns server list
+	uint32_t	dnss_count;						// dns server count
+	uint32_t	dnss_nodyn;						// dns dynamic update disabled
+	char		dnss_suffix[ CONF_STRLEN ];
+
+	in_addr		nbns_list[ IPSEC_NBNS_MAX ];	// wins server list
+	uint32_t	nbns_count;						// wins server count
+	uint32_t	nbns_nopts;						// wins options
+	uint32_t	nbns_ntype;						// wins node type
+
+}IKE_NSCFG;
+
+
 typedef struct _IKE_XCONF
 {
-	long		opts;	// enabled options
-	long		rqst;	// request options
+	long		opts;		// enabled options
+	long		rqst;		// request options
 
-	in_addr			addr;	// network address
-	in_addr			mask;	// network mask
-	in_addr			dnss;	// dns server
-	in_addr			nbns;	// wins server
-	unsigned short	dhgr;	// pfs dh group
-	unsigned short	svpw;	// save password
+	in_addr		addr;		// network address
+	in_addr		mask;		// network mask
+	in_addr		dhcp;		// dhcp servers
 
-	char		suffix[ CONF_STRLEN ];
+	uint32_t	expi;		// address expires
+	uint32_t	vmtu;		// adapter mtu
+	uint16_t	dhgr;		// pfs dh group
+	uint16_t	svpw;		// save password
+
+	IKE_NSCFG	nscfg;		// name service config
 
 }IKE_XCONF;
 
@@ -608,12 +657,12 @@ typedef struct _IKE_NOTIFY
 	uint8_t		proto;
 	uint16_t	code;
 
-	IKE_SPI			spi;
-	BDATA			data;
+	IKE_SPI		spi;
+	BDATA		data;
 
 }IKE_NOTIFY;
 
-typedef struct _IKE_FRAG
+typedef struct _IKE_FRAG : public IDB_ENTRY
 {
 	public:
 
@@ -625,14 +674,102 @@ typedef struct _IKE_FRAG
 
 #pragma pack( 1 )
 
-typedef struct _IKE_UNITY_NET
+typedef struct _IKE_SUBNET
 {
 	in_addr addr;
 	in_addr mask;
-	char	padd[ 6 ];
+
+}IKE_SUBNET;
+
+typedef struct _IKE_UNITY_NET
+{
+	in_addr		addr;
+	in_addr		mask;
+	uint8_t		prot;
+	uint8_t		pad;
+	uint16_t	port_loc;
+	uint16_t	port_rmt;
 
 }IKE_UNITY_NET;
 
 #pragma pack()
 
+//
+// XXX : these need to move back
+//       into iked.h
+//
+
+#pragma pack( 1 )
+
+typedef struct _IKE_HEADER
+{
+	IKE_COOKIES	cookies;
+
+	uint8_t		payload;	// initial payload
+	uint8_t		version;	// isakmp version
+	uint8_t		exchange;	// exchange type
+	uint8_t		flags;		// flags
+	uint32_t	msgid;		// message id
+	uint32_t	length;		// message length
+
+}IKE_HEADER;
+
+typedef struct _IKE_PAYLOAD
+{
+	uint8_t		next;		// next payload
+	uint8_t		reserved;	// reserved
+	uint16_t	length;		// payload size
+
+}IKE_PAYLOAD;
+
+#pragma pack()
+
+typedef struct _PLD_DATA
+{
+	size_t	oset;
+	size_t	size;
+
+}PLD_DATA;
+
+typedef class _PACKET_IKE : public _PACKET
+{
+	protected:
+
+	PLD_DATA	pld_stack[ 8 ];
+	long		pld_depth;
+
+	IKE_HEADER	header;
+
+	public:
+
+	unsigned char	notify;
+
+	_PACKET_IKE();
+	~_PACKET_IKE();
+
+	void	reset();
+
+	void	set_msgid( uint32_t msgid );
+	void	get_msgid( uint32_t & msgid );
+
+	bool	add_payload( bool encap, uint8_t next );
+	bool	get_payload( bool encap, uint8_t & next );
+	void	end_payload( bool decap, bool write = true );
+
+	size_t	get_payload_left();
+
+	bool	write( IKE_COOKIES & cookies,
+					uint8_t payload,
+					uint8_t exchange,
+					uint8_t flags );
+
+	bool	read( IKE_COOKIES & cookies,
+					uint8_t & payload,
+					uint8_t & exchange,
+					uint8_t & flags );
+
+	bool	done();
+
+}PACKET_IKE;
+
 #endif
