@@ -41,109 +41,80 @@
 
 #include "iked.h"
 
-//
-// phase1 event functions
-//
+//==============================================================================
+// ike phase1 exchange events
+//==============================================================================
 
-bool _ITH_EVENT_PH1DPD::func()
+bool _ITH_EVENT_PH1SOFT::func()
 {
-	long diff = ph1->dpd_req - ph1->dpd_res;
+	iked.log.txt( LLOG_INFO,
+		"ii : phase1 sa is expiring\n"
+		"ii : %04x%04x:%04x%04x\n",
+		htonl( *( long * ) &ph1->cookies.i[ 0 ] ),
+		htonl( *( long * ) &ph1->cookies.i[ 4 ] ),
+		htonl( *( long * ) &ph1->cookies.r[ 0 ] ),
+		htonl( *( long * ) &ph1->cookies.r[ 4 ] ) );
 
-	if( diff >= 2 )
+	//
+	// if the tunnel peer definition states
+	// that we are to act as a client, replace
+	// this phase1 sa and add it to our list
+	//
+
+	if( ph1->tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
 	{
-		iked.log.txt( LOG_ERROR,
-				"!! : phase1 sa dpd timeout\n"
-				"!! : %04x%04x:%04x%04x\n",
-				htonl( *( long * ) &ph1->cookies.i[ 0 ] ),
-				htonl( *( long * ) &ph1->cookies.i[ 4 ] ),
-				htonl( *( long * ) &ph1->cookies.r[ 0 ] ),
-				htonl( *( long * ) &ph1->cookies.r[ 4 ] ) );
+		if( ph1->vendopts_l.flag.xauth )
+		{
+			ph1->tunnel->tstate &= ~TSTATE_SENT_XAUTH;
+			ph1->tunnel->tstate &= ~TSTATE_RECV_XAUTH;
+			ph1->tunnel->tstate &= ~TSTATE_SENT_XRSLT;
+			ph1->tunnel->tstate &= ~TSTATE_RECV_XRSLT;
+		}
 
-		if( ph1->tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
-			ph1->tunnel->close = TERM_PEER_DEAD;
+		if( ph1->tunnel->peer->xconf_mode == CONFIG_MODE_PUSH )
+		{
+			ph1->tunnel->tstate &= ~TSTATE_RECV_CONFIG;
+			ph1->tunnel->tstate &= ~TSTATE_SENT_CONFIG;
+			ph1->tunnel->tstate &= ~TSTATE_RECV_ACK;
+			ph1->tunnel->tstate &= ~TSTATE_SENT_ACK;
+		}
 
-		ph1->lstate |= ( LSTATE_EXPIRE | LSTATE_NOTIFY | LSTATE_DELETE );
-		ph1->dec( true );
-
-		return false;
+		IDB_PH1 * addph1 = new IDB_PH1( ph1->tunnel, true, NULL );
+		addph1->add( false );
+		iked.process_phase1_send( addph1 );
+		addph1->dec( false );
 	}
 
-	//
-	// obtain next sequence number
-	// and convert to network byte
-	// order
-	//
-
-	uint32_t dpdseq = htonl( ++ph1->dpd_req );
+	ph1->status( XCH_STATUS_EXPIRING, XCH_NORMAL, 0 );
+	ph1->dec( true );
 
 	//
-	// add sequence number and send
+	// FIXME : dpd and natt events should stop for
+	//         this sa after the new sa negotiates
 	//
 
-	BDATA bdata;
-	bdata.add( &dpdseq, sizeof( dpdseq ) );
-
-	iked.inform_new_notify( ph1, NULL, ISAKMP_N_DPD_R_U_THERE, &bdata );
-
-	iked.log.txt( LOG_DEBUG, "ii : DPD ARE-YOU-THERE sequence %08x requested\n", ph1->dpd_req );
-
-	return true;
-}
-
-bool _ITH_EVENT_PH1NATT::func()
-{
-	//
-	// encapsulate natt keep alive
-	//
-
-	PACKET_UDP packet_udp;
-
-	packet_udp.write(
-		ph1->tunnel->saddr_l.saddr4.sin_port,
-		ph1->tunnel->saddr_r.saddr4.sin_port );
-
-	packet_udp.add_byte( 0xff );
-
-	packet_udp.done(
-		ph1->tunnel->saddr_l.saddr4.sin_addr,
-		ph1->tunnel->saddr_r.saddr4.sin_addr );
-
-	PACKET_IP packet_ip;
-
-	packet_ip.write(
-		ph1->tunnel->saddr_l.saddr4.sin_addr,
-		ph1->tunnel->saddr_r.saddr4.sin_addr,
-		iked.ident++,
-		PROTO_IP_UDP );
-
-	packet_ip.add( packet_udp );
-
-	packet_ip.done();
-
-	//
-	// send ike packet
-	//
-
-	char txtaddr_l[ LIBIKE_MAX_TEXTADDR ];
-	char txtaddr_r[ LIBIKE_MAX_TEXTADDR ];
-
-	iked.text_addr( txtaddr_l, &ph1->tunnel->saddr_l, true );
-	iked.text_addr( txtaddr_r, &ph1->tunnel->saddr_r, true );
-
-	iked.log.txt( LOG_DEBUG,
-		"-> : send NAT-T:KEEP-ALIVE packet %s -> %s\n",
-		txtaddr_l, 
-		txtaddr_r );
-
-	iked.send_ip(
-		packet_ip );
-
-	return true;
+	return false;
 }
 
 bool _ITH_EVENT_PH1HARD::func()
 {
-	iked.log.txt( LOG_INFO,
+	iked.log.txt( LLOG_INFO,
+		"ii : phase1 sa is expired\n"
+		"ii : %04x%04x:%04x%04x\n",
+		htonl( *( long * ) &ph1->cookies.i[ 0 ] ),
+		htonl( *( long * ) &ph1->cookies.i[ 4 ] ),
+		htonl( *( long * ) &ph1->cookies.r[ 0 ] ),
+		htonl( *( long * ) &ph1->cookies.r[ 4 ] ) );
+
+	ph1->status( XCH_STATUS_EXPIRED, XCH_FAILED_EXPIRED, 0 );
+	ph1->dec( true );
+
+	return false;
+}
+
+bool _ITH_EVENT_PH1DEAD::func()
+{
+	iked.log.txt( LLOG_INFO,
 		"ii : phase1 sa is dead\n"
 		"ii : %04x%04x:%04x%04x\n",
 		htonl( *( long * ) &ph1->cookies.i[ 0 ] ),
@@ -151,18 +122,167 @@ bool _ITH_EVENT_PH1HARD::func()
 		htonl( *( long * ) &ph1->cookies.r[ 0 ] ),
 		htonl( *( long * ) &ph1->cookies.r[ 4 ] ) );
 
-	if( ph1->tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
-		ph1->tunnel->close = TERM_EXPIRE;
-
-	ph1->lstate |= ( LSTATE_EXPIRE | LSTATE_NOTIFY | LSTATE_DELETE );
+	ph1->status( XCH_STATUS_DEAD, XCH_FAILED_EXPIRED, 0 );
 	ph1->dec( true );
 
 	return false;
 }
 
-//
-// phase1 security association class
-//
+//==============================================================================
+// ike phase1 exchange handle list
+//==============================================================================
+
+IDB_PH1 * _IDB_LIST_PH1::get( int index )
+{
+	return static_cast<IDB_PH1*>( get_entry( index ) );
+}
+
+bool _IDB_LIST_PH1::find( bool lock, IDB_PH1 ** ph1, IDB_TUNNEL * tunnel, XCH_STATUS min, XCH_STATUS max, IKE_COOKIES * cookies )
+{
+	if( ph1 != NULL )
+		*ph1 = NULL;
+
+	if( lock )
+		iked.lock_idb.lock();
+
+	//
+	// step through our list of sa's
+	// and locate a match
+	//
+
+	long ph1_count = count();
+	long ph1_index = 0;
+
+	for( ; ph1_index < ph1_count; ph1_index++ )
+	{
+		//
+		// get the next sa in our list
+		//
+
+		IDB_PH1 * tmp_ph1 = get( ph1_index );
+
+		//
+		// match sa minimum status level
+		//
+
+		if( min != XCH_STATUS_ANY )
+			if( tmp_ph1->status() < min )
+				continue;
+
+		//
+		// match sa maximum status level
+		//
+
+		if( max != XCH_STATUS_ANY )
+			if( tmp_ph1->status() > max )
+				continue;
+
+		//
+		// match the tunnel id
+		//
+
+		if( tunnel != NULL )
+			if( tmp_ph1->tunnel != tunnel )
+				continue;
+
+		//
+		// match the cookies
+		//
+
+		if( cookies != NULL )
+		{
+			//
+			// next match the initiator cookie
+			//
+
+			if( memcmp( tmp_ph1->cookies.i, cookies->i, ISAKMP_COOKIE_SIZE ) )
+			{
+				//
+				// the initiator cookie should
+				// always match if we are to
+				// return a known sa
+				//
+
+				continue;
+			}
+
+			//
+			// next match the responder cookie
+			//
+
+			if( memcmp( tmp_ph1->cookies.r, cookies->r, ISAKMP_COOKIE_SIZE ) )
+			{
+				//
+				// responder cookie did not match,
+				// if we are the intiator for this
+				// sa, the responder cookie is null
+				// and we are waiting on an sa
+				// payload, it should match
+				//
+
+				if( tmp_ph1->initiator )
+				{
+					//
+					// check to see if we solicited
+					// a response from this host
+					//
+
+					if( !( tmp_ph1->xstate & XSTATE_SENT_SA ) ||
+						 ( tmp_ph1->xstate & XSTATE_RECV_SA ) )
+						 continue;
+
+					//
+					// check the responder cookie
+					// for a null value
+					//
+
+					for( long x = 0; x < ISAKMP_COOKIE_SIZE; x++ )
+						if( tmp_ph1->cookies.r[ x ] )
+							continue;
+
+					//
+					// store the responders cookie in
+					// our existing sa
+					//
+
+					memcpy( tmp_ph1->cookies.r, cookies->r, ISAKMP_COOKIE_SIZE );
+				}
+			}
+		}
+
+		//
+		// looks like we found a match
+		//
+
+		iked.log.txt( LLOG_DEBUG, "DB : phase1 found\n" );
+
+		//
+		// increase our refrence count
+		//
+
+		if( ph1 != NULL )
+		{
+			tmp_ph1->inc( false );
+			*ph1 = tmp_ph1;
+		}
+
+		if( lock )
+			iked.lock_idb.unlock();
+
+		return true;
+	}
+
+	iked.log.txt( LLOG_DEBUG, "DB : phase1 not found\n" );
+
+	if( lock )
+		iked.lock_idb.unlock();
+
+	return false;
+}
+
+//==============================================================================
+// ike phase1 exchange handle list entry
+//==============================================================================
 
 _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * set_cookies )
 {
@@ -171,23 +291,11 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 
 	memset( &cookies, 0, sizeof( cookies ) );
 
-	xauth_l = false;
-	xauth_r = false;
+	vendopts_l.flags = 0;
+	vendopts_r.flags = 0;
 
-	unity_l = true;		// always true for now
-	unity_r = false;
-
-	natt_l = false;
-	natt_r = false;
-
-	dpd_l = false;
-	dpd_r = false;
-
-	dpd_req = 0;
-	dpd_res = 0;
-
-	frag_l = false;
-	frag_r = false;
+	natt_version = IPSEC_NATT_NONE;
+	natt_pldtype = ISAKMP_PAYLOAD_NONE;
 
 	natted_l = false;
 	natted_r = false;
@@ -202,6 +310,7 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	//
 
 	tunnel = set_tunnel;
+	tunnel->inc( true );
 
 	//
 	// initialize initiator value
@@ -245,60 +354,71 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	// initialize nonce data
 	//
 
-	nonce_l.set( 0, ISAKMP_NONCE_SIZE );
+	nonce_l.size( ISAKMP_NONCE_SIZE );
 	iked.rand_bytes( nonce_l.buff(), ISAKMP_NONCE_SIZE );
 
+	//
+	// always advertise as shrew soft
+	//
+
+	vendopts_l.flag.ssoft = true;
+
+	//
+	// determine if this is a client tunnel
+	//
+
+	if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
+	{
+		vendopts_l.flag.unity = true;
+		vendopts_l.flag.netsc = true;
+		vendopts_l.flag.zwall = true;
+		vendopts_l.flag.swind = true;
+		vendopts_l.flag.chkpt = true;
+	}
+
 	//
 	// determine ike fragmentation negotiation
 	//
 
 	if( tunnel->peer->frag_ike_mode >= IPSEC_FRAG_ENABLE )
-		frag_l = true;
+		vendopts_l.flag.frag = true;
 
 	if( tunnel->peer->frag_ike_mode == IPSEC_FRAG_FORCE )
-		frag_r = true;
+		vendopts_r.flag.frag = true;
 
 	//
 	// determine natt negotiation
 	//
 
 	if( tunnel->peer->natt_mode >= IPSEC_NATT_ENABLE )
-		natt_l = true;
+		vendopts_l.flag.natt = true;
 
 	//
 	// determine dpd negotiation
 	//
 
 	if( tunnel->peer->dpd_mode >= IPSEC_DPD_ENABLE )
-	{
-		dpd_l = true;
-
-		long dpdseq;
-		iked.rand_bytes( &dpdseq, sizeof( dpdseq ) );
-		dpdseq >>= 2;
-
-		dpd_req = dpdseq;
-		dpd_res = dpdseq;
-	}
+		vendopts_l.flag.dpdv1 = true;
 
 	if( tunnel->peer->dpd_mode == IPSEC_DPD_FORCE )
-		dpd_r = true;
+		vendopts_r.flag.dpdv1 = true;
 
 	//
 	// locate the first isakmp proposal
 	//
 
 	IKE_PROPOSAL * proposal;
-	tunnel->peer->prop_list.get( &proposal, 0, ISAKMP_PROTO_ISAKMP );
+	tunnel->peer->proposals.get( &proposal, 0, ISAKMP_PROTO_ISAKMP );
 
 	//
-	// if we are the initiator, preset
-	// our isakmp proposal to the first
-	// entry in our peer proposal list
+	// if we are the initiator, obtain
+	// the authentication type
 	//
 
 	if( initiator )
 	{
+		auth_id = proposal->auth_id;
+
 		//
 		// determine xauth negotiation
 		//
@@ -306,14 +426,13 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 		if( ( proposal->auth_id == XAUTH_AUTH_INIT_PSK ) ||
 			( proposal->auth_id == XAUTH_AUTH_INIT_RSA ) ||
 			( proposal->auth_id == HYBRID_AUTH_INIT_RSA ) )
-			xauth_l = true;
+			vendopts_l.flag.xauth = true;
 	}
 
 	//
-	// aggressive mode must include a
-	// key exchange payload in the
-	// first packet. make sure we setup
-	// the dhgroup in advance
+	// aggressive mode must include a kex
+	// payload in the first packet. make
+	// sure we setup the dhgroup in advance
 	//
 
 	if( tunnel->peer->exchange == ISAKMP_EXCH_AGGRESSIVE )
@@ -323,9 +442,9 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	// initialize event info
 	//
 
-	event_dpd.ph1 = this;
-	event_natt.ph1 = this;
+	event_soft.ph1 = this;
 	event_hard.ph1 = this;
+	event_dead.ph1 = this;
 
 	//
 	// build text strings for logging
@@ -341,7 +460,7 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	// phase 1 created
 	//
 
-	iked.log.txt( LOG_DEBUG,
+	iked.log.txt( LLOG_DEBUG,
 		"DB : new phase1 ( ISAKMP %s )\n"
 		"DB : exchange type is %s\n"
 		"DB : %s <-> %s\n"
@@ -359,10 +478,164 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 _IDB_PH1::~_IDB_PH1()
 {
 	clean();
+
+	//
+	// derefrence our tunnel
+	//
+
+	tunnel->dec( false );
 }
 
+//------------------------------------------------------------------------------
+// abstract functions from parent class
 //
-//
+
+char * _IDB_PH1::name()
+{
+	static char * xname = "phase1";
+	return xname;
+}
+
+IDB_RC_LIST * _IDB_PH1::list()
+{
+	return &iked.idb_list_ph1;
+}
+
+void _IDB_PH1::beg()
+{
+}
+
+void _IDB_PH1::end()
+{
+	//
+	// clear the resend queue
+	//
+
+	resend_clear( false );
+
+	//
+	// remove scheduled events
+	//
+
+	if( iked.ith_timer.del( &event_soft ) )
+	{
+		idb_refcount--;
+		iked.log.txt( LLOG_DEBUG,
+			"DB : phase1 soft event canceled ( ref count = %i )\n",
+			idb_refcount );
+	}
+
+	if( iked.ith_timer.del( &event_hard ) )
+	{
+		idb_refcount--;
+		iked.log.txt( LLOG_DEBUG,
+			"DB : phase1 hard event canceled ( ref count = %i )\n",
+			idb_refcount );
+	}
+
+	if( iked.ith_timer.del( &event_dead ) )
+	{
+		idb_refcount--;
+		iked.log.txt( LLOG_DEBUG,
+			"DB : phase1 dead event canceled ( ref count = %i )\n",
+			idb_refcount );
+	}
+
+	//
+	// send a delete message if required
+	//
+
+	if( ( lstate & LSTATE_HASKEYS ) &&
+		( xch_errorcode != XCH_FAILED_EXPIRED ) &&
+		( xch_errorcode != XCH_FAILED_PEER_DELETE ) )
+		iked.inform_new_delete( this, NULL );
+
+	//
+	// if this sa never reached maturity,
+	// locate any pending phase2 handles
+	// for this tunnel and delete them
+	//
+	// FIXME : This should be timer driven
+	//
+
+	if( !( lstate & LSTATE_HASKEYS ) )
+	{
+		//
+		// FIXME : Use find here
+		//
+
+		long ph2_count = iked.idb_list_ph2.count();
+		long ph2_index = 0;
+
+		for( ; ph2_index < ph2_count; ph2_index++ )
+		{
+			//
+			// get the next phase2 in our list
+			// and attempt to match tunnel ids
+			// 
+
+			IDB_PH2 * ph2 = iked.idb_list_ph2.get( ph2_index );
+			if( ( ph2->tunnel == tunnel ) && ( ph2->status() == XCH_STATUS_PENDING ) )
+			{
+				ph2->inc( false );
+
+				ph2->status( XCH_STATUS_DEAD, XCH_FAILED_PENDING, 0 );
+
+				if( ph2->dec( false ) )
+				{
+					ph2_index--;
+					ph2_count--;
+				}
+			}
+		}
+	}
+
+	//
+	// log deletion
+	//
+
+	if( xch_errorcode != XCH_FAILED_EXPIRED )
+		iked.log.txt( LLOG_INFO, "ii : phase1 removal before expire time\n" );
+	else
+		iked.log.txt( LLOG_INFO, "ii : phase1 removal after expire time\n" );
+
+	//
+	// if we have negotiated a replacement
+	// isakmp sa, change our delete status
+	// to expired. some gateways will send
+	// a delete message for the old sa and
+	// we don't want to treat this as an
+	// error condition below.
+	//
+
+	if( xch_errorcode == XCH_FAILED_PEER_DELETE )
+		if( iked.idb_list_ph1.find(
+				false,
+				NULL,
+				tunnel,
+				XCH_STATUS_MATURE,
+				XCH_STATUS_MATURE,
+				NULL ) )
+			xch_errorcode = XCH_FAILED_EXPIRED;
+
+	//
+	// if this is a client tunnel and there
+	// was an error negotiating phase1, set
+	// a close error message
+	//
+
+	if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
+	{
+		if( xch_errorcode != XCH_FAILED_EXPIRED )
+		{
+			tunnel->close = xch_errorcode;
+			tunnel->ikei->wakeup();
+		}
+	}
+}
+
+//------------------------------------------------------------------------------
+// additional functions
 //
 
 bool _IDB_PH1::setup_dhgrp( IKE_PROPOSAL * proposal )
@@ -375,7 +648,7 @@ bool _IDB_PH1::setup_dhgrp( IKE_PROPOSAL * proposal )
 	dh_size = BN_num_bytes( dh->p );
 	dh_create_e( dh, dh_size );
 
-	xl.set( 0, dh_size );
+	xl.size( dh_size );
 	BN_bn2bin( dh->pub_key, xl.buff() );
 
 	return true;
@@ -497,10 +770,7 @@ void _IDB_PH1::clean()
 	natd_rs.del( true );
 	natd_rd.del( true );
 
-	cert_r.del( true );
 	sign_r.del( true );
-
-	resend_clear();
 }
 
 bool _IDB_PH1::frag_add( unsigned char * data, unsigned long size, long index, bool last )
@@ -525,7 +795,7 @@ bool _IDB_PH1::frag_add( unsigned char * data, unsigned long size, long index, b
 	// add the fragment to our list
 	//
 
-	return frags.add_item( frag );
+	return frags.add_entry( frag );
 }
 
 bool _IDB_PH1::frag_get( PACKET_IKE & packet )
@@ -545,12 +815,12 @@ bool _IDB_PH1::frag_get( PACKET_IKE & packet )
 		// and look for the next index
 		//
 
-		int count = frags.get_count();
+		int count = frags.count();
 		int index = 0;
 
 		for( ; index < count; index++ )
 		{
-			IKE_FRAG * frag = ( IKE_FRAG * ) frags.get_item( index );
+			IKE_FRAG * frag = static_cast<IKE_FRAG*>( frags.get_entry( index ) );
 
 			//
 			// does this match our next index
@@ -605,12 +875,12 @@ bool _IDB_PH1::frag_get( PACKET_IKE & packet )
 		// and look for the next index
 		//
 
-		int count = frags.get_count();
+		int count = frags.count();
 		int index = 0;
 
 		for( ; index < count; index++ )
 		{
-			IKE_FRAG * frag = ( IKE_FRAG * ) frags.get_item( index );
+			IKE_FRAG * frag = static_cast<IKE_FRAG*>( frags.get_entry( index ) );
 
 			//
 			// does this match our next index
@@ -649,358 +919,8 @@ bool _IDB_PH1::frag_get( PACKET_IKE & packet )
 	// purge our fragment list
 	//
 
-	while( frags.get_count() )
-	{
-		IKE_FRAG * frag = ( IKE_FRAG * ) frags.get_item( 0 );
-		frags.del_item( frag );
-		delete frag;
-	}
+	frags.clean();
 
 	return true;
 }
 
-bool _IKED::get_phase1( bool lock, IDB_PH1 ** ph1, IDB_TUNNEL * tunnel, long lstate, long nolstate, IKE_COOKIES * cookies )
-{
-	if( ph1 != NULL )
-		*ph1 = NULL;
-
-	if( lock )
-		lock_sdb.lock();
-
-	//
-	// step through our list of sa's
-	// and locate a match
-	//
-
-	long count = list_phase1.get_count();
-	long index = 0;
-
-	for( ; index < count; index++ )
-	{
-		//
-		// get the next sa in our list
-		//
-
-		IDB_PH1 * tmp_ph1 = ( IDB_PH1 * ) list_phase1.get_item( index );
-
-		//
-		// match sa mature state
-		//
-
-		if( lstate )
-			if( !( tmp_ph1->lstate & lstate ) )
-				continue;
-
-		//
-		// match sa not state flags
-		//
-
-		if( nolstate )
-			if( tmp_ph1->lstate & nolstate )
-				continue;
-
-		//
-		// match the tunnel id
-		//
-
-		if( tunnel != NULL )
-			if( tmp_ph1->tunnel != tunnel )
-				continue;
-
-		//
-		// match the cookies
-		//
-
-		if( cookies != NULL )
-		{
-			//
-			// next match the initiator cookie
-			//
-
-			if( memcmp( tmp_ph1->cookies.i, cookies->i, ISAKMP_COOKIE_SIZE ) )
-			{
-				//
-				// the initiator cookie should
-				// always match if we are to
-				// return a known sa
-				//
-
-				continue;
-			}
-
-			//
-			// next match the responder cookie
-			//
-
-			if( memcmp( tmp_ph1->cookies.r, cookies->r, ISAKMP_COOKIE_SIZE ) )
-			{
-				//
-				// responder cookie did not match,
-				// if we are the intiator for this
-				// sa, the responder cookie is null
-				// and we are waiting on an sa
-				// payload, it should match
-				//
-
-				if( tmp_ph1->initiator )
-				{
-					//
-					// check to see if we solicited
-					// a response from this host
-					//
-
-					if( !( tmp_ph1->xstate & XSTATE_SENT_SA ) ||
-						 ( tmp_ph1->xstate & XSTATE_RECV_SA ) )
-						 continue;
-
-					//
-					// check the responder cookie
-					// for a null value
-					//
-
-					for( long x = 0; x < ISAKMP_COOKIE_SIZE; x++ )
-						if( tmp_ph1->cookies.r[ x ] )
-							continue;
-
-					//
-					// store the responders cookie in
-					// our existing sa
-					//
-
-					memcpy( tmp_ph1->cookies.r, cookies->r, ISAKMP_COOKIE_SIZE );
-				}
-			}
-		}
-
-		//
-		// looks like we found a match
-		//
-
-		log.txt( LOG_DEBUG, "DB : phase1 found\n" );
-
-		//
-		// increase our refrence count
-		//
-
-		if( ph1 != NULL )
-		{
-			tmp_ph1->inc( false );
-			*ph1 = tmp_ph1;
-		}
-
-		if( lock )
-			lock_sdb.unlock();
-
-		return true;
-	}
-
-	log.txt( LOG_DEBUG, "DB : phase1 not found\n" );
-
-	if( lock )
-		lock_sdb.unlock();
-
-	return false;
-}
-
-bool _IDB_PH1::add( bool lock )
-{
-	if( lock )
-		iked.lock_sdb.lock();
-
-	inc( false );
-	tunnel->inc( false );
-
-	bool result = iked.list_phase1.add_item( this );
-
-	iked.log.txt( LOG_DEBUG, "DB : phase1 added\n" );
-
-	if( lock )
-		iked.lock_sdb.unlock();
-
-	return result;
-}
-
-bool _IDB_PH1::inc( bool lock )
-{
-	if( lock )
-		iked.lock_sdb.lock();
-
-	refcount++;
-
-	iked.log.txt( LOG_LOUD,
-		"DB : phase1 ref increment ( ref count = %i, phase1 count = %i )\n",
-		refcount,
-		iked.list_phase1.get_count() );
-
-	if( lock )
-		iked.lock_sdb.unlock();
-
-	return true;
-}
-
-bool _IDB_PH1::dec( bool lock )
-{
-	if( lock )
-		iked.lock_sdb.lock();
-
-	//
-	// if we are marked for deletion,
-	// attempt to remove any events
-	// that may be scheduled
-	//
-
-	if( lstate & LSTATE_DELETE )
-	{
-		if( iked.ith_timer.del( &event_resend ) )
-		{
-			refcount--;
-			iked.log.txt( LOG_DEBUG,
-				"DB : phase1 resend event canceled ( ref count = %i )\n",
-				refcount );
-		}
-
-		if( iked.ith_timer.del( &event_dpd ) )
-		{
-			refcount--;
-			iked.log.txt( LOG_DEBUG,
-				"DB : phase1 dpd event canceled ( ref count = %i )\n",
-				refcount );
-		}
-
-		if( iked.ith_timer.del( &event_natt ) )
-		{
-			refcount--;
-			iked.log.txt( LOG_DEBUG,
-				"DB : phase1 natt event canceled ( ref count = %i )\n",
-				refcount );
-		}
-
-		if( iked.ith_timer.del( &event_hard ) )
-		{
-			refcount--;
-			iked.log.txt( LOG_DEBUG,
-				"DB : phase1 hard event canceled ( ref count = %i )\n",
-				refcount );
-		}
-	}
-
-	assert( refcount > 0 );
-
-	refcount--;
-
-	//
-	// check for deletion
-	//
-
-	if( refcount || !( lstate & LSTATE_DELETE ) )
-	{
-		iked.log.txt( LOG_LOUD,
-			"DB : phase1 ref decrement ( ref count = %i, phase1 count = %i )\n",
-			refcount,
-			iked.list_phase1.get_count() );
-
-		if( lock )
-			iked.lock_sdb.unlock();
-
-		return false;
-	}
-
-	//
-	// send a delete message if required
-	//
-
-	if(  ( lstate & LSTATE_MATURE ) && 
-		!( lstate & LSTATE_NOTIFY ) )
-		iked.inform_new_delete( this, NULL );
-
-	//
-	// terminate client thread if relevant
-	//
-
-	if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
-		if( !tunnel->close )
-			tunnel->close = TERM_PEER_DEAD;
-
-	//
-	// cleaup after client based tunnels
-	//
-
-	if( !initiator )
-	{
-		if( tunnel->peer->plcy_mode != POLICY_MODE_DISABLE )
-			iked.policy_list_remove( tunnel, false );
-
-		if( tunnel->xconf.opts & IPSEC_OPTS_ADDR )
-			tunnel->peer->xconf_source->pool4_rel( tunnel->xconf.addr );
-	}
-
-	//
-	// if this sa never reached maturity,
-	// locate any pending phase2 handles
-	// for this tunnel and delete them
-	//
-
-	if( !( lstate & LSTATE_MATURE ) )
-	{
-		long count = iked.list_phase2.get_count();
-		long index = 0;
-
-		for( ; index < count; index++ )
-		{
-			//
-			// get the next phase2 in our list
-			// and attempt to match tunnel ids
-			// 
-
-			IDB_PH2 * ph2 = ( IDB_PH2 * ) iked.list_phase2.get_item( index );
-			if( ( ph2->tunnel == tunnel ) &&
-				( ph2->lstate & LSTATE_PENDING ) )
-			{
-				ph2->inc( false );
-				ph2->lstate |= LSTATE_DELETE;
-
-				if( ph2->dec( false ) )
-				{
-					index--;
-					count--;
-				}
-			}
-		}
-	}
-
-	//
-	// remove from our list
-	//
-
-	iked.list_phase1.del_item( this );
-
-	//
-	// log deletion
-	//
-
-	if( !( lstate & LSTATE_EXPIRE ) )
-		iked.log.txt( LOG_INFO,
-			"DB : phase1 deleted before expire time ( phase1 count = %i )\n",
-			iked.list_phase1.get_count() );
-	else
-		iked.log.txt( LOG_INFO,
-			"DB : phase1 deleted after expire time ( phase1 count = %i )\n",
-			iked.list_phase1.get_count() );
-
-	//
-	// derefrence our tunnel
-	//
-
-	tunnel->dec( false );
-
-	if( lock )
-		iked.lock_sdb.unlock();
-
-	//
-	// free
-	//
-
-	delete this;
-
-	return true;
-}
