@@ -57,9 +57,9 @@ long _IKED::loop_ike_pfkey()
 	// begin pfkey thread
 	//
 
-	log.txt( LOG_INFO, "ii : pfkey process thread begin ...\n" );
+	loop_ref_inc( "pfkey" );
 
-	refcount++;
+	PFKI_MSG msg;
 
 	while( state == DSTATE_ACTIVE )
 	{
@@ -67,18 +67,19 @@ long _IKED::loop_ike_pfkey()
 		// read the next pfkey message
 		//
 
-		PFKI_MSG msg;
+		long result = pfki.recv_message( msg );
 
-		long result = pfki.next_msg( msg );
-
-		if( result == PFKI_NODATA )
+		if( result == IPCERR_NODATA )
 			continue;
 
-		if( result != PFKI_OK )
+		if( result == IPCERR_CLOSED )
 		{
-			if( pfki.open() != PFKI_OK )
+			pfki.detach();
+
+			if( pfki.attach( 1000 ) != IPCERR_OK )
 			{
-				Sleep( 100 );
+				log.txt( LLOG_ERROR, "!! : unable to connect to pfkey interface\n" );
+				Sleep( 1000 );
 				continue;
 			}
 
@@ -86,11 +87,11 @@ long _IKED::loop_ike_pfkey()
 			// register for certain protocol types
 			//
 
-			if( ( pfki.send_register( SADB_SATYPE_AH ) != PFKI_OK ) ||
-				( pfki.send_register( SADB_SATYPE_ESP ) != PFKI_OK ) ||
-				( pfki.send_register( SADB_X_SATYPE_IPCOMP ) != PFKI_OK ) )
+			if( ( pfki.send_register( SADB_SATYPE_AH ) != IPCERR_OK ) ||
+				( pfki.send_register( SADB_SATYPE_ESP ) != IPCERR_OK ) ||
+				( pfki.send_register( SADB_X_SATYPE_IPCOMP ) != IPCERR_OK ) )
 			{
-				log.txt( LOG_ERROR, "!! : unable to send pfkey register message\n" );
+				log.txt( LLOG_ERROR, "!! : unable to send pfkey register message\n" );
 				return LIBIKE_FAILED;
 			}
 
@@ -98,21 +99,21 @@ long _IKED::loop_ike_pfkey()
 			// initiate an SPD dump
 			//
 
-			if( pfki.send_spdump() != PFKI_OK )
+			if( pfki.send_spdump() != IPCERR_OK )
 			{
-				log.txt( LOG_ERROR, "!! : unable to send pfkey spd dump message\n" );
+				log.txt( LLOG_ERROR, "!! : unable to send pfkey spd dump message\n" );
 				return LIBIKE_FAILED;
 			}
 
 			continue;
 		}
 
-		if( msg.hdr->sadb_msg_errno )
+		if( msg.header.sadb_msg_errno )
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"K! : recv %s message failure ( errno = %i )\n",
-				pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-				msg.hdr->sadb_msg_errno );
+				pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+				msg.header.sadb_msg_errno );
 
 			continue;
 		}
@@ -121,29 +122,29 @@ long _IKED::loop_ike_pfkey()
 		// process the message by type
 		//
 
-		switch( msg.hdr->sadb_msg_type )
+		switch( msg.header.sadb_msg_type )
 		{
 			case SADB_REGISTER:
 
 				if( msg.local() )
-					log.txt( LOG_DEBUG,
-						"K< : recv %s %s pfkey message\n",
-						pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-						pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+					log.txt( LLOG_DEBUG,
+						"K< : recv pfkey %s %s message\n",
+						pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+						pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 				else
-					log.txt( LOG_DEBUG,
-						"K< : recv %s %s pfkey message ( ignored )\n",
-						pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-						pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+					log.txt( LLOG_DEBUG,
+						"K< : recv pfkey %s %s message ( ignored )\n",
+						pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+						pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				break;
 
 			case SADB_FLUSH:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				pfkey_recv_flush( msg );
 
@@ -151,10 +152,10 @@ long _IKED::loop_ike_pfkey()
 
 			case SADB_ACQUIRE:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				pfkey_recv_acquire( msg );
 
@@ -162,10 +163,10 @@ long _IKED::loop_ike_pfkey()
 
 			case SADB_GETSPI:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				pfkey_recv_getspi( msg );
 
@@ -173,52 +174,62 @@ long _IKED::loop_ike_pfkey()
 
 			case SADB_UPDATE:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				break;
 
 			case SADB_DELETE:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				break;
 
 			case SADB_X_SPDFLUSH:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				pfkey_recv_spflush( msg );
 
 				break;
 
-			case SADB_X_SPDDUMP:
-			case SADB_X_SPDGET:
 			case SADB_X_SPDADD:
+			case SADB_X_SPDGET:
 
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
-				pfkey_recv_spinfo( msg );
+				pfkey_recv_spadd( msg );
+
+				break;
+
+			case SADB_X_SPDDUMP:
+
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
+
+				pfkey_recv_spnew( msg );
 
 				break;
 
 			case SADB_X_SPDDELETE2:
 				
-				log.txt( LOG_DEBUG,
-					"K< : recv %s %s pfkey message\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					pfki.name( NAME_SATYPE, msg.hdr->sadb_msg_satype ) );
+				log.txt( LLOG_DEBUG,
+					"K< : recv pfkey %s %s message\n",
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					pfki.name( NAME_SATYPE, msg.header.sadb_msg_satype ) );
 
 				pfkey_recv_spdel( msg );
 
@@ -227,20 +238,18 @@ long _IKED::loop_ike_pfkey()
 
 			default:
 
-				log.txt( LOG_ERROR,
+				log.txt( LLOG_ERROR,
 					"K! : unhandled pfkey message type %s ( %i )\n",
-					pfki.name( NAME_MSGTYPE, msg.hdr->sadb_msg_type ),
-					msg.hdr->sadb_msg_type );
+					pfki.name( NAME_MSGTYPE, msg.header.sadb_msg_type ),
+					msg.header.sadb_msg_type );
 
 				break;
 		}
 	}
 
-	pfki.close();
+	pfki.detach();
 
-	refcount--;
-
-	log.txt( LOG_INFO, "ii : pfkey process thread exit ...\n" );
+	loop_ref_dec( "pfkey" );
 
 	return LIBIKE_OK;
 }
@@ -256,7 +265,6 @@ bool _IKED::paddr_ph2id( PFKI_ADDR & paddr, IKE_PH2ID & ph2id )
 			else
 				ph2id.prot = 0;
 
-			ph2id.prot = paddr.proto;
 			ph2id.port = paddr.saddr4.sin_port;
 
 			ph2id.type = ISAKMP_ID_IPV4_ADDR;
@@ -280,7 +288,7 @@ bool _IKED::paddr_ph2id( PFKI_ADDR & paddr, IKE_PH2ID & ph2id )
 		}
 
 		default:
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : ph2id -> pfkiaddr, unhandled address faimily %i\n",
 				paddr.saddr.sa_family );
 	}
@@ -322,7 +330,7 @@ bool _IKED::ph2id_paddr( IKE_PH2ID & ph2id, PFKI_ADDR & paddr )
 		}
 
 		default:
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : ph2id -> pfkiaddr, unhandled id type %i\n",
 				ph2id.type );
 	}
@@ -330,23 +338,253 @@ bool _IKED::ph2id_paddr( IKE_PH2ID & ph2id, PFKI_ADDR & paddr )
 	return false;
 }
 
-long _IKED::pfkey_recv_spinfo( PFKI_MSG & msg )
+long _IKED::pfkey_init_phase2( bool nailed, u_int16_t plcytype, u_int32_t plcyid, u_int32_t seq )
+{
+	//
+	// locate oubound policy by id
+	//
+
+	IDB_POLICY * policy_out;
+
+	if( !idb_list_policy.find(
+			true,
+			&policy_out,
+			IPSEC_DIR_OUTBOUND,
+			plcytype,
+			NULL,
+			&plcyid,
+			NULL,
+			NULL,
+			NULL,
+			NULL ) )
+	{
+		log.txt( LLOG_ERROR, "!! : unable to locate outbound policy for init phase2\n" );
+
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// if this policy was marked as
+	// nailed for a client tunnel,
+	// ignore the request if it came
+	// in the form of an aquire
+	//
+
+	if( policy_out->nailed && !nailed )
+	{
+		log.txt( LLOG_INFO, "ii : ignoring init phase2 by acquire, tunnel is nailed\n" );
+
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// locate inbound policy by the
+	// source and destination addrs
+	// and ids
+	//
+
+	IKE_SADDR src, dst;
+	policy_get_addrs( policy_out, src, dst );
+
+	IKE_PH2ID ids, idd;
+	paddr_ph2id( policy_out->paddr_src, ids );
+	paddr_ph2id( policy_out->paddr_dst, idd );
+
+	IDB_POLICY * policy_in;
+
+	if( !idb_list_policy.find(
+			true,
+			&policy_in,
+			IPSEC_DIR_INBOUND,
+			plcytype,
+			NULL,
+			NULL,
+			&dst,
+			&src,
+			&idd,
+			&ids ) )
+	{
+		log.txt( LLOG_ERROR, "!! : unable to locate inbound policy for init phase2\n" );
+
+		policy_out->dec( true );
+
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// attempt to locate an existing
+	// tunnel for the policy
+	//
+
+	IDB_TUNNEL * tunnel;
+	if( !idb_list_tunnel.find(
+			true,
+			&tunnel,
+			NULL,
+			&dst,
+			false ) )
+	{
+		//
+		// attempt to locate an existing
+		// peer config for the policy
+		//
+		
+		IDB_PEER * peer;
+		if( !idb_list_peer.find(
+				true,
+				&peer,
+				&dst ) )
+		{
+			log.txt( LLOG_ERROR, "!! : unable to locate peer config for policy\n" );
+
+			policy_in->dec( true );
+			policy_out->dec( true );
+
+			return LIBIKE_FAILED;
+		}
+
+		//
+		// attempt to locate the socket
+		// for our address and the ike
+		// port value
+		//
+
+		if( socket_locate( src ) != LIBIKE_OK )
+		{
+			char txtaddr[ LIBIKE_MAX_TEXTADDR ];
+			text_addr( txtaddr, &src, false );
+
+			log.txt( LLOG_ERROR,
+				"!! : unable to create tunnel, no socket for address %s\n",
+				txtaddr );
+
+			policy_in->dec( true );
+			policy_out->dec( true );
+
+			return LIBIKE_FAILED;
+		}
+
+		//
+		// attempt to create and add a new
+		// tunnel for the peer
+		//
+
+		tunnel = new IDB_TUNNEL( peer, &src, &peer->saddr );
+
+		if( tunnel == NULL )
+		{
+			log.txt( LLOG_ERROR, "!! : unable to create new tunnel object\n" );
+
+			peer->dec( true );
+			policy_in->dec( true );
+			policy_out->dec( true );
+
+			return LIBIKE_FAILED;
+		}
+
+		if( !tunnel->add( true ) )
+		{
+			log.txt( LLOG_ERROR, "!! : unable to add tunnel object\n" );
+
+			delete tunnel;
+
+			peer->dec( true );
+			policy_in->dec( true );
+			policy_out->dec( true );
+
+			return LIBIKE_FAILED;
+		}
+
+		peer->dec( true );
+	}
+
+	//
+	// create a new phase2 handler
+	// for the security association
+	//
+
+	IDB_PH2 * ph2 = new IDB_PH2( tunnel, true, 0, seq );
+	if( ph2 == NULL )
+	{
+		tunnel->dec( true );
+		policy_in->dec( true );
+		policy_out->dec( true );
+
+		return LIBIKE_FAILED;
+	}
+
+	if( !ph2->add( true ) != LIBIKE_OK )
+	{
+		delete ph2;
+
+		tunnel->dec( true );
+		policy_in->dec( true );
+		policy_out->dec( true );
+
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// configure client nailed policy id
+	//
+
+	if( nailed )
+		ph2->nailed_plcyid = policy_out->sp.id;
+
+	//
+	// configure the phase2 network ids
+	//
+
+	ph2->ph2id_ls = ids;
+	ph2->ph2id_ld = idd;
+
+	//
+	// configure the phase2 proposal list
+	//
+
+	phase2_gen_prop( ph2, policy_out );
+
+	//
+	// configure the phase2 dh group
+	//
+
+	ph2->setup_dhgrp();
+
+	//
+	// acquire any needed pfkey spis
+	//
+
+	pfkey_send_getspi( policy_in, ph2 );
+
+	//
+	// cleanup
+	//
+
+	ph2->dec( true );
+	tunnel->dec( true );
+	policy_in->dec( true );
+	policy_out->dec( true );
+
+	return LIBIKE_OK;
+}
+
+long _IKED::pfkey_recv_spadd( PFKI_MSG & msg )
 {
 	PFKI_SPINFO spinfo;
 	memset( &spinfo, 0, sizeof( spinfo ) );
 
-	if( pfki.read_policy( msg, spinfo ) != PFKI_OK )
+	if( pfki.read_policy( msg, spinfo ) != IPCERR_OK )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read basic policy info\n" );
 
 		return LIBIKE_FAILED;
 	}
 
-	if( ( pfki.read_address_src( msg, spinfo.paddr_src ) != PFKI_OK ) ||
-		( pfki.read_address_dst( msg, spinfo.paddr_dst ) != PFKI_OK ) )
+	if( ( pfki.read_address_src( msg, spinfo.paddr_src ) != IPCERR_OK ) ||
+		( pfki.read_address_dst( msg, spinfo.paddr_dst ) != IPCERR_OK ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read policy address info\n" );
 
 		return LIBIKE_FAILED;
@@ -358,7 +596,7 @@ long _IKED::pfkey_recv_spinfo( PFKI_MSG & msg )
 	text_addr( txtid_src, &spinfo.paddr_src, true, true );
 	text_addr( txtid_dst, &spinfo.paddr_dst, true, true );
 
-	log.txt( LOG_DECODE,
+	log.txt( LLOG_DECODE,
 		"ii : - id   = %i\n"
 		"ii : - type = %s\n"
 		"ii : - dir  = %s\n"
@@ -379,7 +617,7 @@ long _IKED::pfkey_recv_spinfo( PFKI_MSG & msg )
 				if( xindex )
 					break;
 
-				log.txt( LOG_ERROR, "!! : failed to add policy, no transforms defind\n" );
+				log.txt( LLOG_ERROR, "!! : failed to add policy, no transforms defind\n" );
 				return LIBIKE_FAILED;
 			}
 
@@ -389,7 +627,132 @@ long _IKED::pfkey_recv_spinfo( PFKI_MSG & msg )
 			text_addr( txtaddr_src, &spinfo.xforms[ xindex ].saddr_src, false );
 			text_addr( txtaddr_dst, &spinfo.xforms[ xindex ].saddr_dst, false );
 
-			log.txt( LOG_DECODE,
+			log.txt( LLOG_DECODE,
+				"ii : - transform #%i\n"
+				"ii : -- proto = %i\n"
+				"ii : -- level = %s\n"
+				"ii : -- mode  = %s\n"
+				"ii : -- reqid = %i\n"
+				"ii : -- tsrc  = %s\n"
+				"ii : -- tdst  = %s\n",
+				xindex,
+				spinfo.xforms[ xindex ].proto,
+				pfki.name( NAME_SPLEVEL, spinfo.xforms[ xindex ].level ),
+				pfki.name( NAME_SPMODE, spinfo.xforms[ xindex ].mode ),
+				spinfo.xforms[ xindex ].reqid,
+				txtaddr_src,
+				txtaddr_dst );
+		}
+	}
+
+	//
+	// locate policy
+	//
+
+	IDB_POLICY * policy;
+
+	if( !idb_list_policy.find(
+			true,
+			&policy,
+			spinfo.sp.dir,
+			spinfo.sp.type,
+			&msg.header.sadb_msg_seq,
+			NULL,
+			NULL,
+			NULL,
+			NULL,
+			NULL ) )
+	{
+		log.txt( LLOG_ERROR,
+			"!! : unable to locate policy with sequence 0x%08x\n",
+			spinfo.seq );
+
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// update the policy id
+	//
+
+	policy->sp.id = spinfo.sp.id;
+
+	//
+	// if this policy was marked as
+	// nailed for a client tunnel,
+	// call init phase2 now
+	//
+
+	if( policy->nailed )
+	{
+		log.txt( LLOG_DEBUG, "ii : calling init phase2 for nailed policy\n" );
+		pfkey_init_phase2( true, spinfo.sp.type, spinfo.sp.id, 0 );
+	}
+
+	policy->dec( true );
+
+	return LIBIKE_OK;
+}
+
+long _IKED::pfkey_recv_spnew( PFKI_MSG & msg )
+{
+	PFKI_SPINFO spinfo;
+	memset( &spinfo, 0, sizeof( spinfo ) );
+
+	if( pfki.read_policy( msg, spinfo ) != IPCERR_OK )
+	{
+		log.txt( LLOG_ERROR,
+			"K! : failed to read basic policy info\n" );
+
+		return LIBIKE_FAILED;
+	}
+
+	if( ( pfki.read_address_src( msg, spinfo.paddr_src ) != IPCERR_OK ) ||
+		( pfki.read_address_dst( msg, spinfo.paddr_dst ) != IPCERR_OK ) )
+	{
+		log.txt( LLOG_ERROR,
+			"K! : failed to read policy address info\n" );
+
+		return LIBIKE_FAILED;
+	}
+
+	char txtid_src[ LIBIKE_MAX_TEXTP2ID ];
+	char txtid_dst[ LIBIKE_MAX_TEXTP2ID ];
+
+	text_addr( txtid_src, &spinfo.paddr_src, true, true );
+	text_addr( txtid_dst, &spinfo.paddr_dst, true, true );
+
+	log.txt( LLOG_DECODE,
+		"ii : - id   = %i\n"
+		"ii : - type = %s\n"
+		"ii : - dir  = %s\n"
+		"ii : - src  = %s\n"
+		"ii : - dst  = %s\n",
+		spinfo.sp.id,
+		pfki.name( NAME_SPTYPE, spinfo.sp.type ),
+		pfki.name( NAME_SPDIR, spinfo.sp.dir ),
+		txtid_src,
+		txtid_dst );
+
+	if( spinfo.sp.type == IPSEC_POLICY_IPSEC )
+	{
+		for( long xindex = 0; xindex < PFKI_MAX_XFORMS; xindex++ )
+		{
+			if( !spinfo.xforms[ xindex ].proto )
+			{
+				if( xindex )
+					break;
+
+				log.txt( LLOG_ERROR, "!! : failed to add policy, no transforms defind\n" );
+				return LIBIKE_FAILED;
+			}
+
+			char txtaddr_src[ LIBIKE_MAX_TEXTADDR ];
+			char txtaddr_dst[ LIBIKE_MAX_TEXTADDR ];
+
+			text_addr( txtaddr_src, &spinfo.xforms[ xindex ].saddr_src, false );
+			text_addr( txtaddr_dst, &spinfo.xforms[ xindex ].saddr_dst, false );
+
+			log.txt( LLOG_DECODE,
 				"ii : - transform #%i\n"
 				"ii : -- proto = %i\n"
 				"ii : -- level = %s\n"
@@ -430,9 +793,9 @@ long _IKED::pfkey_recv_acquire( PFKI_MSG & msg )
 	PFKI_SPINFO spinfo;
 	memset( &spinfo, 0, sizeof( spinfo ) );
 
-	if( pfki.read_policy( msg, spinfo ) != PFKI_OK )
+	if( pfki.read_policy( msg, spinfo ) != IPCERR_OK )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read basic policy info\n" );
 
 		return LIBIKE_FAILED;
@@ -440,7 +803,7 @@ long _IKED::pfkey_recv_acquire( PFKI_MSG & msg )
 
 	if( spinfo.sp.type != IPSEC_POLICY_IPSEC )
 	{
-		log.txt( LOG_DECODE,
+		log.txt( LLOG_DECODE,
 			"ii : - id   = %i\n"
 			"ii : - type = %s\n"
 			"ii : - dir  = %s\n",
@@ -451,10 +814,10 @@ long _IKED::pfkey_recv_acquire( PFKI_MSG & msg )
 		return LIBIKE_OK;
 	}
 
-	if( ( pfki.read_address_src( msg, spinfo.paddr_src ) != PFKI_OK ) ||
-		( pfki.read_address_dst( msg, spinfo.paddr_dst ) != PFKI_OK ) )
+	if( ( pfki.read_address_src( msg, spinfo.paddr_src ) != IPCERR_OK ) ||
+		( pfki.read_address_dst( msg, spinfo.paddr_dst ) != IPCERR_OK ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read policy address info\n" );
 
 		return LIBIKE_FAILED;
@@ -466,7 +829,7 @@ long _IKED::pfkey_recv_acquire( PFKI_MSG & msg )
 	text_addr( txtid_src, &spinfo.paddr_src, true, true );
 	text_addr( txtid_dst, &spinfo.paddr_dst, true, true );
 
-	log.txt( LOG_DECODE,
+	log.txt( LLOG_DECODE,
 		"ii : - id   = %i\n"
 		"ii : - type = %s\n"
 		"ii : - dir  = %s\n"
@@ -479,207 +842,21 @@ long _IKED::pfkey_recv_acquire( PFKI_MSG & msg )
 		txtid_dst );
 
 	//
-	// locate oubound policy by id
+	// initiate phase2 based on the aquire info
 	//
 
-	IDB_POLICY * policy_out;
-
-	if( !get_policy(
-		true,
-		&policy_out,
-		IPSEC_DIR_OUTBOUND,
-		spinfo.sp.type,
-		&spinfo.sp.id,
-		NULL,
-		NULL,
-		NULL,
-		NULL ) )
-	{
-		log.txt( LOG_ERROR, "!! : unable to locate outbound policy for acquire\n" );
-
-		return LIBIKE_FAILED;
-	}
-
-	//
-	// locate inbound policy by the
-	// source and destination addrs
-	// and ids
-	//
-
-	IKE_SADDR src, dst;
-	policy_get_addrs( policy_out, src, dst );
-
-	IKE_PH2ID ids, idd;
-	paddr_ph2id( policy_out->paddr_src, ids );
-	paddr_ph2id( policy_out->paddr_dst, idd );
-
-	IDB_POLICY * policy_in;
-
-	if( !get_policy(
-		true,
-		&policy_in,
-		IPSEC_DIR_INBOUND,
-		spinfo.sp.type,
-		NULL,
-		&dst,
-		&src,
-		&idd,
-		&ids ) )
-	{
-		log.txt( LOG_ERROR, "!! : unable to locate inbound policy for acquire\n" );
-
-		policy_out->dec( true );
-
-		return LIBIKE_FAILED;
-	}
-
-	//
-	// attempt to locate an existing
-	// tunnel for the policy
-	//
-
-	IDB_TUNNEL * tunnel;
-	if( !get_tunnel( true, &tunnel, NULL, &dst, false ) )
-	{
-		//
-		// attempt to locate an existing
-		// peer config for the policy
-		//
-		
-		IDB_PEER * peer;
-		if( !get_peer( true, &peer, &dst ) )
-		{
-			log.txt( LOG_ERROR, "!! : unable to locate peer config for policy\n" );
-
-			policy_in->dec( true );
-			policy_out->dec( true );
-
-			return LIBIKE_FAILED;
-		}
-
-		//
-		// attempt to locate the socket
-		// for our address and the ike
-		// port value
-		//
-
-		if( socket_locate( src ) != LIBIKE_OK )
-		{
-			char txtaddr[ LIBIKE_MAX_TEXTADDR ];
-			text_addr( txtaddr, &src, false );
-
-			log.txt( LOG_ERROR,
-				"!! : unable to create tunnel, no socket for address %s\n",
-				txtaddr );
-
-			policy_in->dec( true );
-			policy_out->dec( true );
-
-			return LIBIKE_FAILED;
-		}
-
-		//
-		// attempt to create and add a new
-		// tunnel for the peer
-		//
-
-		tunnel = new IDB_TUNNEL( peer, &src, &peer->saddr );
-
-		if( tunnel == NULL )
-		{
-			log.txt( LOG_ERROR, "!! : unable to create new tunnel object\n" );
-
-			peer->dec( true );
-			policy_in->dec( true );
-			policy_out->dec( true );
-
-			return LIBIKE_FAILED;
-		}
-
-		if( !tunnel->add( true ) )
-		{
-			log.txt( LOG_ERROR, "!! : unable to add tunnel object\n" );
-
-			delete tunnel;
-
-			peer->dec( true );
-			policy_in->dec( true );
-			policy_out->dec( true );
-
-			return LIBIKE_FAILED;
-		}
-
-		peer->dec( true );
-	}
-
-	//
-	// create a new phase2 handler
-	// for the security association
-	//
-
-	IDB_PH2 * ph2 = new IDB_PH2( tunnel, true, 0, msg.hdr->sadb_msg_seq );
-	if( ph2 == NULL )
-	{
-		tunnel->dec( true );
-		policy_in->dec( true );
-		policy_out->dec( true );
-
-		return LIBIKE_FAILED;
-	}
-
-	if( !ph2->add( true ) != LIBIKE_OK )
-	{
-		delete ph2;
-
-		tunnel->dec( true );
-		policy_in->dec( true );
-		policy_out->dec( true );
-
-		return LIBIKE_FAILED;
-	}
-
-	//
-	// configure the phase2 network ids
-	//
-
-	ph2->ph2id_ls = ids;
-	ph2->ph2id_ld = idd;
-
-	//
-	// configure the phase2 proposal list
-	//
-
-	phase2_gen_prop( ph2, policy_out );
-
-	//
-	// configure the phase2 dh group
-	//
-
-	ph2->setup_dhgrp();
-
-	//
-	// acquire any needed pfkey spis
-	//
-
-	pfkey_send_getspi( policy_in, ph2 );
-
-	//
-	// cleanup
-	//
-
-	ph2->dec( true );
-	tunnel->dec( true );
-	policy_in->dec( true );
-	policy_out->dec( true );
-
-	return LIBIKE_OK;
+	return pfkey_init_phase2(
+				false,
+				spinfo.sp.type,
+				spinfo.sp.id,
+				msg.header.sadb_msg_seq );
 }
 
 long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 {
 	if( !msg.local() )
 	{
-		log.txt( LOG_DECODE,
+		log.txt( LLOG_DECODE,
 			"ii : - message ignored ( not local )\n" );
 
 		return LIBIKE_OK;
@@ -688,9 +865,9 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 	PFKI_SA sa;
 	memset( &sa, 0, sizeof( sa ) );
 
-	if( pfki.read_sa( msg, sa ) != PFKI_OK )
+	if( pfki.read_sa( msg, sa ) != IPCERR_OK )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read security association info\n" );
 
 		return LIBIKE_FAILED;
@@ -699,10 +876,10 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 	PFKI_ADDR paddr_src;
 	PFKI_ADDR paddr_dst;
 
-	if( ( pfki.read_address_src( msg, paddr_src ) != PFKI_OK ) ||
-		( pfki.read_address_dst( msg, paddr_dst ) != PFKI_OK ) )
+	if( ( pfki.read_address_src( msg, paddr_src ) != IPCERR_OK ) ||
+		( pfki.read_address_dst( msg, paddr_dst ) != IPCERR_OK ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"K! : failed to read policy address info\n" );
 
 		return LIBIKE_FAILED;
@@ -714,12 +891,12 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 	text_addr( txtid_src, &paddr_src, true, true );
 	text_addr( txtid_dst, &paddr_dst, true, true );
 
-	log.txt( LOG_DECODE,
+	log.txt( LLOG_DECODE,
 		"ii : - seq  = 0x%08x\n"
 		"ii : - spi  = 0x%08x\n"
 		"ii : - src  = %s\n"
 		"ii : - dst  = %s\n",
-		msg.hdr->sadb_msg_seq,
+		msg.header.sadb_msg_seq,
 		ntohl( sa.spi ),
 		txtid_src,
 		txtid_dst );
@@ -731,7 +908,7 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 
 	unsigned char proto;
 
-	switch( msg.hdr->sadb_msg_satype )
+	switch( msg.header.sadb_msg_satype )
 	{
 		case SADB_SATYPE_AH:
 			proto = ISAKMP_PROTO_IPSEC_AH;
@@ -747,9 +924,9 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 
 		default:
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled pfkey spi protocol type %i\n",
-				msg.hdr->sadb_msg_satype );
+				msg.header.sadb_msg_satype );
 
 			return LIBIKE_FAILED;
 		}
@@ -761,11 +938,20 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 	//
 
 	IDB_PH2 * ph2;
-	if( !get_phase2( true, &ph2, NULL, 0, 0, &msg.hdr->sadb_msg_seq, NULL, NULL, NULL ) )
+	if( !idb_list_ph2.find(
+			true,
+			&ph2,
+			NULL,
+			XCH_STATUS_ANY,
+			XCH_STATUS_ANY,
+			&msg.header.sadb_msg_seq,
+			NULL,
+			NULL,
+			NULL ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : unable to locate phase2 for getspi update ( msg seq = %u )\n",
-			msg.hdr->sadb_msg_seq );
+			msg.header.sadb_msg_seq );
 
 		return LIBIKE_FAILED;
 	}
@@ -776,7 +962,7 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 	// inbound sas.
 	//
 
-	if( msg.hdr->sadb_msg_seq == ph2->seqid_out )
+	if( msg.header.sadb_msg_seq == ph2->seqid_out )
 	{
 		ph2->dec( true );
 		return LIBIKE_OK;
@@ -820,7 +1006,7 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 		pcount++;
 	}
 
-	log.txt( LOG_DEBUG,
+	log.txt( LLOG_DEBUG,
 		"ii : updated spi for %i %s proposal\n",
 		pcount,
 		find_name( NAME_PROTOCOL, proto ) );
@@ -833,50 +1019,89 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 
 	ph2->spicount--;
 
-	if( ph2->spicount )
-		log.txt( LOG_DEBUG, "ii : waiting for %i spi updates\n", ph2->spicount );
+	if( ph2->spicount > 0 )
+		log.txt( LLOG_DEBUG, "ii : waiting for %i spi updates\n", ph2->spicount );
 	else
 	{
 		ph2->lstate |= LSTATE_HASSPI;
 
-		//
-		// attempt to locate a phase1 sa
-		// for this tunnel
-		//
+		IDB_PH1 * ph1 = NULL;
 
-		IDB_PH1 * ph1;
-		if( !get_phase1( true, &ph1, ph2->tunnel, 0, 0, NULL ) )
+		if( ph2->initiator )
 		{
 			//
-			// mark phase2 as pending
+			// phase2 initiator
 			//
 
-			ph2->lstate |= LSTATE_PENDING;
-
-			//
-			// initiate a new phase1
-			//
-
-			ph1 = new IDB_PH1( ph2->tunnel, true, NULL );
-
-			if( ph1 == NULL )
+			if( !idb_list_ph1.find(
+					true,
+					&ph1,
+					ph2->tunnel,
+					XCH_STATUS_MATURE,
+					XCH_STATUS_EXPIRING,
+					NULL ) )
 			{
+				//
+				// mark the phase2 as pending
+				//
+
+				ph2->status( XCH_STATUS_PENDING, XCH_NORMAL, 0 );
+
+				//
+				// initiate a new phase1
+				//
+
+				ph1 = new IDB_PH1( ph2->tunnel, true, NULL );
+
+				if( ph1 == NULL )
+				{
+					ph2->dec( true );
+					return LIBIKE_FAILED;
+				}
+
+				if( !ph1->add( true ) )
+				{
+					delete ph1;
+					ph2->dec( true );
+					return LIBIKE_FAILED;
+				}
+
+				process_phase1_send( ph1 );
+			}
+		}
+		else
+		{
+			//
+			// phase2 responder
+			//
+
+			if( !idb_list_ph1.find(
+					true,
+					&ph1,
+					ph2->tunnel,
+					XCH_STATUS_MATURE,
+					XCH_STATUS_EXPIRING,
+					&ph2->cookies ) )
+			{
+				//
+				// some gateways expect a response to
+				// be protected using the same isakmp
+				// sa it was initiated with. best to
+				// ignore this and wait for another.
+				//
+
+				ph2->status( XCH_STATUS_DEAD, XCH_NORMAL, 0 );
 				ph2->dec( true );
 				return LIBIKE_FAILED;
 			}
-
-			if( !ph1->add( true ) )
-			{
-				delete ph1;
-
-				ph2->dec( true );
-				return LIBIKE_FAILED;
-			}
-
-			process_phase1_send( ph1 );
 		}
 
-		if( ph1->lstate & LSTATE_MATURE )
+		//
+		// if we have a mature phase1 sa,
+		// send the next phase2 message
+		//
+
+		if( ph1->status() >= XCH_STATUS_MATURE )
 			process_phase2_send( ph1, ph2 );
 
 		ph1->dec( true );
@@ -893,26 +1118,7 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 
 long _IKED::pfkey_recv_flush( PFKI_MSG & msg )
 {
-	lock_sdb.lock();
-
-	long count = iked.list_phase2.get_count();
-	long index = 0;
-
-	for( ; index < count; index++ )
-	{
-		IDB_PH2 * ph2 = ( IDB_PH2 * ) iked.list_phase2.get_item( index );
-
-		ph2->inc( false );
-		ph2->lstate |= ( LSTATE_DELETE | LSTATE_FLUSHED );
-
-		if( ph2->dec( false ) )
-		{
-			index--;
-			count--;
-		}
-	}
-
-	lock_sdb.unlock();
+	idb_list_ph2.flush();
 
 	return LIBIKE_OK;
 }
@@ -922,15 +1128,15 @@ long _IKED::pfkey_recv_spdel( PFKI_MSG & msg )
 	PFKI_SPINFO	spinfo;
 	memset( &spinfo, 0, sizeof( spinfo ) );
 
-	if( pfki.read_policy( msg, spinfo ) != PFKI_OK )
+	if( pfki.read_policy( msg, spinfo ) != IPCERR_OK )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : failed to read spdel policy data\n" );
 
 		return LIBIKE_FAILED;
 	}
 
-	log.txt( LOG_DECODE,
+	log.txt( LLOG_DECODE,
 		"ii : - id   = %i\n"
 		"ii : - type = %s\n"
 		"ii : - dir  = %s\n",
@@ -943,18 +1149,19 @@ long _IKED::pfkey_recv_spdel( PFKI_MSG & msg )
 	//
 
 	IDB_POLICY * policy;
-	if( !get_policy(
+	if( !idb_list_policy.find(
 			true,
 			&policy,
 			spinfo.sp.dir,
 			spinfo.sp.type,
+			NULL,
 			&spinfo.sp.id,
 			NULL,
 			NULL,
 			NULL,
 			NULL ) )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : failed to locate policy by id %i\n",
 			spinfo.sp.id );
 
@@ -965,34 +1172,14 @@ long _IKED::pfkey_recv_spdel( PFKI_MSG & msg )
 	// attempt to remove sp
 	//
 
-	policy->lstate = LSTATE_DELETE;
-	policy->dec( true );
+	policy->dec( true, true );
 
 	return LIBIKE_OK;
 }
 
 long _IKED::pfkey_recv_spflush( PFKI_MSG & msg )
 {
-	lock_sdb.lock();
-
-	long count = iked.list_policy.get_count();
-	long index = 0;
-
-	for( ; index < count; index++ )
-	{
-		IDB_POLICY * policy = ( IDB_POLICY * ) iked.list_policy.get_item( index );
-
-		policy->inc( false );
-		policy->lstate |= LSTATE_DELETE;
-
-		if( policy->dec( false ) )
-		{
-			index--;
-			count--;
-		}
-	}
-
-	lock_sdb.unlock();
+	idb_list_policy.flush();
 
 	return LIBIKE_OK;
 }
@@ -1007,7 +1194,7 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 	//
 
 	bool use_ports = false;
-	if( ph2->tunnel->natt_v != IPSEC_NATT_NONE )
+	if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
 		use_ports = true;
 
 	//
@@ -1168,12 +1355,12 @@ long _IKED::pfkey_send_getspi( IDB_POLICY * policy, IDB_PH2 * ph2 )
 		// send our getspi request to pfkey
 		//
 
-		log.txt( LOG_DEBUG,
-			"K> : send %s %s pfkey message\n",
+		log.txt( LLOG_DEBUG,
+			"K> : send pfkey %s %s message\n",
 			pfki.name( NAME_MSGTYPE, SADB_GETSPI ),
 			pfki.name( NAME_SATYPE, sainfo.satype ) );
 
-		log.txt( LOG_DECODE,
+		log.txt( LLOG_DECODE,
 			"ii : - seq   = 0x%08x\n"
 			"ii : - mode  = %s\n"
 			"ii : - reqid = 0x%08x\n"
@@ -1209,14 +1396,14 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	switch( proposal->encap )
 	{
 		case ISAKMP_ENCAP_TRANSPORT:
+		case ISAKMP_ENCAP_VXX_UDP_TRANSPORT:
 		case ISAKMP_ENCAP_RFC_UDP_TRANSPORT:
-		case ISAKMP_ENCAP_V02_UDP_TRANSPORT:
 			sainfo.sa2.mode = IPSEC_MODE_TRANSPORT;
 			break;
 
 		case ISAKMP_ENCAP_TUNNEL:
+		case ISAKMP_ENCAP_VXX_UDP_TUNNEL:
 		case ISAKMP_ENCAP_RFC_UDP_TUNNEL:
-		case ISAKMP_ENCAP_V02_UDP_TUNNEL:
 			sainfo.sa2.mode = IPSEC_MODE_TUNNEL;
 			break;
 	}
@@ -1253,9 +1440,9 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 					sainfo.sa.encrypt = SADB_EALG_3DESCBC;
 					break;
 
-//				case ISAKMP_ESP_CAST:
-//					sainfo.sa.encrypt = SADB_X_EALG_CAST128CBC;
-//					break;
+				case ISAKMP_ESP_CAST:
+					sainfo.sa.encrypt = SADB_X_EALG_CAST128CBC;
+					break;
 
 				case ISAKMP_ESP_BLOWFISH:
 					sainfo.sa.encrypt = SADB_X_EALG_BLOWFISHCBC;
@@ -1279,7 +1466,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 				default:
 				{
-					log.txt( LOG_ERROR,
+					log.txt( LLOG_ERROR,
 						"!! : unhandled ESP transform %s ( %i )\n",
 						find_name( NAME_XFORM_ESP, proposal->xform ),
 						proposal->xform );
@@ -1300,7 +1487,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 				default:
 				{
-					log.txt( LOG_ERROR,
+					log.txt( LLOG_ERROR,
 						"!! : unhandled ESP auth type %s ( %i )\n",
 						find_name( NAME_MAUTH, proposal->hash_id ),
 						proposal->hash_id );
@@ -1313,12 +1500,15 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 			switch( proposal->encap )
 			{
+				case ISAKMP_ENCAP_VXX_UDP_TUNNEL:
 				case ISAKMP_ENCAP_RFC_UDP_TUNNEL:
-				case ISAKMP_ENCAP_V02_UDP_TUNNEL:
+				case ISAKMP_ENCAP_VXX_UDP_TRANSPORT:
 				case ISAKMP_ENCAP_RFC_UDP_TRANSPORT:
-				case ISAKMP_ENCAP_V02_UDP_TRANSPORT:
 				{
-					sainfo.natt.type = UDP_ENCAP_ESPINUDP;
+					if( ph2->tunnel->natt_version >= IPSEC_NATT_V02 )
+						sainfo.natt.type = UDP_ENCAP_ESPINUDP;
+					else
+						sainfo.natt.type = UDP_ENCAP_ESPINUDP_NON_IKE;
 
 					switch( dir )
 					{
@@ -1359,7 +1549,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 				default:
 				{
-					log.txt( LOG_ERROR,
+					log.txt( LLOG_ERROR,
 						"!! : unhandled AH transform %s ( %i )\n",
 						find_name( NAME_XFORM_AH, proposal->xform ),
 						proposal->xform );
@@ -1392,7 +1582,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 				default:
 				{
-					log.txt( LOG_ERROR,
+					log.txt( LLOG_ERROR,
 						"!! : unhandled IPCOMP transform %s ( %i )\n",
 						find_name( NAME_XFORM_AH, proposal->xform ),
 						proposal->xform );
@@ -1406,7 +1596,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 
 		default:
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled ike protocol type %i\n",
 				proposal->proto );
 
@@ -1433,7 +1623,7 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	//
 
 	bool use_ports = false;
-	if( ph2->tunnel->natt_v != IPSEC_NATT_NONE )
+	if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
 		use_ports = true;
 
 	//
@@ -1489,12 +1679,12 @@ long _IKED::pfkey_send_update( IDB_PH2 * ph2, IKE_PROPOSAL * proposal, BDATA & e
 	if( sainfo.satype == SADB_X_SATYPE_IPCOMP )
 		nametype = NAME_SACOMP;
 
-	log.txt( LOG_DEBUG,
-		"K> : send %s %s pfkey message\n",
+	log.txt( LLOG_DEBUG,
+		"K> : send pfkey %s %s message\n",
 		pfki.name( NAME_MSGTYPE, SADB_UPDATE ),
 		pfki.name( NAME_SATYPE, sainfo.satype ) );
 
-	log.txt( LOG_DECODE,
+	log.txt( LLOG_DECODE,
 		"ii : - spi  = 0x%08x\n"
 		"ii : - src  = %s\n"
 		"ii : - dst  = %s\n"
@@ -1545,7 +1735,7 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 		//
 
 		bool use_ports = false;
-		if( ph2->tunnel->natt_v != IPSEC_NATT_NONE )
+		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
 			use_ports = true;
 
 		cpy_sockaddr( ph2->tunnel->saddr_l.saddr, sainfo.paddr_src.saddr, use_ports );
@@ -1579,12 +1769,12 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 				break;
 		}
 
-		log.txt( LOG_DEBUG,
-			"K> : send %s %s pfkey message\n",
+		log.txt( LLOG_DEBUG,
+			"K> : send pfkey %s %s message\n",
 			pfki.name( NAME_MSGTYPE, SADB_DELETE ),
 			pfki.name( NAME_SATYPE, sainfo.satype ) );
 
-		log.txt( LOG_DECODE,
+		log.txt( LLOG_DECODE,
 			"ii : - spi   = 0x%08x\n"
 			"ii : - src   = %s\n"
 			"ii : - dst   = %s\n",
@@ -1607,7 +1797,7 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 		//
 
 		bool use_ports = false;
-		if( ph2->tunnel->natt_v != IPSEC_NATT_NONE )
+		if( ph2->tunnel->natt_version != IPSEC_NATT_NONE )
 			use_ports = true;
 
 		cpy_sockaddr( ph2->tunnel->saddr_r.saddr, sainfo.paddr_src.saddr, use_ports );
@@ -1641,12 +1831,12 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 				break;
 		}
 
-		log.txt( LOG_DEBUG,
-			"K> : send %s %s pfkey message\n",
+		log.txt( LLOG_DEBUG,
+			"K> : send pfkey %s %s message\n",
 			pfki.name( NAME_MSGTYPE, SADB_DELETE ),
 			pfki.name( NAME_SATYPE, sainfo.satype ) );
 
-		log.txt( LOG_DECODE,
+		log.txt( LLOG_DECODE,
 			"ii : - spi   = 0x%08x\n"
 			"ii : - src   = %s\n"
 			"ii : - dst   = %s\n",
@@ -1662,13 +1852,13 @@ long _IKED::pfkey_send_delete( IDB_PH2 * ph2 )
 
 long _IKED::pfkey_send_spadd( PFKI_SPINFO * spinfo )
 {
-	log.txt( LOG_DEBUG,
-		"K> : send %s %s pfkey message\n",
+	log.txt( LLOG_DEBUG,
+		"K> : send pfkey %s %s message\n",
 		pfki.name( NAME_MSGTYPE, SADB_X_SPDADD ),
 		pfki.name( NAME_SATYPE, SADB_SATYPE_UNSPEC ) );
 
 	long result = pfki.send_spadd( *spinfo );
-	if( result != PFKI_OK )
+	if( result != IPCERR_OK )
 		return LIBIKE_FAILED;
 
 	return LIBIKE_OK;
@@ -1676,13 +1866,13 @@ long _IKED::pfkey_send_spadd( PFKI_SPINFO * spinfo )
 
 long _IKED::pfkey_send_spdel( PFKI_SPINFO * spinfo )
 {
-	log.txt( LOG_DEBUG,
-		"K> : send %s %s pfkey message\n",
+	log.txt( LLOG_DEBUG,
+		"K> : send pfkey %s %s message\n",
 		pfki.name( NAME_MSGTYPE, SADB_X_SPDDELETE2 ),
 		pfki.name( NAME_SATYPE, SADB_SATYPE_UNSPEC ) );
 
 	long result = pfki.send_spdel( *spinfo );
-	if( result != PFKI_OK )
+	if( result != IPCERR_OK )
 		return LIBIKE_FAILED;
 
 	return LIBIKE_OK;
