@@ -264,8 +264,8 @@ const char * _IKED::find_name( long type, long id )
 			static const char * pload12 = "delete";
 			static const char * pload13 = "vendor";
 			static const char * pload14 = "attribute";
-			static const char * pload15 = "nat discovery v02";
-			static const char * pload16 = "nat original adress v02";
+			static const char * pload15 = "nat discovery draft";
+			static const char * pload16 = "nat original adress draft";
 			static const char * pload17 = "nat discovery rfc";
 			static const char * pload18 = "nat original adress rfc";
 			static const char * pload19 = "fragment";
@@ -317,10 +317,10 @@ const char * _IKED::find_name( long type, long id )
 				case ISAKMP_PAYLOAD_ATTRIB:
 					return pload14;
 
-				case ISAKMP_PAYLOAD_NAT_V02_DISC:
+				case ISAKMP_PAYLOAD_NAT_VXX_DISC:
 					return pload15;
 
-				case ISAKMP_PAYLOAD_NAT_V02_ORIG:
+				case ISAKMP_PAYLOAD_NAT_VXX_ORIG:
 					return pload16;
 
 				case ISAKMP_PAYLOAD_NAT_RFC_DISC:
@@ -388,10 +388,10 @@ const char * _IKED::find_name( long type, long id )
 			static const char * auth9 = "hybrid-responder-dss";
 			static const char * auth10 = "xauth-initiator-psk";
 			static const char * auth11 = "xauth-responder-psk";
-			static const char * auth12 = "xauth-initiator-rsa";
-			static const char * auth13 = "xauth-responder-rsa";
-			static const char * auth14 = "xauth-initiator-dss";
-			static const char * auth15 = "xauth-responder-dss";
+			static const char * auth12 = "xauth-initiator-dss";
+			static const char * auth13 = "xauth-responder-dss";
+			static const char * auth14 = "xauth-initiator-rsa";
+			static const char * auth15 = "xauth-responder-rsa";
 			static const char * auth16 = "xauth-initiator-rsa-encryption";
 			static const char * auth17 = "xauth-responder-rsa-encryption";
 			static const char * auth18 = "xauth-initiator-rsa-revised-encryption";
@@ -617,10 +617,10 @@ const char * _IKED::find_name( long type, long id )
 		{
 			static const char * encap1 = "tunnel";
 			static const char * encap2 = "transport";
-			static const char * encap3 = "udp-tunnel v02";
-			static const char * encap4 = "udp-transport v02";
-			static const char * encap5 = "udp-tunnel rfc";
-			static const char * encap6 = "udp-transport rfc";
+			static const char * encap3 = "udp-tunnel ( draft )";
+			static const char * encap4 = "udp-transport ( draft )";
+			static const char * encap5 = "udp-tunnel ( rfc )";
+			static const char * encap6 = "udp-transport ( rfc )";
 
 			switch( id )
 			{
@@ -630,10 +630,10 @@ const char * _IKED::find_name( long type, long id )
 				case ISAKMP_ENCAP_TRANSPORT:
 					return encap2;
 
-				case ISAKMP_ENCAP_V02_UDP_TUNNEL:
+				case ISAKMP_ENCAP_VXX_UDP_TUNNEL:
 					return encap3;
 
-				case ISAKMP_ENCAP_V02_UDP_TRANSPORT:
+				case ISAKMP_ENCAP_VXX_UDP_TRANSPORT:
 					return encap4;
 
 				case ISAKMP_ENCAP_RFC_UDP_TUNNEL:
@@ -649,6 +649,7 @@ const char * _IKED::find_name( long type, long id )
 
 		case NAME_IDENT:
 		{
+			static const char * ident0  = "none";
 			static const char * ident1  = "ipv4-host";
 			static const char * ident2  = "fqdn";
 			static const char * ident3  = "user-fqdn";
@@ -663,6 +664,9 @@ const char * _IKED::find_name( long type, long id )
 
 			switch( id )
 			{
+				case ISAKMP_ID_NONE:
+					return ident0;
+
 				case ISAKMP_ID_IPV4_ADDR:
 					return ident1;
 
