@@ -80,6 +80,12 @@
 # endif
 # include "compat/winstring.h"
 # include "conf.parse.hpp"
+# ifndef SOCKET
+#  define SOCKET int
+# endif
+# ifndef INVALID_SOCKET
+#  define INVALID_SOCKET -1
+# endif
 #endif
 
 #ifdef OPT_LDAP
@@ -92,17 +98,17 @@
 #include "libith.h"
 #include "libpfk.h"
 #include "libike.h"
+#include "libidb.h"
+#include "crypto.h"
 #include "ike.h"
-#include "idb.h"
+#include "iked.idb.h"
 #include "xauth.h"
 #include "xconf.h"
-#include "crypto.h"
 
 #ifdef OPT_DTP
 #include "libdtp.h"
 #endif
 
-
 //
 // Win32 specific
 //
@@ -150,20 +156,50 @@ YY_DECL;
 // IKED constants
 //
 
+// Netscreen-01 299ee8289f40a8973bc78687e2e7226b532c3b76
+// Netscreen-02 3a15e1f3cf2a63582e3ac82d1c64cbe3b6d779e7
+// Netscreen-03 47d2b126bfcd83489760e2cf8c5d4d5a03497c15
+// Netscreen-04 4a4340b543e02b84c88a8b96a8af9ebe77d9accc
+// Netscreen-05 64405f46f03b7660a23be116a1975058e69e8387
+// Netscreen-06 699369228741c6d4ca094c93e242c9de19e7b7c6
+// Netscreen-07 8c0dc6cf62a0ef1b5c6eabd1b67ba69866adf16a
+// Netscreen-08 92d27a9ecb31d99246986d3453d0c3d57a222a61
+// Netscreen-09 9b096d9ac3275a7d6fe8b91c583111b09efed1a0
+// Netscreen-10 bf03746108d746c904f1f3547de24f78479fed12
+// Netscreen-11 c2e80500f4cc5fbf5daaeed3bb59abaeee56c652
+// Netscreen-12 c8660a62b03b1b6130bf781608d32a6a8d0fb89f
+// Netscreen-13 f885da40b1e7a9abd17655ec5bbec0f21f0ed52e
+// Netscreen-14 2a2bcac19b8e91b426107807e02e7249569d6fd3
+// Netscreen-15 166f932d55eb64d8e4df4fd37e2313f0d0fd8451
+// Netscreen-16 a35bfd05ca1ac0b3d2f24e9e82bfcbff9c9e52b5
+
 #define	VEND_XAUTH		{ 0x09, 0x00, 0x26, 0x89, 0xDF, 0xD6, 0xB7, 0x12 }
-#define	VEND_UNITY		{ 0x12, 0xf5, 0xf2, 0x8c, 0x45, 0x71, 0x68, 0xa9, 0x70, 0x2d, 0x9f, 0xe2, 0x74, 0xcc, 0x01, 0x00 }
 #define VEND_FRAG		{ 0x40, 0x48, 0xb7, 0xd5, 0x6e, 0xbc, 0xe8, 0x85, 0x25, 0xe7, 0xde, 0x7f, 0x00, 0xd6, 0xc2, 0xd3, 0x80, 0x00, 0x00, 0x00 }
-#define	VEND_NATT_V02	{ 0x90, 0xcb, 0x80, 0x91, 0x3e, 0xbb, 0x69, 0x6e, 0x08, 0x63, 0x81, 0xb5, 0xec, 0x42, 0x7b, 0x1f }
-#define	VEND_NATT_RFC	{ 0x4a, 0x13, 0x1c, 0x81, 0x07, 0x03, 0x58, 0x45, 0x5c, 0x57, 0x28, 0xf2, 0x0e, 0x95, 0x45, 0x2f }
-#define VEND_KAME		{ 0x70, 0x03, 0xcb, 0xc1, 0x09, 0x7d, 0xbe, 0x9c, 0x26, 0x00, 0xba, 0x69, 0x83, 0xbc, 0x8b, 0x35 }
 #define VEND_DPD1		{ 0xaf, 0xca, 0xd7, 0x13, 0x68, 0xa1, 0xf1, 0xc9, 0x6b, 0x86, 0x96, 0xfc, 0x77, 0x57, 0x01, 0x00 }
+#define VEND_HBEAT		{ 0x48, 0x65, 0x61, 0x72, 0x74, 0x42, 0x65, 0x61, 0x74, 0x5f, 0x4e, 0x6f, 0x74, 0x69, 0x66, 0x79, 0x38, 0x6b, 0x01, 0x00 }
+#define	VEND_NATT_V00	{ 0x44, 0x85, 0x15, 0x2d, 0x18, 0xb6, 0xbb, 0xcd, 0x0b, 0xe8, 0xa8, 0x46, 0x95, 0x79, 0xdd, 0xcc }
+#define	VEND_NATT_V01	{ 0x16, 0xf6, 0xca, 0x16, 0xe4, 0xa4, 0x06, 0x6d, 0x83, 0x82, 0x1a, 0x0f, 0x0a, 0xea, 0xa8, 0x62 }
+#define	VEND_NATT_V02	{ 0x90, 0xcb, 0x80, 0x91, 0x3e, 0xbb, 0x69, 0x6e, 0x08, 0x63, 0x81, 0xb5, 0xec, 0x42, 0x7b, 0x1f }
+#define VEND_NATT_V03	{ 0x7d, 0x94, 0x19, 0xa6, 0x53, 0x10, 0xca, 0x6f, 0x2c, 0x17, 0x9d, 0x92, 0x15, 0x52, 0x9d, 0x56 }
+#define	VEND_NATT_RFC	{ 0x4a, 0x13, 0x1c, 0x81, 0x07, 0x03, 0x58, 0x45, 0x5c, 0x57, 0x28, 0xf2, 0x0e, 0x95, 0x45, 0x2f }
+
+#define VEND_SSOFT		{ 0xf1, 0x4b, 0x94, 0xb7, 0xbf, 0xf1, 0xfe, 0xf0, 0x27, 0x73, 0xb8, 0xc4, 0x9f, 0xed, 0xed, 0x26 }
+#define VEND_KAME		{ 0x70, 0x03, 0xcb, 0xc1, 0x09, 0x7d, 0xbe, 0x9c, 0x26, 0x00, 0xba, 0x69, 0x83, 0xbc, 0x8b, 0x35 }
+#define	VEND_UNITY		{ 0x12, 0xf5, 0xf2, 0x8c, 0x45, 0x71, 0x68, 0xa9, 0x70, 0x2d, 0x9f, 0xe2, 0x74, 0xcc, 0x01, 0x00 }
+#define VEND_NETSC		{ 0x16, 0x6f, 0x93, 0x2d, 0x55, 0xeb, 0x64, 0xd8, 0xe4, 0xdf, 0x4f, 0xd3, 0x7e, 0x23, 0x13, 0xf0, 0xd0, 0xfd, 0x84, 0x51 }
+#define VEND_ZWALL		{ 0x62, 0x50, 0x27, 0x74, 0x9d, 0x5a, 0xb9, 0x7f, 0x56, 0x16, 0xc1, 0x60, 0x27, 0x65, 0xcf, 0x48, 0x0a, 0x3b, 0x7d, 0x0b }
+#define VEND_SWIND		{ 0x84, 0x04, 0xad, 0xf9, 0xcd, 0xa0, 0x57, 0x60, 0xb2, 0xca, 0x29, 0x2e, 0x4b, 0xff, 0x53, 0x7b }
+#define VEND_SWALL		{ 0x40, 0x4B, 0xF4, 0x39, 0x52, 0x2C, 0xA3, 0xF6 }
+#define VEND_CHKPT		{ 0xf4, 0xed, 0x19, 0xe0, 0xc1, 0x14, 0xeb, 0x51, 0x6f, 0xaa, 0xac, 0x0e, 0xe3, 0x7d, 0xaf, 0x28, 0x07, 0xb4, 0x38, 0x1f }
 
 #define LIBIKE_IKE_PORT			500		// default isakmp port
 #define LIBIKE_NATT_PORT		4500	// default nat-t port
 
+#define LIBIKE_MAX_TEXTPROT		5		// max text protocol length
 #define LIBIKE_MAX_TEXTADDR		24		// max text address length
+#define LIBIKE_MAX_TEXTPORT		6		// max text port length
 #define LIBIKE_MAX_TEXTP1ID		256		// max text phase1 id length
-#define LIBIKE_MAX_TEXTP2ID		50		// max text phase2 id length
+#define LIBIKE_MAX_TEXTP2ID		64		// max text phase2 id length
 #define LIBIKE_MAX_TEXTSPI		64		// max text phase2 id length
 #define LIBIKE_MAX_VARID		512		// max variable id length
 #define LIBIKE_MAX_DHGRP		512		// max dh group size
@@ -224,31 +260,29 @@ YY_DECL;
 #define XSTATE_RECV_LP			0x000100000
 #define XSTATE_SENT_LP			0x000200000
 
-#define LSTATE_PENDING			0x000000001		// pending phase1 negotiation
-#define LSTATE_HASSPI			0x000000002		// pfkey spi obtained
-#define LSTATE_CHKPROP			0x000000004		// proposal verified
-#define LSTATE_CHKHASH			0x000000008		// hash verified
-#define LSTATE_CHKIDS			0x000000010		// identity verified
-#define LSTATE_GENNATD			0x000000020		// natt discovery generated
-#define LSTATE_CHKNATD			0x000000040		// natt discovery verified
-#define LSTATE_HASNATP			0x000000080		// natt ports floated
-#define LSTATE_HASKEYS			0x000000100		// keys generated
-#define LSTATE_CLAIMLT			0x000000200		// claim reponder lifetime
-#define LSTATE_MATURE			0x000000400		// mature and usable
-#define LSTATE_EXPIRE			0x000000800		// lifetime expired
-#define LSTATE_NOTIFY			0x000001000		// skip peer notify
-#define LSTATE_DELETE			0x000002000		// ready for delete
-#define LSTATE_FLUSHED			0x000004000		// pfkey flushed
+#define LSTATE_HASSPI			0x000000001		// pfkey spi obtained
+#define LSTATE_CHKPROP			0x000000002		// proposal verified
+#define LSTATE_CHKHASH			0x000000004		// hash verified
+#define LSTATE_CHKIDS			0x000000008		// identity verified
+#define LSTATE_GENNATD			0x000000010		// natt discovery generated
+#define LSTATE_HASKEYS			0x000000020		// keys generated
+#define LSTATE_CLAIMLT			0x000000040		// claim reponder lifetime
 
-#define TSTATE_INITIALIZED		0x00000001
-#define TSTATE_RECV_XAUTH		0x00000002
-#define TSTATE_SENT_XAUTH		0x00000004
-#define TSTATE_RECV_XRSLT		0x00000008
-#define TSTATE_SENT_XRSLT		0x00000010
-#define TSTATE_SENT_CONFIG		0x00000020
-#define TSTATE_RECV_CONFIG		0x00000040
-#define TSTATE_VNET_ENABLE		0x00000080
-#define TSTATE_DELETE			0x00000100
+#define TSTATE_NATT_FLOAT		0x000000001
+#define TSTATE_INITIALIZED		0x000000002
+#define TSTATE_RECV_XUSER		0x000000004
+#define TSTATE_SENT_XUSER		0x000000008
+#define TSTATE_RECV_XPASS		0x000000010
+#define TSTATE_SENT_XPASS		0x000000020
+#define TSTATE_RECV_XAUTH		( TSTATE_RECV_XUSER | TSTATE_RECV_XPASS )
+#define TSTATE_SENT_XAUTH		( TSTATE_SENT_XUSER | TSTATE_SENT_XPASS )
+#define TSTATE_RECV_XRSLT		0x000000040
+#define TSTATE_SENT_XRSLT		0x000000080
+#define TSTATE_RECV_CONFIG		0x000000100
+#define TSTATE_SENT_CONFIG		0x000000200
+#define TSTATE_RECV_ACK			0x000000400
+#define TSTATE_SENT_ACK			0x000000800
+#define TSTATE_VNET_ENABLE		0x000001000
 
 #define DSTATE_ACTIVE			0
 #define DSTATE_TERMINATE		1
@@ -256,19 +290,9 @@ YY_DECL;
 
 #define RLEVEL_DAEMON			2
 
-#define TERM_CLIENT				1
-#define TERM_SOCKET				2
-#define TERM_EXPIRE				3
-#define TERM_BADMSG				4
-#define TERM_USER_AUTH			5
-#define TERM_PEER_AUTH			6
-#define TERM_USER_CLOSE			7
-#define TERM_PEER_CLOSE			8
-#define TERM_PEER_DEAD			9
-
 #define FILE_OK					0
-#define FILE_FAIL				1
-#define FILE_PATH				2
+#define FILE_PATH				1
+#define FILE_FAIL				2
 
 //
 // IKED main classes and structures
@@ -276,7 +300,7 @@ YY_DECL;
 
 #ifdef UNIX
 
-typedef struct _SOCK_INFO
+typedef struct _SOCK_INFO : public IDB_ENTRY
 {
 	int			sock;
 	IKE_SADDR	saddr;
@@ -292,11 +316,17 @@ typedef struct _VNET_ADAPTER
 
 #endif
 
-typedef class _ITH_ADMIN : public _ITH_EXEC
+typedef class _ITH_IKES : public _ITH_EXEC
 {
 	virtual long func( void * arg );
 
-}ITH_ADMIN;
+}ITH_IKES;
+
+typedef class _ITH_IKEC : public _ITH_EXEC
+{
+	virtual long func( void * arg );
+
+}ITH_IKEC;
 
 typedef class _ITH_NWORK : public _ITH_EXEC
 {
@@ -312,24 +342,39 @@ typedef class _ITH_PFKEY : public _ITH_EXEC
 
 typedef class _IKED
 {
-	friend class _ITH_ADMIN;
+	friend class _ITH_IKES;
+	friend class _ITH_IKEC;
 	friend class _ITH_NWORK;
 	friend class _ITH_PFKEY;
 
+	friend class _IDB_RC_ENTRY;
 	friend class _IDB_PEER;
-	friend class _IDB_POLICY;
 	friend class _IDB_TUNNEL;
+	friend class _IDB_POLICY;
 	friend class _IDB_XCH;
 	friend class _IDB_PH1;
 	friend class _IDB_PH2;
 	friend class _IDB_CFG;
 	friend class _IDB_INF;
 
+	friend class _IDB_LIST_IKED;
+	friend class _IDB_LIST_PEER;
+	friend class _IDB_LIST_TUNNEL;
+	friend class _IDB_LIST_POLICY;
+	friend class _IDB_LIST_PH1;
+	friend class _IDB_LIST_PH2;
+	friend class _IDB_LIST_CFG;
+
+	friend class _ITH_EVENT_TUNDHCP;
+	friend class _ITH_EVENT_TUNDPD;
+	friend class _ITH_EVENT_TUNNATT;
+	friend class _ITH_EVENT_TUNSTATS;
+
 	friend class _ITH_EVENT_RESEND;
 
-	friend class _ITH_EVENT_PH1DPD;
-	friend class _ITH_EVENT_PH1NATT;
+	friend class _ITH_EVENT_PH1SOFT;
 	friend class _ITH_EVENT_PH1HARD;
+	friend class _ITH_EVENT_PH1DEAD;
 
 	friend class _ITH_EVENT_PH2SOFT;
 	friend class _ITH_EVENT_PH2HARD;
@@ -348,17 +393,21 @@ typedef class _IKED
 
 	private:
 
-	char	path_ins[ MAX_PATH ];	// install path
-	char	path_log[ MAX_PATH ];	// logfile path
-	char	path_ike[ MAX_PATH ];	// debug output path
-	char	path_pub[ MAX_PATH ];	// debug output path
-
-	long	level;				// logfile level
+	char	path_ins[ MAX_PATH ];		// install path
+	char	path_log[ MAX_PATH ];		// logfile path
+	char	path_decrypt[ MAX_PATH ];	// decrypted pcap path
+	char	path_encrypt[ MAX_PATH ];	// encrypted pcap path
+	
+	long	level;				// logging level
+	long	logflags;			// logging options
 
+	long	peercount;			// peer reference count
+	long	loopcount;			// loop reference count
 	long	state;				// daemon run state
-	long	refcount;			// reference count
+
 	long	tunnelid;			// next tunnel id
 	short	policyid;			// next request id
+	long	dnsgrpid;			// next dns group id
 
 	long	retry_count;		// packet retry count
 	long	retry_delay;		// packet retry delay
@@ -368,7 +417,8 @@ typedef class _IKED
 	IPROUTE		iproute;		// ip route config interface
 	IPFRAG		ipfrag;			// ip fragment handling interface
 
-	ITH_ADMIN	ith_admin;		// admin thread
+	ITH_IKES	ith_ikes;		// server ipc thread
+	ITH_IKEC	ith_ikec;		// client ipc thread
 	ITH_NWORK	ith_nwork;		// network thread
 	ITH_PFKEY	ith_pfkey;		// pfkey thread
 
@@ -376,14 +426,22 @@ typedef class _IKED
 
 	short	ident;				// ip identity
 
-	LIST	list_socket;		// socket list
-	LIST	list_netgrp;		// net groups list
-	LIST	list_peer;			// ipsec peer list
-	LIST	list_tunnel;		// ipsec tunnel list
-	LIST	list_policy;		// ipsec policy list
-	LIST	list_phase1;		// phase 1 exchanges
-	LIST	list_phase2;		// phase 2 exchanges
-	LIST	list_config;		// config exchanges
+	ITH_COND	cond_idb;		// idb null reference condition
+	ITH_COND	cond_run;		// daemon null reference condition
+
+	ITH_LOCK	lock_run;
+	ITH_LOCK	lock_net;
+	ITH_LOCK	lock_idb;
+
+	IDB_LIST	list_socket;		// socket list
+
+	IDB_LIST			idb_list_netgrp;
+	IDB_LIST_PEER		idb_list_peer;
+	IDB_LIST_TUNNEL		idb_list_tunnel;
+	IDB_LIST_POLICY		idb_list_policy;
+	IDB_LIST_PH1		idb_list_ph1;
+	IDB_LIST_PH2		idb_list_ph2;
+	IDB_LIST_CFG		idb_list_cfg;
 
 	long	sock_ike_open;
 	long	sock_natt_open;
@@ -393,26 +451,28 @@ typedef class _IKED
 	// known vendor ids
 
 	BDATA	vend_xauth;
-	BDATA	vend_unity;
 	BDATA	vend_frag;
-	BDATA	vend_natt_v02;
-	BDATA	vend_natt_rfc;
 	BDATA	vend_dpd1;
+	BDATA	vend_hbeat;
+	BDATA	vend_natt_v00;
+	BDATA	vend_natt_v01;
+	BDATA	vend_natt_v02;
+	BDATA	vend_natt_v03;
+	BDATA	vend_natt_rfc;
+
+	BDATA	vend_ssoft;
 	BDATA	vend_kame;
+	BDATA	vend_unity;
+	BDATA	vend_netsc;
+	BDATA	vend_zwall;
+	BDATA	vend_swind;
+	BDATA	vend_chkpt;
 
-	long	dump_ike;			// packet dump decoded traffic
-	long	dump_pub;			// packet dump encoded traffic
+	long	dump_decrypt;		// packet dump decoded traffic
+	long	dump_encrypt;		// packet dump encoded traffic
 
-	PCAP_DUMP	pcap_ike;
-	PCAP_DUMP	pcap_pub;
-	PCAP_DUMP	pcap_frg;
-	PCAP_DUMP	pcap_prv;
-
-	// locking functions
-
-	ITH_LOCK	lock_sdb;
-	ITH_LOCK	lock_net;
-	ITH_LOCK	lock_ipq;
+	PCAP_DUMP	pcap_decrypt;
+	PCAP_DUMP	pcap_encrypt;
 
 	// xauth and xconf classes
 
@@ -420,7 +480,9 @@ typedef class _IKED
 	_IKED_XCONF_LOCAL	xconf_local;
 
 #ifdef OPT_LDAP
+
 	IKED_XAUTH_LDAP		xauth_ldap;
+
 #endif
 
 	// id name helper functions
@@ -439,6 +501,13 @@ typedef class _IKED
 	long	socket_locate( IKE_SADDR & saddr );
 	long	socket_select( unsigned long timeout );
 
+#ifdef WIN32
+
+	long	tunnel_filter_add( IDB_TUNNEL * tunnel, bool natt );
+	long	tunnel_filter_del( IDB_TUNNEL * tunnel );
+
+#endif
+
 	long	header( PACKET_IP & packet, ETH_HEADER & ethhdr );
 	long	recv_ip( PACKET_IP & packet, ETH_HEADER * ethhdr = NULL );
 	long	send_ip( PACKET_IP & packet, ETH_HEADER * ethhdr = NULL );
@@ -446,11 +515,21 @@ typedef class _IKED
 	bool	vnet_init();
 	bool	vnet_get( VNET_ADAPTER ** adapter );
 	bool	vnet_rel( VNET_ADAPTER * adapter );
-	bool	vnet_set( VNET_ADAPTER * adapter, bool enable );
-	bool	vnet_setup(	VNET_ADAPTER * adapter, IKE_XCONF & xconf );
 
+	bool	client_setup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel );
+	bool	client_cleanup( VNET_ADAPTER * adapter, IDB_TUNNEL * tunnel );
+
+#ifdef OPT_DTP
+
+	bool	dnsproxy_setup( DTPI & dtpi, IDB_TUNNEL * tunnel );
+	void	dnsproxy_cleanup( DTPI & dtpi );
+
+#endif
+
+	void	text_prot( char * text, int prot );
 	void	text_addr( char * text, in_addr & addr );
 	void	text_mask( char * text, in_addr & addr );
+	void	text_port( char * text, int port );
 	void	text_addr( char * text, sockaddr * saddr, bool port );
 	void	text_addr( char * text, IKE_SADDR * iaddr, bool port );
 	void	text_addr( char * text, PFKI_ADDR * paddr, bool port, bool netmask );
@@ -470,24 +549,22 @@ typedef class _IKED
 	bool	cert_2_bdata( BDATA & cert, X509 * x509 );
 	bool	bdata_2_cert( X509 ** x509, BDATA & cert );
 
-	void	load_path( char * file, char * fpath );
-
-	long	cert_save( BDATA & cert, char * file );
-	long	cert_load( BDATA & cert, char * file, bool ca, BDATA & pass );
+	long	cert_save( BDATA & cert, char * fpath );
+	long	cert_load( BDATA & cert, char * fpath, bool ca, BDATA & pass );
 	bool	cert_load_pem( BDATA & cert, FILE * fp, bool ca, BDATA & pass );
 	bool	cert_load_p12( BDATA & cert, FILE * fp, bool ca, BDATA & pass );
 	bool	cert_desc( BDATA & cert, BDATA & text );
 	bool	cert_subj( BDATA & cert, BDATA & subj );
 	bool	asn1_text( BDATA & asn1, BDATA & text );
 	bool	text_asn1( BDATA & text, BDATA & asn1 );
-	bool	cert_verify( BDATA & cert, BDATA & ca );
+	bool	cert_verify( IDB_LIST_CERT & certs, BDATA & ca, BDATA & cert );
 
-	long	prvkey_rsa_load( EVP_PKEY ** evp_pkey, char * file, BDATA & pass );
+	long	prvkey_rsa_load( EVP_PKEY ** evp_pkey, char * fpath, BDATA & pass );
 	bool	prvkey_rsa_load_pem( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass );
 	bool	prvkey_rsa_load_p12( EVP_PKEY ** evp_pkey, FILE * fp, BDATA & pass );
 	bool	pubkey_rsa_read( BDATA & cert, EVP_PKEY ** evp_pkey );
-	bool	prvkey_rsa_encrypt( EVP_PKEY * evp_pkey, BDATA & data );
-	bool	pubkey_rsa_decrypt( EVP_PKEY * evp_pkey, BDATA & sign );
+	bool	prvkey_rsa_encrypt( EVP_PKEY * evp_pkey, BDATA & hash, BDATA & sign );
+	bool	pubkey_rsa_decrypt( EVP_PKEY * evp_pkey, BDATA & sign, BDATA & hash );
 
 	// id helper functions
 
@@ -497,15 +574,6 @@ typedef class _IKED
 	bool	cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt );
 	bool	cmp_ph2id( IKE_PH2ID & idt, IKE_PH2ID & ids, bool exact );
 
-	// ike security db functions
-
-	bool	get_peer( bool lock, IDB_PEER ** peer, IKE_SADDR * saddr );
-	bool	get_policy( bool lock, IDB_POLICY ** policy, long dir, u_int16_t type, u_int32_t * plcyid, IKE_SADDR * src, IKE_SADDR * dst, IKE_PH2ID * ids, IKE_PH2ID * idd );
-	bool	get_tunnel( bool lock, IDB_TUNNEL ** tunnel, long * tunnelid, IKE_SADDR * saddr, bool port );
-	bool	get_phase1( bool lock, IDB_PH1 ** ph1, IDB_TUNNEL * tunnel, long state, long nostate, IKE_COOKIES * cookies );
-	bool	get_phase2( bool lock, IDB_PH2 ** ph2, IDB_TUNNEL * tunnel, long state, long nostate, u_int32_t * seqid, uint32_t * msgid, IKE_SPI * spi_l, IKE_SPI * spi_r );
-	bool	get_config( bool lock, IDB_CFG ** cfg, IDB_TUNNEL * tunnel, unsigned long msgid );
-
 	// ike packet handler functions
 
 	long	packet_ike_encap( PACKET_IKE & packet_ike, PACKET_IP & packet_ip, IKE_SADDR & src, IKE_SADDR & dst, long natt );
@@ -528,16 +596,30 @@ typedef class _IKED
 	long	process_inform_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned char payload );
 	long	process_inform_send( IDB_PH1 * ph1, IDB_XCH * inform );
 
+	// dhcp over ipsec helper functions
+
+	long	socket_dhcp_create( IDB_TUNNEL * tunnel );
+	long	socket_dhcp_remove( IDB_TUNNEL * tunnel );
+
+	long	socket_dhcp_send( IDB_TUNNEL * tunnel, PACKET & packet );
+	long	socket_dhcp_recv( IDB_TUNNEL * tunnel, PACKET & packet );
+
+	long	process_dhcp_send( IDB_TUNNEL * tunnel );
+	long	process_dhcp_recv( IDB_TUNNEL * tunnel );
+
 	// policy helper functions
 
 	bool	policy_get_addrs( PFKI_SPINFO * spinfo, IKE_SADDR & src, IKE_SADDR & dst );
 	bool	policy_cmp_prots( PFKI_SPINFO * spinfo1, PFKI_SPINFO * spinfo2 );
 
+	bool	policy_dhcp_create( IDB_TUNNEL * tunnel );
+	bool	policy_dhcp_remove( IDB_TUNNEL * tunnel );
+
 	bool	policy_list_create( IDB_TUNNEL * tunnel, bool initiator );
 	bool	policy_list_remove( IDB_TUNNEL * tunnel, bool initiator );
 
-	bool	policy_create( IDB_TUNNEL * tunnel, u_int16_t type, IKE_PH2ID & id1, IKE_PH2ID & id2 );
-	bool	policy_remove( IDB_TUNNEL * tunnel, u_int16_t type, IKE_PH2ID & id1, IKE_PH2ID & id2 );
+	bool	policy_create( IDB_TUNNEL * tunnel, u_int16_t type, IKE_PH2ID & id1, IKE_PH2ID & id2, bool route );
+	bool	policy_remove( IDB_TUNNEL * tunnel, u_int16_t type, IKE_PH2ID & id1, IKE_PH2ID & id2, bool route );
 
 	// proposal helper functions
 
@@ -555,7 +637,7 @@ typedef class _IKED
 	long	phase1_gen_hash_i( IDB_PH1 * ph1, BDATA & hash );
 	long	phase1_gen_hash_r( IDB_PH1 * ph1, BDATA & hash );
 	bool	phase1_chk_port( IDB_PH1 * ph1, IKE_SADDR * saddr_r, IKE_SADDR * saddr_l );
-	long	phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet );
+	long	phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next );
 	long	phase1_chk_vend( IDB_PH1 * ph1, BDATA & vend );
 	long	phase1_chk_hash( IDB_PH1 * ph1 );
 	long	phase1_chk_sign( IDB_PH1 * ph1 );
@@ -578,8 +660,8 @@ typedef class _IKED
 
 	// config exchange helper functions
 
-	long	config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask );
-	long	config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask );
+	long	config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VENDOPTS vendopts );
+	long	config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VENDOPTS vendopts );
 
 	long	config_chk_hash( IDB_PH1 * ph1, IDB_CFG * cfg, unsigned long msgid );
 	long	config_message_send( IDB_PH1 * ph1, IDB_CFG * cfg );
@@ -605,8 +687,8 @@ typedef class _IKED
 	long	payload_add_attr( PACKET_IKE & packet, IKE_ATTR & attrib  );
 	long	payload_get_attr( PACKET_IKE & packet, IKE_ATTR & attrib );
 
-	long	payload_add_sa( PACKET_IKE & packet, IKE_PLIST & plist, uint8_t next );
-	long	payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist );
+	long	payload_add_sa( PACKET_IKE & packet, IDB_LIST_PROPOSAL & plist, uint8_t next );
+	long	payload_get_sa( PACKET_IKE & packet, IDB_LIST_PROPOSAL & plist );
 
 	long	payload_add_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal, uint8_t next );
 	long	payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal );
@@ -664,7 +746,10 @@ typedef class _IKED
 	bool	paddr_ph2id( PFKI_ADDR & paddr, IKE_PH2ID & ph2id );
 	bool	ph2id_paddr( IKE_PH2ID & ph2id, PFKI_ADDR & paddr );
 
-	long	pfkey_recv_spinfo( PFKI_MSG & msg );
+	long	pfkey_init_phase2( bool nailed, u_int16_t plcytype, u_int32_t plcyid, u_int32_t seq );
+
+	long	pfkey_recv_spadd( PFKI_MSG & msg );
+	long	pfkey_recv_spnew( PFKI_MSG & msg );
 	long	pfkey_recv_acquire( PFKI_MSG & msg );
 	long	pfkey_recv_getspi( PFKI_MSG & msg );
 	long	pfkey_recv_flush( PFKI_MSG & msg );
@@ -677,17 +762,16 @@ typedef class _IKED
 	long	pfkey_send_spadd( PFKI_SPINFO * spinfo );
 	long	pfkey_send_spdel( PFKI_SPINFO * spinfo );
 
-	//
-	// admiministrative interface handlers
-	//
-
-	void	attach_ike_admin();
-
 	//
 	// execution thread loops
 	//
 
-	long	loop_ike_admin( IKEI * ikei );
+	void	loop_ref_inc( char * name );
+	void	loop_ref_dec( char * name );
+
+	long	loop_ipc_server();
+	long	loop_ipc_client( IKEI * ikei );
+
 	long	loop_ike_nwork();
 	long	loop_ike_pfkey();
 
