@@ -77,7 +77,7 @@
 #include <time.h>
 #include <assert.h>
 #include "export.h"
-#include "utils.list.h"
+#include "libidb.h"
 
 #define MEDIA_ETHERNET			0x0001
 
@@ -163,6 +163,7 @@
 #define BOOTP_REPLY				0x02
 
 #define BOOTP_HW_EHTERNET		0x01
+#define BOOTP_HW_IPSEC			0x1f
 
 #define DHCP_MSG_DISCOVER		0x01
 #define DHCP_MSG_OFFER			0x02
@@ -182,6 +183,7 @@
 #define DHCP_OPT_NBNS			0x2c	// netbios name server
 #define DHCP_OPT_NBNT			0x2e	// netbios node type
 #define DHCP_OPT_NBOTS			0x2f	// netbios over tcp scope
+#define DHCP_OPT_ADDRESS		0x32	// requested address
 #define DHCP_OPT_LEASE			0x33	// lease period
 #define DHCP_OPT_MSGTYPE		0x35	// message type
 #define DHCP_OPT_SERVER			0x36	// server address
@@ -340,59 +342,11 @@ struct pcap_pkthdr {
 	uint32_t	len;		// length this packet (off wire)
 };
 
-//
-// basic data class
-//
-
-#define BDATA_ALL		~0
-
-typedef class DLX _BDATA
-{
-	protected:
-
-	unsigned char *	data_buff;
-	size_t			data_size;
-	size_t			data_oset;
-
-	public:
-
-	_BDATA();
-	~_BDATA();
-
-	bool set( _BDATA & bdata );
-	bool set( int value, size_t size );
-	bool set( char * buff, size_t size );
-	bool set( void * buff, size_t size );
-
-	bool ins( _BDATA & bdata, size_t oset = 0 );
-	bool ins( int value, size_t size, size_t oset = 0 );
-	bool ins( char * buff, size_t size, size_t oset = 0 );
-	bool ins( void * buff, size_t size, size_t oset = 0 );
-
-	bool add( _BDATA & bdata );
-	bool add( int value, size_t size );
-	bool add( char * buff, size_t size );
-	bool add( void * buff, size_t size );
-
-	bool get( _BDATA & bdata, size_t size = BDATA_ALL );
-	bool get( char * buff, size_t size );
-	bool get( void * buff, size_t size );
-
-	bool dec( size_t size );
-	void del( bool null = false );
-
-	char *			text();
-	unsigned char *	buff();
-	size_t			size();
-	size_t			oset();
-
-}BDATA, *PBDATA;
-
 //
 // packet classes
 //
 
-typedef class DLX _PACKET : public _BDATA
+typedef class DLX _PACKET : public _BDATA, public IDB_ENTRY
 {
 	public:
 
@@ -437,7 +391,7 @@ typedef class DLX _PACKET_UDP : public _PACKET
 
 }PACKET_UDP;
 
-typedef struct DLX _DNS_QUERY
+typedef struct DLX _DNS_QUERY : public IDB_ENTRY
 {
 	char * name;
 	unsigned short	type;
@@ -445,7 +399,7 @@ typedef struct DLX _DNS_QUERY
 
 }DNS_QUERY;
 
-typedef struct DLX _DNS_RECORD
+typedef struct DLX _DNS_RECORD : public IDB_ENTRY
 {
 	char * name;
 	unsigned short	type;
@@ -459,10 +413,10 @@ typedef class DLX _PACKET_DNS : public _PACKET
 {
 	private:
 
-	LIST	list_ques;
-	LIST	list_answ;
-	LIST	list_ath_rr;
-	LIST	list_add_rr;
+	IDB_LIST	list_ques;
+	IDB_LIST	list_answ;
+	IDB_LIST	list_ath_rr;
+	IDB_LIST	list_add_rr;
 
 	bool	read_name( char * name, long & size );
 	bool	read_query( DNS_QUERY ** query );
@@ -490,7 +444,7 @@ typedef class DLX _PACKET_DNS : public _PACKET
 
 }PACKET_DNS;
 
-typedef class _IPFRAG_ENTRY
+typedef class _IPFRAG_ENTRY : IDB_ENTRY
 {
 	friend class _IPFRAG;
 
@@ -503,7 +457,9 @@ typedef class DLX _IPFRAG
 {
 	private:
 
-	LIST	frags;
+	IDB_LIST	used;
+	IDB_LIST	free;
+
 	time_t	lastchk;
 
 	public:
@@ -520,22 +476,18 @@ typedef class DLX _IPFRAG
 
 }IPFRAG;
 
-typedef class DLX _IPQUEUE
+typedef class DLX _IPQUEUE : private IDB_LIST
 {
-	private:
-
-	LIST	queue;
-
 	public:
 
 	_IPQUEUE();
-	~_IPQUEUE();
+	virtual ~_IPQUEUE();
 
 	bool	add( PACKET_IP & packet );
 	bool	get( PACKET_IP & packet, long index );
 
 	long	count();
-	void	flush();
+	void	clean();
 
 }IPQUEUE;
 
@@ -543,12 +495,14 @@ typedef class DLX _IPROUTE
 {
 	private:
 
-	int seq;
+	int				seq;
+	unsigned long	osver_maj;
+	unsigned long	osver_min;
 
 	public:
 
-	bool iface_2_addr( in_addr & iface, unsigned long & index );
-	bool addr_2_iface( unsigned long & index, in_addr & iface );
+	bool	iface_2_addr( in_addr & iface, unsigned long & index );
+	bool	addr_2_iface( unsigned long & index, in_addr & iface );
 
 	public:
 
@@ -563,7 +517,8 @@ typedef class DLX _IPROUTE
 	bool increment( in_addr addr, in_addr mask );
 	bool decrement( in_addr addr, in_addr mask );
 
-	bool flusharp( in_addr &iface );
+	bool islocal( in_addr & iface );
+	bool flusharp( in_addr & iface );
 
 }IPROUTE;
 
